package freebase;

import freebase.client.GuiPrompt;
import freebase.client.SqlPrompt;
import freebase.server.DatabaseNode;
import freebase.test.BootScript;
import freebase.util.MemoryDumpThread;
import freebase.util.Props;
import org.apache.log4j.PropertyConfigurator;

import java.io.File;
import java.io.IOException;
import java.util.Date;

/**
 *  Entry point into the database system.
 */
public class StartDaemon {
    /**
     * One possibility
     * x.7.y is pre-alpha
     * x.8.y is alpha
     * x.9.y is beta
     */
    final public static String FREEBASE_VERSION = "0.7.0";

    final public static String DEFAULTS_FILENAME = "config/defaults.prop";

    public static void main(String[] args) throws IOException {
        System.out.println("Welcome to Freebase version " + FREEBASE_VERSION + "   " + (new Date()));
        
          //  Load properties
        initProperties(args);

        try{
              //  hack, but something like this should evolve for debugging in the indefinite future
            if(argsContain(args, "-newdb")) {
                new File("data/logfile.hex").delete();
            }

            //DatabaseNode dbNode = new DatabaseNode(props);
            DatabaseNode dbNode = startDatabaseServer();

            //  temp code begin
            dbNode.createDefaultSchema();

            if(argsContain(args, "-newdb")) {
                dbNode.createTestSchema();
            }
            //  temp code end

            if(argsContain(args, "tracememory")) {
                System.out.println("Tracing memory to 'memorylog.csv'");
                new MemoryDumpThread("memorylog.csv").start();
            }

            /*
            if(true) { //args.length > 0 && args[0].equals("debug")) {
                DBConnection con = DatabaseNode.getLocalDBConnection("", "");
                con.executeQuery(new freebase.sql.SelectStatement());
            }

            if(true) { //args.length > 0 && args[0].equals("debug")) {
                DBConnection con = DatabaseNode.getLocalDBConnection("", "");
                con.executeQuery(new freebase.sql.SelectStatement());
            }
            */

              //  run the actual prompt
            if(args.length > 0 &&
                 (argsContain(args, "-prompt") || argsContain(args, "-double"))) {
                SqlPrompt prompt = new SqlPrompt("JJ", "blahblah");
                prompt.start();
            }

                  //  run the actual prompt
            if(args.length > 0 &&
                 (argsContain(args, "-gui") || argsContain(args, "-double"))) {
                GuiPrompt prompt = new GuiPrompt("JJ", "blahblah");
                //prompt.start();
            }

            if(argsContain(args, "-runscript")) {
                BootScript.runScript(dbNode);
            }

        }catch(Throwable t) {
            t.printStackTrace();
        }
    }

    /**
     * This is pretty trashy but will work for now
     */
    public static DatabaseNode startDatabaseServer() throws Exception {
          //  if no properties were set, set up basic local memory DB.
        if(Props.getInstance() == null) {
            Props.setInstance(new Props(DEFAULTS_FILENAME));
        }

        PropertyConfigurator.configure("config/server_log_conf.txt");
        return new DatabaseNode();
    }

    public static boolean argsContain(String[] args, String arg) {
        for(int i = 0; i < args.length; i++) {
            if(args[i].equals(arg)) {
                return true;
            }
        }

        return false;
    }

    public static String getFirstNonOption(String[] args) {
        for(int i = 0; i < args.length; i++) {
            if(! args[i].startsWith("-")) {
                return args[i];
            }
        }

        return null;
    }

    public static void initProperties(String[] commandLineArgs) throws IOException {
        Props.setInstance(new Props(DEFAULTS_FILENAME, getFirstNonOption(commandLineArgs)));
    }
}