/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 24, 2001
 * Time: 10:29:51 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.api;

import freebase.engine.Transaction;

import java.util.HashSet;

/**
 * External interface to deadlock detector.  Apis
 * are designed in this way to be compatible with distributed
 * deadlock detectors, which are asynchronous in nature.
 */
public interface DeadlockDetector {
    /**
     * Adds a waiter edge to the resource graph.  Tells the deadlock
     * detector that transaction a is now waiting for transaction b to
     * finish.  a -> b.  When a deadlock is detected, the detector will
     * call the callback explictly with an event (all locking handled externally)
     */
    void addWaitingEdge(Transaction a, Transaction b,
                        DeadlockableResource res);

    /**
     * Adds all in the set, except any occurance of a.  This last is critically
     * important!!!
     */
    void addWaitingEdges(Transaction a, HashSet set, DeadlockableResource res);

    /**
     * It is still questionable whether this design will hold up
     * under parallel updates.  A true challenge
     */
    void removeWaitingEdge(Transaction a, Transaction b,
                           DeadlockableResource res);

    /**
     * used when one thread is waiting on multiple resources, and
     * needs to remove them all
     */
    void removeAllWaitingEdges(Transaction a);
}
