/**
 * Copyright 2004 JJ Furman
 * Time: 10:24:10 AM
 */
package freebase.api;

import freebase.server.ExecutionPlan;

import java.sql.SQLException;

/**
 * Using the word Filter from java io parlance, this is essentially a base class for delegating all requests on
 * to the source.  Use it to reduce the number of lines of code in other filtering TupleIterator (nearly all of them!)
 * TODO:  apply this to existing TupleIterator implementations to reduce code
 */
public class FilterTupleIterator implements TupleIterator {
    /**
     * Allow subclasses to directly see this.
     */
    protected TupleIterator source;

    public FilterTupleIterator(TupleIterator source) {
        this.source = source;
    }

    public TupleIterator getSource() {
        return source;
    }

    /**************************************************************************
     * TupleIterator methods
     */

    /**
     * reset
     * @throws java.sql.SQLException
     */
    public void reset() throws SQLException {
        source.reset();
    }

    public boolean next() throws SQLException {
        return source.next();
    }

    public void close() throws SQLException {
        source.close();
    }

    public ExecutionPlan getExecutionPlan() throws SQLException {
        return source.getExecutionPlan();
    }

    public Comparable getValue(int i) throws SQLException {
        return source.getValue(i);
    }

    public void setValue(int i, Comparable value) throws SQLException {
        source.setValue(i, value);
    }

    public int getNumColumns() throws SQLException {
        return source.getNumColumns();
    }

    public String getColumnName(int i) throws SQLException {
        return source.getColumnName(i);
    }

    public ColumnInfo getColumnInfo(int i) throws SQLException {
        return source.getColumnInfo(i);
    }

    public boolean isUpdateable() throws SQLException {
        return source.isUpdateable();
    }

    public void deleteTuple() throws SQLException {
        source.deleteTuple();
    }
}
