/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 12, 2001
 * Time: 6:56:22 PM
 * To change template for new interface use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.api;

import freebase.engine.Transaction;
import freebase.log.LogEntry;
import freebase.log.LogEntryHeader;
import freebase.log.LogEntryManager;

import java.io.IOException;
import java.sql.SQLException;

/**
 * "What rolls down stairs
 *  alone or in pairs,
 *  and over your neighbor's dog?
 *  What's great for a snack,
 *  And fits on your back?
 *  It's log, log, log"    -BLAMMO product placement
 *  <p>
 * The LogManager is the ultimate truth in a database system.  It keeps track
 * of changes to the database, typically grouped by transactions, the unit of change on the database.<BR><BR>
 *
 * Some databases have multiple logs, but usually each transaction is in one log at a time.<BR><BR>
 *
 * TODO:  add some initialization information, ie from XML or whatever
 */
public interface LogManager {
    /**
     * writes an entry to the log.  returns the LSN of the log entry written.
     */
    long writeEntry(Transaction t, LogEntry entry, boolean flush) throws SQLException;

    /**
     * Fetch an entry from the log.  Used to traverse the logs for a transaction.
     * TODO: strongly consider passing in an empty LogEntryHeader object here to overwrite!!!!
     *   Since the log is generally scanned there is no reason to create lots of
     *   extra objects.  This redesign should be done eventually, got so save where we can!
     */
    LogEntryHeader getLogEntry(long LSN) throws SQLException;

    /**
     * Tells the log when a portion of it is not longer needed to be kept
     * online.  Since the log is logically (and in many cases physically)
     * a linear structure, this command is used to tell it when it is ok
     * to archive and pull data offline, or in the memory case when it is ok
     * to nuke some records.<BR><BR>
     *
     * The hardLSN passed in is the lowest value that absolutely must remain online
     * (a begin transaction command) due to uncommitted transactions.
     * This will be called when the lowest view number is no longer active.<BR><BR>
     *
     * The softLSN is a number, generally lower than the hardLSN that would be nice
     * to keep online for the purposes of addressing live queries, but is not absolutely
     * necessary.
     *
     * This method may be called quite frequently by the transaction manager
     * so make sure you implement it in a way that is very fast.<BR><BR>
     *
     * LSN of zero (0) means that none of the log needs to stay online.<BR><BR>
     *
     * It is not ideal to always aggresively expunge all old log entries.  Some are
     * need to reconstruct old views.  (above softLSN)
     * @param softLSN is lower than hardLSN.  We would LIKE to have softLSN but we require hardLSN,
     *          to prevent snapshot too old message
     */
    void raiseLowWaterMark(long hardLSN, long softLSN);

    //  TODO:  long ensureFlushed(long LSN);    ensure that the log is flushed up to the point specified.  Called from block writout procedures

    //  long getHighestLSN?  (Transaction Processing, pg 501)

    //  float getPercentFull...
    //  -or-  isLogSpaceCritical

    void setLogEntryManager(LogEntryManager logEntryManager) throws SQLException, IOException;
}
