/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Dec 6, 2001
 * Time: 7:56:39 PM
 * To change template for new interface use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.api;

import freebase.engine.Transaction;
import freebase.engine.TransactionManager;

import java.math.BigInteger;
import java.sql.SQLException;

/**
 * Used to initialize, and then manage storage objects during
 * the lifecycle of a database.
 */
public interface StorageManager extends LogEntryProcessor {
    //  should be able to accept log operations direct?
    void setTransactionManager(TransactionManager tm) throws SQLException;

    //  table storage: init, create (w/ storage params)
    //  TODO:  add storage parameters

    /**
     * The first table storage created must be id 0 ("TABLES" table)
     */
    TableStorage createTableStorage(Transaction t) throws SQLException;
    TableStorage getTableStorage(int tableId) throws SQLException;

    /**
     * Write a log entry and remove the reference (shouldn't be found anyway, catalog entry is removed).
     * Should wait to do cleanup until all readers are out of it.
     * @param t
     * @param tableId
     * @throws SQLException
     */
    void dropTableStorage(Transaction t, int tableId) throws SQLException;

    //  index storage: init, create (w/ storage params)
    IndexStorage createIndexStorage(Transaction t, boolean unique) throws SQLException;
    IndexStorage getIndexStorage(int indexId) throws SQLException;

    /**
     * drop index storage.  Just like dropTable.  Note, might as well pass in the id rather than
     * the object because we have to remove it by id anyway.
     * @param t
     * @param indexId
     * @throws SQLException
     */
    void dropIndexStorage(Transaction t, int indexId) throws SQLException;

    int getMaxTableId() throws SQLException;
    int getMaxIndexId() throws SQLException;
    //int getMaxSequenceId() throws SQLException;  <-- what is this for?

    SequenceStorage getSequenceStorage(int sequenceId) throws SQLException;
    SequenceStorage createSequenceStorage(Transaction t, BigInteger startValue) throws SQLException;
    void dropSequenceStorage(Transaction t, int sequenceId) throws SQLException;

    /**
     * Generic cleanup method that frees up any unnecessary structures,
     * primarily due to committed transactions  (Where is this called!?)
     */
    void cleanup() throws SQLException;

    /**
     * Write out all changes, return only when done.
     * @throws SQLException
     */
    void checkpoint() throws SQLException;

    //  sequence storage?  or just table data?
    //  log anchor storage?

    //  log access, or independent?

    // Thought:  either the catalog registers events with this or the catalog
    //    takes over accepting the log events directly upon... preferably the former
    //  therefore catalog holds recovery/normal state
}
