/*
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: Sep 18, 2001
 * Time: 11:32:54 PM
 * To change template for new interface use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.api;

import freebase.engine.Transaction;

import java.sql.SQLException;

/**
 * Interface for getting at the table data.  Is this a good name?
 * Perhaps this class should be abstract, and do all of the index stuff too
 * Implement tuple?  No, probably a byte[] type deal for abstraction from
 * implementation.  The ideal would be a
 *
 * Should have multiple modes, possibly read and read/write mode
 *
 * Probably rename this to DataAccess.  This layer does no validation at all
 */
public abstract class TableCursor {
    /**
     * Table cursor access modes  TODO: wire them in
     */
    final public static int SELECT_MODE = 0;
    final public static int UPDATE_MODE = 1;
    final public static int SELECT_FOR_UPDATE_MODE = 2;

    /**
     * This only manages the hooking.
     * TODO:  don't need to pass the transaction in to every
     */
    public TableCursor(Transaction t) {  }

    /**
     * Resets the cursor position to before the beginning
     */
    public abstract void reset() throws SQLException;

    /**
     *  Move forward one row in the table.  Using this can be a hint to lookahead bulk block reads.
     * For the first implementation of a disk based persistence this should probably release the
     * lock when moving between block boundaries.
     */
    public abstract boolean next(Transaction t) throws SQLException;

    /**
     *  get the current rownum.  Not necessarily one more that the last
     */
    public abstract long getCurrentRowNum() throws SQLException;

    /**
     *  set current rownum
     * @return false if the row is deleted (from our perspective)
     */
    public abstract boolean setCurrentRowNum(Transaction t, long row) throws SQLException;

    /**
     * The lock for the row is assumed to be already obtained.
     */
    //  Nice and easy to debug, later may convert this to a RowObject for direct comparison stuff
    //  collapse this implementation and the TableTupleIterator
    public abstract void setRowData(Transaction t, Object[] data) throws SQLException;

    /**
     * Copies the current data into the passed byte array
     */
    public abstract boolean getRowData(Transaction t, Object[] data) throws SQLException;

    //  was this:  byte[] getRowData()

    //  Transaction getLastModifier(long row)  for locks?

    /** update ->  assumed that the block is already fixed I guess
     *    1.  lock (was first lock ?  modify lockRow command)
     *    2.  do update (return the log entry from below)
     *    3.  write log entry
     *    4.  tell all appropriate indexes about the update
     */

    /**
     * Should be lockRowPending, since that is what it does
     */
    public abstract void reserveRowLock(Transaction t, boolean tableRowExclusive) throws SQLException;

    /**
     * Is only called when the lock was just for doing a quick check
     */
    public abstract void releaseRowLock() throws SQLException;

    public abstract long insertRow(Transaction t, Object[] data) throws SQLException;

    /**
     *  delete this current row.
     * The lock for the row is assumed to be already obtained.
     */
    public abstract void deleteRow(Transaction t) throws SQLException;

    /**
     * Get the SCN of the most recent writer.  Used in SERIALIZABLE and SELECT FOR UPDATE
     * modes
     */
    public abstract long getLastSCN() throws SQLException;

    /**
     * All things that extend this must call close!
     * Could enforce this by making this final and putting cursor specific close stuff in
     * a different method.
     *
     * TODO:  consider decrementing the number of outstanding uses of a table, thus allowing it to
     * get cleaned up and out if it has already been dropped
     */
    public void close() throws SQLException {
        //  TODO:  unhook our cursor from the connection (via the transaction)
    }
}
