/*
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: Sep 10, 2001
 * Time: 10:08:45 PM
 * To change template for new interface use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.api;

import java.sql.SQLException;

/**
 * represents a tuple.  Just cleaner to read this way in the other classes.  All internal offsets
 * start at zero, unlike JDBC.  I've never known why they insisted on starting at one.
 */
public interface Tuple {
    /**
     * get the ith value of the current relation.
     */
    Comparable getValue(int i) throws SQLException;

    /**
     * set the ith value of the current relation, if it is an updateable relation.
     * Not all are.  Is this too granular a method?
     */
    void setValue(int i, Comparable value) throws SQLException;

    /**
     * Get the number of columns in the current relation
     */
    int getNumColumns() throws SQLException;

    //  TODO:  getType(int i)?

    /**
     * Get the name of the given column.
     */
    String getColumnName(int i) throws SQLException;

    /**
     * Get column info.
     * TODO:  maybe add unique information, to help sorting?
     */
    ColumnInfo getColumnInfo(int i) throws SQLException;

    /**
     * Is the current row updateable.  I don't know who calls this.
     */
    boolean isUpdateable() throws SQLException;

    //  update expression.  Underlying table can use internal space as a cache for
    //  storing the indexes to update.  Of course, technically we already know the
    //  update before we fetch the table.  Perhaps we can just pretell the table
    //  access what we plan TODO.  Whatever, maybe not


    /**
     * Deletes the current tuple.
     */
    void deleteTuple() throws SQLException;
}
