/*
 * Created by IntelliJ IDEA.
 * User: user
 * Date: Mar 8, 2002
 * Time: 11:03:20 AM
 * To change template for new class use
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.catalog;

import freebase.api.ColumnInfo;
import freebase.engine.Transaction;
import freebase.sql.*;
import freebase.util.Bool;
import freebase.util.DateHelper;

import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;

/**
 * Might as well be all static, though technically might change.
 * TODO:  a good part of this could be autogenerated...  perhaps manually each time the schema changes
 */
public class DataDictionary extends CatalogHelper {
    final public static String SYS_USER = "SYS";

    /**
     * Common table columns, others may exist but not required or used yet
     */
    final public static String[] TABLES_COLUMNS = new String[] {"TABLE_ID", "TABLE_NAME", "USER_NAME"};
    final public static String[] USERS_COLUMNS = new String[] {"NAME"};
      //  TODO:  add default value (string is fine, then parsed even functional maybe)
    final public static String[] COLUMNS_COLUMNS = new String[]
        {"COLUMN_ID", "POSITION", "NAME", "TYPE", "SIZE", "PRECISION", "NULLABLE_FLAG", "TABLE_ID"};
    final public static String[] SEQUENCES_COLUMNS = new String[]
        {"SEQUENCE_ID", "NAME", "SCHEMA", "INC", "MIN_VAL", "MAX_VAL", "CACHE", "CYCLE_FLAG", "ORDER_FLAG", "FLUSH_FLAG"};

    /**
     * Temp structure for finding position in array faster
     */
    static HashMap tableToOffsetMap = new HashMap();

    Table[] sysTables = new Table[CatalogDD.NUM_CATALOG_TABLES];

    static {
        for(int i = 0; i < CatalogDD.NUM_CATALOG_TABLES; i++) {
            tableToOffsetMap.put(CatalogDD.ALL_TABLES[i], new Integer(i));
        }
    }

    /**
     * Helper method to convert the Integers.  Autoboxing is acomin'
     * @param t
     * @param tableId
     * @param posInTable
     * @param colDef
     * @throws SQLException
     */
    protected void insertIntoCOLUMNS(Transaction t, int tableId, int posInTable, ColumnDefinition colDef)
            throws SQLException {
        insertIntoCOLUMNS(t, new Integer(posInTable), new Integer(posInTable), colDef.getFieldName(),
                    new Integer(colDef.getSqlType()), new Integer(colDef.getSize()), new Integer(colDef.getScale()),
                    Bool.getBool(colDef.isNullable()), new Integer(tableId));
    }
    /*************************************************************
     *  other methods
     */

    /**
     * Return the table name of the data dictionary table
     */
    protected Table getSysTable(int offset) {
        return sysTables[offset];
    }

    /**
     * allows this sys table to be looked up by name later
     */
    protected void addSysTable(Transaction t, Table table) throws SQLException {
        registerSysTable(table);
        //insertRowIntoTABLES(t, table.getTableId(), SYS_USER, table.getName());
        insertIntoTABLES(t, new Integer(table.getTableId()), table.getName(), SYS_USER, DateHelper.now());
    }

    /**
     * Adds a local reference to this system table so that it can be found by name later.
     * Maybe eventually could just use a schema object.  This whole design seems strange.
     */
    protected void registerSysTable(Table table) {
        sysTables[getOffsetForTable(table.getName())] = table;
        //sysTables.put(table.getName(), table);
    }

    private int getOffsetForTable(String tablename) {
        Integer i = (Integer)tableToOffsetMap.get(tablename);
        if(i == null) {
            throw new NullPointerException("Failed to find table '" + tablename + "'");
        }
        return i.intValue();
    }

    /**
     * Put all registered tables into this schema.  Called after all of the tables are created,
     * and the actual schema is finally created
     */
    protected void putAllTables(Schema s) throws SQLException {
        synchronized(sysTables) {
            for(int i = 0; i < sysTables.length; i++) {
                s.addTable(sysTables[i]);
            }
        }
    }

    /************************************************************
     * Here are methods for getting CreateTableStatements for some
     * key bootstrapping tables:  TODO:   move these methods to CatalogDD!!!!!
     */
    /**
     * Constructs the first table, the TABLES table.  This must match what is found in
     * the catalog.
     *
     * CREATE TABLE TABLES(
     *   TABLE_ID INTEGER NOT NULL CONSTRAINT PK_TABLES1 PRIMARY KEY,
     *   TABLE_NAME VARCHAR2(255) NOT NULL,
     *   USER_NAME VARCHAR2(255) NOT NULL,
     *   UNIQUE_ID NUMBER,
     *   CONSTRAINT UC_TABLES1 UNIQUE(TABLE_NAME));
     *
     * TODO:  eventually this could actually pull up the catalog and find the actual specification
     * of the TABLES table
     * TODO:  have this hardcoded in CatalogDD
     */
    public CreateTableStatement getCreateTABLES() {
        CreateTableStatement createTables = new CreateTableStatement();
        createTables.setTableName(new TableRef(SYS_USER, TABLES_TABLE));

        createTables.addColumn(new ColumnDefinition("TABLE_ID", Types.INTEGER, 15, 0, false));
        createTables.addColumn(new ColumnDefinition("TABLE_NAME", Types.VARCHAR, 255, 0, false));
        createTables.addColumn(new ColumnDefinition("USER_NAME", Types.VARCHAR, 255, 0, false));
        createTables.addColumn(new ColumnDefinition("HACKDATE", Types.DATE, 255, 0, false));

        return createTables;
    }

    /**
     * Adds the correct columns to the "COLUMNS" table.  This must match what is found in
     * the catalog.
     *
     * CREATE TABLE COLUMNS(
     *   COLUMN_ID INTEGER NOT NULL CONSTRAINT PK_COLUMNS1 PRIMARY KEY,
     *   NAME VARCHAR2(255) NOT NULL,
     *   TYPE INTEGER,
     *   SIZE INTEGER,
     *   PRECISION INTEGER,
     *   NULLABLE NUMBER(1) NOT NULL,
     *   TABLE_ID INTEGER NOT NULL);
     *
     */
    public void addCOLUMNSColumns(Table columns) {
        //System.out.println("COLUMNS TABLE: " + columns);
        columns.addColumn(new ColumnInfo("COLUMN_ID", Types.INTEGER, 15, 0, false));
        columns.addColumn(new ColumnInfo("POSITION", Types.INTEGER, 255, 0, false));
        columns.addColumn(new ColumnInfo("NAME", Types.VARCHAR, 255, 0, false));
        columns.addColumn(new ColumnInfo("TYPE", Types.INTEGER, 255, 0, false));
        columns.addColumn(new ColumnInfo("SIZE", Types.INTEGER, 255, 0, false));
        columns.addColumn(new ColumnInfo("PRECISION", Types.INTEGER, 255, 0, false));
        columns.addColumn(new ColumnInfo("NULLABLE", Types.BOOLEAN, 1, 0, false));
        columns.addColumn(new ColumnInfo("TABLE_ID", Types.INTEGER, 15, 0, false));
    }

    /**
     * helper for "delete from sys.tables where user_name=? and table_name=?"
     * @param table
     * @return
     */
    public DeleteStatement getDeleteTableEntry(Table table) {
        return new DeleteStatement(TABLES_TABLE, SqlCondition.andCondition(
                SqlCondition.fieldEqualsLiteral("USER_NAME", table.getSchema()),
                SqlCondition.fieldEqualsLiteral("TABLE_NAME", table.getName())));
    }
}
