/*
 * Created by IntelliJ IDEA.
 * User: user
 * Date: Mar 9, 2002
 * Time: 12:06:05 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.catalog;

import freebase.util.ErrorCode;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Holds a set of tables for a given user.
 * TODO:  put the password here, in encrypted form I guess.  Eventually password matching should be delegated out
 */
public class Schema {
    Catalog catalog;
    String name;

    /**
     * tablesViewsAndSequences by name.
     * TODO:  check for class case exception and throw a corresponding SQL Exception
     */
    HashMap tablesViewsAndSequences = new HashMap();

    /**
     * Indexes by name.
     */
    HashMap indexes = new HashMap();


    public Schema(Catalog catalog, String user) {
        this.catalog = catalog;
        this.name = user;
    }

    public String getName() {
        return name;
    }

    public Catalog getCatalog() {
        return catalog;
    }

    public String getUser() {
        return name;
    }

    public void addIndex(Index i) throws SQLException {
        synchronized(indexes) {
            if(indexes.containsKey(i.getName())) {
                throw new SQLException("INDEX '" + i.getName() + "' already exists in schema '" + name + "'", null, ErrorCode.INDEX_DOES_NOT_EXIST);
            }
            indexes.put(i.getName(), i);
        }
    }

    public Index getIndex(String indexName) throws SQLException {
        synchronized(indexes) {
            Index i = (Index)indexes.get(indexName);
            if(i == null) {
                throw new SQLException("Index '" + name + "." + indexName + "' does not exist",  "none", ErrorCode.INDEX_DOES_NOT_EXIST);
            }
            return i;
        }
    }

    /**
     * Throws an sql exception if the table does not exist
     */
    public Table getTable(String tableName) throws SQLException {
        synchronized(tablesViewsAndSequences) {
            Table t = (Table)tablesViewsAndSequences.get(tableName);
            if(t == null) {
                throw new SQLException("Table '" + name + "." + tableName + "' does not exist",  "none", ErrorCode.TABLE_DOES_NOT_EXIST);
            }
            return t;
        }
    }

    /**
     * adds the table to this schema.  It is assumed that the caller already inserted a row into
     * the data dictionary.  For double checking, we will also make sure this is kosher.
     */
    public void addTable(Table t) throws SQLException {
          //  latch to prevent hashmap scrambling
        synchronized(tablesViewsAndSequences) {
            if(tablesViewsAndSequences.containsKey(t.getName())) {
                throw new SQLException("TABLE '" + t.getName() + "' already exists in schema '" + name + "'");
            }
            tablesViewsAndSequences.put(t.getName(), t);
        }
    }

    public void removeTable(Table tab) throws SQLException {
          //  latch to prevent hashmap scrambling while concurrent writing TODO:  (may change with schema use lock)
        synchronized(tablesViewsAndSequences) {

            tablesViewsAndSequences.remove(tab.getName());
        }
    }

    public void removeIndex(Index index) throws SQLException {
          //  latch to prevent hashmap scrambling
        synchronized(indexes) {
            indexes.remove(index.getName());
        }
    }

    /**
     * Does NOT Throw an sql exception for now!  could pass in a boolean flag for required
     * @param required pass in true if you expect this to throw an exception if it is not found
     */
    public Sequence getSequence(String seqName, boolean required) throws SQLException {
        synchronized(tablesViewsAndSequences) {
            Sequence s = (Sequence)tablesViewsAndSequences.get(seqName);
            if(required && s == null) {
                ErrorCode.throwException(ErrorCode.SEQUENCE_DOES_NOT_EXIST, seqName);
            }

            return s;
        }
    }

    /**
     * adds the table to this schema.  It is assumed that the caller already inserted a row into
     * the data dictionary.  For double checking, we will also make sure this is kosher.
     */
    public void addSequence(Sequence s) throws SQLException {
          //  latch to prevent hashmap scrambling
        synchronized(tablesViewsAndSequences) {
              //  TODO:  check that it really is a schema
            if(tablesViewsAndSequences.containsKey(s.getName())) {
                ErrorCode.throwException(ErrorCode.SEQUENCE_ALREADY_EXISTS, s.getName());
            }
            tablesViewsAndSequences.put(s.getName(), s);
        }
    }

    public void removeSequence(Sequence s) throws SQLException {
          //  latch to prevent hashmap scrambling while concurrent writing TODO:  (may change with schema use lock)
        synchronized(tablesViewsAndSequences) {
              //  TODO:  check to ensure its a sequence! (do same above too)
            tablesViewsAndSequences.remove(s.getName());
        }
    }

    /**
     * Return all table names in an arraylist
     */
    public ArrayList getAllTableNames() {
        synchronized(tablesViewsAndSequences) {
            Iterator it = tablesViewsAndSequences.keySet().iterator();

            ArrayList tableNames = new ArrayList();
            while(it.hasNext()) {
                tableNames.add(it.next());
            }

            return tableNames;
        }
    }
}
