/*
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: Sep 10, 2001
 * Time: 10:51:53 PM
 * To change template for new interface use
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.catalog;

import freebase.api.*;
import freebase.engine.TableTupleIterator;
import freebase.engine.Transaction;
import freebase.lock.UseLock;
import freebase.log.LogEntry;
import freebase.log.data.LockTableEntry;
import freebase.sql.ColumnDefinition;
import freebase.sql.CreateTableStatement;
import freebase.sql.InsertStatement;
import freebase.util.ErrorCode;
import freebase.util.SqlTypes;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * Java object representing a table.  DDL type methods to this do not
 * cause any logging operations to occur; this is the role of the catalog.  On
 * the other hand, DML operations definitely do!<BR>
 *
 * Perhaps this should be abstract too.
 */
public class Table implements LogEntryProcessor, ColumnStatisticProvider {
    //TransactionManager tm;
    Catalog catalog;

    String schema;
    String name;

      //  List of ColumnInfos
    ArrayList columns = new ArrayList();
    ArrayList columnStats = new ArrayList();

    /**
     * Map from column name to offset pos.  To use this, ensure that you have a UseLock
     */
    private HashMap columnPosMap = new HashMap();

    /****************************************************************
     * Actual stored data
     */
    TableStorage tableData;

    /**
     * List of all Indexes.  Use this as the lock.
     */
    ArrayList allIndexes = new ArrayList();

    /**
     * Very silly implementation, something eventually needs to be able to make
     * best match via cost function of various indexes.
     */
    HashMap indexesPerColumn = new HashMap();

    //  constraints, such as FK, unique, and checks

    /**
     * is grabbed to prevent someone from using the cur column definitions
     * while some other thread is changing them!!!
     *
     * NOTE:  removed this for now.  it is assumed that an exclusive lock is obtained
     * to alter a table, thus causing the threads to sync.  For reads, its a different
     * story.  A giving reader thread must somehow grab a quick lock to get a snapshot
     * of the columns in a given table... but that isn't this lock currently
     */
    //Object changeLatch = new Object();

    /**
     * Used to prevent changes to the table while another thread is using the structure.  This includes:
     *     <li>alter table</li>
     *     <li>drop table</li>
     *     <li>truncate table</li>
     *     <li>drop index</li>
     *     <li>create index (probably in shared mode w/ notification at the end)</li>
     *
     * It protects these methods (maybe more):
     *     <li>addColumn (X)</li>
           <li>getNumColumns (S)</li>
           <li>getColumnInfo (S)</li>
           <li>getColumnInfo (S)</li>
           <li>getColumnInfo (S)</li>
           <li>getColumnOffset (S)</li>
           <li>hasColumnNamed (S)</li>
     TODO:  these need to be rethought so that they are obtained within the scope of the shared lock
            ie, in the constructor of the TableTupleIterator perhaps
     */
    UseLock useLock = new UseLock();

    /**
     *  Creates the in memory structure for the table.  Currently only deals with the name
     * and the columns.  Will eventually deal will all other aspects, as which time there will
     * be a catch 22 with the startup process.  Perhaps because of this reason it will never deal
     * with any other characteristics directly.
     * TODO:  add schema here?
     */
    public Table(Catalog catalog, TableStorage storage, CreateTableStatement createState) {
        /*
        this.catalog = catalog;
        this.tableData = storage;

        this.schema = createStatement.getSchema();
        this.name = createStatement.getTableName();
        */

        this(catalog, storage, createState.getSchema(), createState.getTableName());

        for(int i = 0; i < createState.getNumColumns(); i++) {
            ColumnDefinition cDef = createState.getColumn(i);
            addColumn(new ColumnInfo(schema, name, cDef));
        }
    }

    /**
     * Currently used during boot time to initialize a table for which we don't know all of the columns yet
     */
    public Table(Catalog catalog, TableStorage storage, String schema, String tablename) {
        this.catalog = catalog;
        this.tableData = storage;
        this.schema = schema;
        this.name = tablename;
    }

    /**
     * construct the table from the statement
     *
     *
    public Table(TransactionManager tm, Catalog catalog, CreateTableStatement createStatement) {
        this.tm = tm;
        this.catalog = catalog;
        this.schema = createStatement.getSchema();
        this.name = createStatement.getTableName();

        for(int i = 0; i < createStatement.getNumColumns(); i++) {
            ColumnDefinition cDef = createStatement.getColumn(i);
            columns.add(new ColumnStorageInfo(cDef));
        }

        //  TODO:  pull the locks from the table Storage
        //locks = new MemoryLockManager(
    }*/

    /**
     * The useLock is similary to Oracle's Library Cache Pin.  It is responsible for protecting
     * the structure of a database object (or truncated stated) while the metadata is in use.
     * @return
     */
    public UseLock getUseLock() {
        return useLock;
    }

    /**
     * Safely called before the system is up (during boot) no clear idea for anytime before that.
     * Probably need to protect the columns with a synchronized body so that getSize and such
     * can be atomic.
     *
     * NOTE:  this is protected by the UseLock.  Before calling addColumn, the UseLock is obtained
     * in exclusive mode.
     * @return the position of the new column
     */
    public int addColumn(ColumnInfo columnDef) {
        columns.add(columnDef);
        columnStats.add(new ColumnStatistics(this));
        int pos = columns.size() - 1;
        columnPosMap.put(columnDef.getName(), new Integer(pos));
        return pos;
    }

    /**
     * Add a column at a specific position.
     */
    public void addColumn(ColumnInfo columnDef, int pos) {
        while(columns.size() <= pos) {
            columns.add(null);
            columnStats.add(null);
        }
        columns.set(pos, columnDef);
        columnStats.set(pos, new ColumnStatistics(this));

        columnPosMap.put(columnDef.getName(), new Integer(pos));
    }

    public String getSchema() {
        return schema;
    }

    public String getName() {
        return name;
    }

    /**
     * Return the storage. Called from TableTupleIterator
     */
    public TableStorage getStorage() {
        return tableData;
    }


    /**
     * Drops the table.  It will only delete the actual data if there are no more readers.
     * TODO:  this should get reinstated only if some of the code moves to here
     *
    public void dropTable(Transaction t) throws SQLException {
          //  drop the storage itself
        tableData.dropTable();
    }
*/

    /**
     * Always called in the scope of a shared use lock. (by TableTupleIterator).
     * TODO:  since the returned array is never written to, it could easily be created
     * once per change.  TODO:  This would save quite a few allocations
     * @return an array that should NEVER be written to!!!  may be shared
     */
    public ColumnInfo[] getColumnInfos() {
        ColumnInfo[] columnInfos = new ColumnInfo[columns.size()];
        columns.toArray(columnInfos);
        return columnInfos;
    }

    /**
     * <p>Caller must have a Use Lock to call this (Shared or Exclusive)
     * @return
     */
    public int getNumColumns() {
        return columns.size();
    }

    /**
     * Get the ColumnInfo object of column i.  Must be called while holding the Use Lock.
     */
    public ColumnInfo getColumnInfo(int i) {
        return (ColumnInfo)columns.get(i);
    }

    /**
     * Gets column info by name.  Currently does no locking itself.
     * Note:  not called from anywhere.
     * <p>Caller must have a Use Lock to call this (Shared or Exclusive)
     */
    public ColumnInfo getColumnInfo(String name) {
        return getColumnInfo(getColumnOffset(name));
    }

    /**
     * Given the column name, this returns the column offset for this column from the beginning
     * of the table, ie first one is 0.  Caller must have a Use Lock to call this (Shared or Exclusive)
     */
    public int getColumnOffset(String columnName) {
        Integer pos = (Integer)columnPosMap.get(columnName);
        if(pos == null) {
            return -1;
        }

        return pos.intValue();
    }

    /**
     * Caller must have the Use Lock.
     * TODO:  implement this faster, ie with a hashtable
     */
    public boolean hasColumnNamed(String columnName) {
        return getColumnOffset(columnName) != -1;
    }


    /*******************************************************************
     *
     */



    /**
     * NOTE:  this is not locked because callers (ie, row changes) are expected to have
     * a ROW_EXCLUSIVE_LOCK, thus incompatible with SHARE_LOCK.  Later, when creating indexes doesn't require
     * a SHARE lock, this will need to change somehow.
     * @return a list of all IndexStorage on this object.
     * Used for inserts and deletes.  Eventually think about returning
     * an Index object
     */
    public ArrayList getAllIndexes() {
        return allIndexes;
    }

    /**
     * Should probably return an empty list if there are none, but null
     * is fine.
     */
    public ArrayList getIndexesFor(String field) {
        synchronized(allIndexes) {
            return (ArrayList)indexesPerColumn.get(field);
        }
    }

    /**
     * add the given index to the table.  Called during rollback or drop index (really won't ever get called
     * except in tests)
     */
    public void addIndex(Index index) {
        String primaryField = index.getField(0);
        synchronized(allIndexes) {
            ArrayList indexes = (ArrayList)indexesPerColumn.get(primaryField);
            if(indexes == null) {
                indexes = new ArrayList();
                indexesPerColumn.put(primaryField, indexes);
            }
            indexes.add(index);

              //  also add it to allIndexes
            allIndexes.add(index);
        }
    }

    /**
     * It is assumed that the SHARE lock is already obtained on this table.
     * NOTE:  this is currently a pretty trashy implementation of this.  The job of
     * creating the index should eventually be delegated to the index itself.  This is
     * especially true because it should create the index in parallel.
     *
     * TODO:  is a use lock required?  It might be nice to pop existing plans to allow
     *   them to use the new index... so perhaps get the exclusive lock at the end.
     *   (must hold it through the commit to ensure that the structure is legal)
     * NOTE:  shared use lock already gotten to use the table columns
     */
    public Index createIndex(Transaction t, String[] fields, Index index, boolean unique)
            throws SQLException {
          //  verify columns (in case there are no rows)
        for(int i = 0; i < fields.length; i++) {
            if(getColumnOffset(fields[i]) == -1) {
                throw new SQLException("Field " + fields[i] + " does not exist!");
            }
        }

        if(unique) {
            t.clearNumUniqueContentions();
        }

        IndexCursor indexCursor = index.getIndexCursor(t, true);

          //  scan through and index all of the data.  For now uses cursor
        TableCursor cursor = getStorage().getTableCursor(t, false);
        Comparable[] rowData = new Comparable[getNumColumns()];
        while(cursor.next(t)) {
            cursor.getRowData(t, rowData);
            Object indexValue = index.extractValue(rowData);
            if(indexValue != null) {
                indexCursor.insertEntry(indexValue, cursor.getCurrentRowNum());
            }
        }
        cursor.close();
        indexCursor.close();

          //  check to see if there were any unique key violations!
        if(t.getNumUniqueContentions() != 0) {
            ErrorCode.throwException(ErrorCode.UNIQUE_KEY_VIOLATION);
        }

        //  TODO:  add it to the appropriate keys lists (or sets), for now this will do.
        addIndex(index);

          //  debugging statement
        //toWhere.dumpContents();

        return index;
    }

    /**
     * remove references to this index in all maps and lists.  The caller has at least a share lock
     * but no guarentees of more so the sync body is required (another thread may be adding an index
     * at same time)
     * @param index
     */
    public void removeIndex(Index index) {
        synchronized(allIndexes) {
            ArrayList columnList = (ArrayList)indexesPerColumn.get(index.getField(0));
            if(columnList != null) {
                columnList.remove(index);
            }

            allIndexes.remove(index);
        }
    }
    /**
     *  Removed for now.  Not clear the future relationship between cursor and TableTupleIterator.
     *
    public TableCursor getTableCursor(Transaction t, boolean readWriteMode) throws SQLException {
        return tableData.getTableCursor(t, readWriteMode);
    }
    */

    /**
     * TODO:  change the input to this to be one of: read, readforupdate, or update
     * ALSO:  pass in a nowait flag for this: can a read be nowait in distributed case?
     * TODO:  when adding views and V$Table, change this to an interface supporting common functions
     */
    public TableTupleIterator getTableTupleIterator(Transaction t, boolean readWriteMode) throws SQLException {
        if(readWriteMode) {
            lockTable(t, TableLockManager.ROW_EXCLUSIVE_LOCK, false);
        }

          //  TODO:  change the signature of
          //  TODO:  lock and pass in the column info?  deal with versioning of this somehow
        return new TableTupleIterator(t, this, readWriteMode);
    }

    /**
     * Direct use of this is for internal only!!
     */
    public TableLockManager getLockManager() throws SQLException {
        return tableData.getLockManager();
    }

    /**
     * Locks this table in the given mode for the passed transaction.  Is idempotent,
     * and will return if the lock is already owned
     */
    public void lockTable(Transaction t, byte mode, boolean nowait) throws SQLException {
        boolean obtainedNewLock = getLockManager().lockTable(t, mode, nowait);
        if(obtainedNewLock) {
            t.writeLogEntry(new LockTableEntry(getTableId(), mode));
        }
    }

    /**
     * consider:  since now this is handled internally, probably don't need this anymore, right?
     */
    public DataFormatter getRowConverter() {
        return null;
    }

    /**
     * get the id code for this one
     */
    public int getTableId() {
        return tableData.getTableId();
    }

    /**
     * insert all columns.  It MUST be right, and thus is only really called from autogenerated code
     * @param t
     * @param data
     * @throws SQLException
     */
    protected void insertRow(Transaction t, Comparable[] data) throws SQLException {
        getUseLock().lockShared();
        try{
            lockTable(t, TableLockManager.ROW_EXCLUSIVE_LOCK, false);
            _insertRow(t, data, null);
        } finally {
            getUseLock().unlockShared();  //  not perfect here, but must be somewhere
        }
    }

    /**
     * Currently NOT CALLED AT ALL.  First thing it does is grab a lock
     * on the table.  This is consistent with Oracle behavior.
     * Is a duplicate of processInsertStatement, used when there is no statement.<BR><BR>
     *
    protected void insertRow(Transaction t, String[] columns, Comparable[] data) throws SQLException {
        lockTable(t, TableLockManager.ROW_EXCLUSIVE_LOCK, false);

        Comparable[] row = null;
        //synchronized(changeLatch) {
            row = getEmptyRow();
            for(int i = 0; i < data.length; i++) {
                String col = columns[i];
                int offset = getColumnOffset(col);
                if(offset == -1) {
                    throw new SQLException("could not find column '" + col + "' in table " + getName());
                }
                //ColumnInfo colInfo = getColumnInfo(offset);
                row[offset] = data[i];
            }
        //}

        _insertRow(t, row);
    }
*/

    /**
     * insert (and check unique constraint).
     */
    private void _insertRow(Transaction t, Comparable[] row, ColumnInfo[] types) throws SQLException {
        validateRow(row, types);

          //  TODO:  could consider doing this checking at a higher level, but for now this is fine.
        t.clearNumUniqueContentions();

        TableTupleIterator tti = new TableTupleIterator(t, this, true);
        try{
            tti.insertTuple(row);
        } finally {
            tti.close();
        }

        if(t.getNumUniqueContentions() != 0) {
            ErrorCode.throwException(ErrorCode.UNIQUE_KEY_VIOLATION);
        }
    }

    /**
     * Creates an empty row with all of the "default" values, null where there is none.
     * Caller must hold a Use Lock (Shared or Exclusive) to call this.
     */
    private Comparable[] getEmptyRow() {
        Comparable[] row = new Comparable[getNumColumns()];

        for(int i = 0; i < row.length; i++) {
            ColumnInfo colInfo = getColumnInfo(i);
            row[i] = colInfo.getDefault();
        }

        return row;
    }

    /**
     * Will return normally if the row is fine, otherwise will throw an exception about the problematic columns.
     * TODO:  for sure move this into the TableTupleIterator!!!  -  on second thought, this is probably because
     * the updates are going directly to there.  Perhaps the updates and corresponding query should be done here
     * instead, making this the correct layer for doing validation.
     *
     * TODO:  add other basic constraints (range) etc
     * @param types TODO: get rid of this.  will only be null if this is an internal call, thus no errors possible
     */
    private void validateRow(Comparable[] row, ColumnInfo[] types) throws SQLException {
        StringBuffer error = null;
        for(int i = 0; i < row.length; i++) {
            ColumnInfo colInfo = getColumnInfo(i);
            if(row[i] == null && ! colInfo.isNullable()) {
                ErrorCode.throwException(ErrorCode.COLUMN_NOT_NULLABLE, colInfo.getName());
            }

              //  for now, convert the data here!
            row[i] = SqlTypes.convertValue(row[i], colInfo);
        }
    }

    /**
     *
     * @param value
     * @param type
     */
    public void validateValue(Object value, ColumnInfo type) {

    }

    /**
     * Compares the Column definitions passed in with the ones in the table to determine
     * if they are compatible.  Throws an exception if the types are no good at a high level.
     * @param types
     * @throws SQLException
     */
    private void validateTypes(ColumnInfo[] types) throws SQLException {
        for(int i = 0; i < types.length; i++) {
              //  if there is a type, check it
            if(types != null && types[i] != null) {
                if(! SqlTypes.isCompatibleType(types[i], getColumnInfo(i))) {
                    throw new SQLException("Column " + getColumnInfo(i).getName() + " can't accept value of type " + SqlTypes.getNameForType(types[i].getType()));
                }
            }
        }
    }

    /**
     *  Takes the insert statement passed and parses all of the column types etc... Then calls _insertRow
     *  Later a missing set of columns
     *   TODO:  why not move this over to StatementProcessor (QueryProcessor)?
     */
    public void processInsertStatement(Transaction t, InsertStatement st) throws SQLException {
        getUseLock().lockShared();
        ColumnInfo[] types = null;
        Comparable[] values = null;

          //  TODO:  get columns definitions faster
        try{
            lockTable(t, TableLockManager.ROW_EXCLUSIVE_LOCK, false);

            values = getEmptyRow();
            types = new ColumnInfo[values.length];

            if(st.getFields() == null) {
                if(st.getNumValues() > values.length) {
                    throw new SQLException("Too many values");
                }

                  //  one to one correspondence expected
                for(int i = 0; i < st.getNumValues(); i++) {
                    values[i] = st.getValue(i).evaluate(null, null);
                    types[i] = st.getValue(i).getColumnInfo(null, null);
                }
            } else {
                if(st.getNumFields() != st.getNumValues()) {
                    throw new SQLException("Num Fields must equal num values");
                }

                for(int i = 0; i < st.getNumFields(); i++) {
                    int offset = getColumnOffset(st.getField(i));
                    values[offset] = st.getValue(i).evaluate(null, null);
                    types[offset] = st.getValue(i).getColumnInfo(null, null);
                }
            }
        } finally {
            getUseLock().unlockShared();
        }

        validateTypes(types);
        _insertRow(t, values, types);
    }


    /**
     * process a statement of type INSERT INTO xyz SELECT
     * TODO:  eventually figure out a way to have use lock held over all planning stages so that
     *     view of tables is guarenteed to be consistent.  Do this by moving this code over to
     *      QueryProcessor
     * @param t
     * @param st
     * @param it this is closed above
     * @throws SQLException
     */
    public long processInsertIntoSelectStatement(Transaction t, InsertStatement st, TupleIterator it)
            throws SQLException {
        getUseLock().lockShared();

        TableTupleIterator tti = null;
        int[] fieldMap;
        Comparable[] values = null;
        try{
              // TODO:  can move this lock lower (and above too)
            lockTable(t, TableLockManager.ROW_EXCLUSIVE_LOCK, false);

              //  build field map...  ofset from
            fieldMap = new int[getNumColumns()];
            if(st.getFields() == null) {
                if(it.getNumColumns() > getNumColumns()) {
                    throw new SQLException("Too many values");
                }

                for(int i = 0; i < getNumColumns(); i++) {
                    fieldMap[i] = i;
                }
            } else {
                if(st.getNumFields() != it.getNumColumns()) {
                    throw new SQLException("Num Fields must equal num columns in select");
                }

                for(int i = 0; i < st.getNumFields(); i++) {
                    fieldMap[i] = getColumnOffset(st.getField(i));
                }
            }

              //  NOTE:  uses its own TableTupleIterator for performance reasons, rather than using _insertRow
            t.clearNumUniqueContentions();
            tti = new TableTupleIterator(t, this, true);

            values = getEmptyRow();
        } finally {
            getUseLock().unlockShared();
        }


        ColumnInfo[] types = new ColumnInfo[values.length];

        //  populate types, then validate
        for(int i = 0; i < it.getNumColumns(); i++) {
            types[fieldMap[i]] = it.getColumnInfo(i);
        }

        validateTypes(types);

        long count = 0;
        try{
              //  do the actual inserting
            while(it.next()) {
                for(int i = 0; i < it.getNumColumns(); i++) {
                    values[fieldMap[i]] = it.getValue(i);
                }

                validateRow(values, null);
                tti.insertTuple(values);
                count++;

                if(t.getNumUniqueContentions() != 0) {
                    ErrorCode.throwException(ErrorCode.UNIQUE_KEY_VIOLATION);
                }
            }

        } finally {
            tti.close();
        }

        return count;
    }

    /**
     * Used for cost estimation.
     * TODO:  should return a better number for this!
     * TODO:  route this down into the storage!!!  the storage should always have some kind of estimate,
     * even if a bit stale
     */
    public long getEstimatedNumRows() {
        return 1000;
    }

    public ColumnStatistics getColumnStatistics(String column) throws SQLException {
        return (ColumnStatistics)columnStats.get(getColumnOffset(column));
    }

    /*******************************************************
     * for working with log messages
     * TODO:  important:  when going to a block oriented system
     *   after undo technically the block header should have a WAL LSN entry
     *   corresponding to the rollback entry given.  I think this is why Gray
     *   suggests writing an explicit undo entry to the end of the log.  Anyway
     *   we are going to save space and try not to.  If it ends up being hard,
     *   then we learned the hard way.  Perhaps we can sneak this value into
     *   the transaction object, since it will be in a special "rolling back mode"
     */
    public void processLogEntry(int applyMode, long lsn, Transaction t, LogEntry logEntry) throws SQLException {
        System.out.println("Table.processLogEntry being called");
        tableData.processLogEntry(applyMode, lsn, t, logEntry);
    }

    public String toString() {
        return "Table: '" + schema + "." + name + "'";
    }
}
