package freebase.client;

import freebase.StartDaemon;
import freebase.api.TableLockManager;
import freebase.api.TupleIterator;
import freebase.lock.LocalDeadlockDetector;
import freebase.memorydb.MemoryLockManager;
import freebase.parser.SqlParser;
import freebase.server.ActionResult;
import freebase.server.DBConnection;
import freebase.server.DatabaseNode;
import freebase.server.ExecutionPlan;
import freebase.sql.InsertStatement;
import freebase.sql.SelectStatement;
import freebase.sql.SqlStatement;
import freebase.util.Helper;
import freebase.util.StringBufferInputStream;
import freebase.util.StringUtil;
import org.apache.log4j.Category;
import org.apache.log4j.PropertyConfigurator;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.Date;
import java.util.StringTokenizer;


/**
 * TODO:  this nasty thing needs some serious reworking!
 * TODO:  add to reading from arbitrary input sources, such
 * as files
 */
public class SqlPrompt extends Thread {
    static Category cat = Category.getInstance(SqlPrompt.class.getName());

    //  TODO:  put word text into String[], make this int indexes... and make boolean[]
    final static String EXPLAIN_SETTING = "SHOWPLAN";
    final static String TIMING_SETTING = "TIMING";
    final static String STACKTRACE_SETTING = "STACKTRACE";

    final static String[] allSettings = {EXPLAIN_SETTING,
                                    TIMING_SETTING,
                                    STACKTRACE_SETTING};

    DBConnection con;

    //  Session Variables here
    static boolean explain = false;
    static boolean timing = false;
    static boolean stacktrace = true;

    /**
     * Kept so that it can be printed later
     */
    Throwable recentException = null;

    boolean isLocal = false;

    public SqlPrompt(String user, String password) {
        try{
            con = DatabaseNode.getLocalDBConnection(user, password);
            isLocal = true;

        }catch(Throwable sql) {
            sql.printStackTrace();
            System.exit(1);
        }
    }

    public SqlPrompt(String ip, int port, String nodeName,
                     String user, String password) {
        try{
            con = DatabaseNode.getRemoteDBConnection(ip, port, nodeName,
                user, password);

            Runtime.getRuntime().addShutdownHook(new ShutdownThread());
        }catch(Throwable sql) {
            sql.printStackTrace();
            System.exit(1);
        }
    }

    public void run() {
        System.out.println();
        promptLoop();
        //oldPrompLoop();
    }

    public void oldPrompLoop() {
        SqlParser parser = new SqlParser(System.in);

        while(true) {
            System.out.println();
            if(isLocal) {
                System.out.print("SERVER> ");
            } else {
                System.out.print("SQL> ");
            }

            try{
            SqlStatement s = parser.TerminatedStatement();
            execute(s);
            }catch(Throwable e) {
                if(e instanceof freebase.parser.TokenMgrError) {
                    System.out.println(e.getMessage());
                } else{
                    e.printStackTrace();
                }

                parser.ReInit(System.in);
            }
        }
    }


    int lineCount = 0;
    StringBuffer statement;
    SqlParser parser = null;
    public void promptLoop() {
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

        while(true) {
            try{
            if(lineCount == 0) {
                statement = new StringBuffer();
                if(isLocal) {
                    System.out.print("SERVER> ");
                } else {
                    System.out.print("SQL> ");
                }
            } else {
                String lineCountString = Integer.toString(lineCount);
                for(int i = 0; i < 3 - lineCountString.length(); i++) {
                    System.out.print(' ');
                }

                System.out.print(lineCount);
                System.out.print("  ");
            }

            String line = in.readLine();

              //  check the condition that ctrl-c was pressed
            if(line == null)
                return;

            if(line.equals("")) {
                lineCount = 0;
                continue;
            }

            if(lineCount == 0 && checkLocalCommand(line)) {
                lineCount = 0;
                continue;
            }

            int semiPos = checkLine(line);
            if(lineCount != 0)
                statement.append('\n');
            statement.append(line);
            if(semiPos != -1) {
                executeStringBuffer(statement);
                lineCount = 0;
                continue;
            }

            lineCount++;
            }catch(Throwable e) {
                recentException = e;
                if(e instanceof freebase.parser.TokenMgrError) {
                    System.out.println(e.getMessage());
                } else{
                    if(stacktrace) {
                        if(e instanceof SQLException) {
                            SQLException sq = (SQLException)e;
                            if(sq.getErrorCode() != 0) {
                                System.out.println("Error: " + e.getMessage() + "  code: " + sq.getErrorCode());
                                System.out.println();
                            } else {
                                e.printStackTrace();
                            }
                        } else {
                            e.printStackTrace();
                        }
                    } else {
                        System.out.println("Error: " + e.getMessage());
                        System.out.println();
                    }
                }

                lineCount = 0;
                //  RESET
            }
        }
    }

    private void executeStringBuffer(StringBuffer sb) throws Throwable {
        if(parser == null) {
            parser = new SqlParser(new StringBufferInputStream(sb));
        } else {
            parser.ReInit(new StringBufferInputStream(sb));
        }

        SqlStatement s = parser.TerminatedStatement();
        execute(s);
    }

    /**
     * Checks the line for quotes or a semi colon.
     * Bad quotes throws an exception.  It returns the position
     * of the semicolon or -1 if there is none
     */
    private int checkLine(String line) throws SQLException {
        boolean inQuotes = false;
        for(int i = 0; i < line.length(); i++) {
            char c = line.charAt(i);

            if(c == '\'') {
                inQuotes = ! inQuotes;
            } else if(c == ';') {
                if(! inQuotes) {
                    return i;
                }
            }
        }

        if(inQuotes)
            throw new SQLException("quoted string not properly terminated");

        return -1;
    }

    /**
     * Returns true if the command was a local one.
     */
    private boolean checkLocalCommand(String line) throws SQLException {
        StringTokenizer st = new StringTokenizer(line);
        String first = st.nextToken();

          //  drop the ; at the end if necessary
        if(first.endsWith(";")) {
            first = first.substring(0, first.length() - 1);
        }

        if(first.equalsIgnoreCase("SET")) {
            return executeSetCommand(extractCommand(first, st));
        } else if(first.startsWith("@")) {
            executeImportCommand(extractCommand(first, st));
            return true;
        } else if(first.equalsIgnoreCase("help")) {
            executeHelpCommand(extractCommand(first, st));
            return true;
        } else if(first.equalsIgnoreCase("gc")) {
            executeGC(extractCommand(first, st));
            return true;
        } else if(first.equalsIgnoreCase("exit") || first.equalsIgnoreCase("quit")) {
            executeExit(extractCommand(first, st));
            return true;
        } else if(first.equalsIgnoreCase("system")) {
            executeSystem(extractCommand(first, st));
            return true;
        }

        return false;
    }

    private String[] extractCommand(String first, StringTokenizer st) {
        String[] buffer = new String[st.countTokens() + 1];
        buffer[0] = first;
        for(int i = 1; i < buffer.length; i++) {
            buffer[i] = st.nextToken();
        }

        String last = buffer[buffer.length - 1];
        if(last.charAt(last.length() - 1) == ';') {
            buffer[buffer.length - 1] = last.substring(0, last.length() - 1);
        }

        return buffer;
    }

    /*******************************************************
     * local commands are here
     */

    /**
     * To remove eventually.
     * TODO:  move this code elsewhere eventually
     */
    private void executeSystem(String[] command) throws SQLException {
        if(! isLocal)
            throw new SQLException("SYSTEM command only legal inside server prompt");

        DatabaseNode localNode = DatabaseNode.getLocalNode();
        if(localNode == null) {
            throw new SQLException("no local node!");
        }

        String subCommand = command[1];
        if(subCommand.equalsIgnoreCase("dumplocks")) {
            TableLockManager tlm = localNode.getCatalog().getTable(command[2], command[3]).getLockManager();
            ((MemoryLockManager)tlm).dumpLocks();
        } else if(subCommand.equalsIgnoreCase("ddstate")) {
            ((LocalDeadlockDetector)localNode.getTransactionManager().getDeadlockDetector()).dumpDetectorState();
        } else {
            System.out.println("Illegal system command: " +StringUtil.dump(command));
        }
    }

    private void executeExit(String[] command) throws SQLException {
        close(true);
        System.exit(0);
    }

    private void executeGC(String[] command) throws SQLException {
        Runtime r = Runtime.getRuntime();
        System.out.println("executing local gc...");
        System.out.println("Before: " + ((r.totalMemory() - r.freeMemory())/1024) + "K");
        System.gc();
        System.out.println("After: " + ((r.totalMemory() - r.freeMemory())/1024) + "K");
    }

    /**
     * Executes the set command, passed in is the rest of the command
     * in the tokenizer
     */
    private boolean executeSetCommand(String[] command) throws SQLException {
        String attribute = command[1];
        if(attribute.equalsIgnoreCase("TRANSACTION")) {
            return false;
        }

        if(command.length > 3) {
            throw new SQLException(command[3] + " not an option for SET");
        }
        
        String value = command[2];

        if(attribute.equalsIgnoreCase(EXPLAIN_SETTING)) {
            explain = getBooleanValue(value);
        } else if(attribute.equalsIgnoreCase(TIMING_SETTING)) {
            timing = getBooleanValue(value);
        } else if(attribute.equalsIgnoreCase(STACKTRACE_SETTING)) {
            stacktrace = getBooleanValue(value);
        } else {
            System.out.println("Error:  Session variable '" + value.toUpperCase() + "' not recognized");
        }

        return true;
    }

    /**
     * ON, TRUE -> true<BR>
     * OFF, FALSE -> false
     */
    private boolean getBooleanValue(String val) throws SQLException {
        if(val.equalsIgnoreCase("ON") || val.equalsIgnoreCase("TRUE")) {
            return true;
        }

        if(val.equalsIgnoreCase("OFF") || val.equalsIgnoreCase("FALSE")) {
            return false;
        }

        throw new SQLException("Legal values are ON, OFF, TRUE, FALSE");
    }

    /**
     * Will execute an import
     */
    private void executeImportCommand(String[] command) {
        System.out.println("File importing is not implemented yet, but is ready to go!");
    }

    private void executeHelpCommand(String[] command) {
        System.out.println("Legal commands:  select, insert, update, set, exit");
    }

    /*******************************************************
     * Exection related methods are here
     */

    /**
     * For each kind of statement, do the right thing
     */
    public void execute(SqlStatement s) throws Throwable {
        long startTime = System.currentTimeMillis();
        //System.out.println("executing " + s);

        //  TODO:  first detect if it is a local prompt command like perhaps a formatting command

        if(s == null)
            return;

        if(s instanceof SelectStatement) {
            TupleIterator set = con.executeQuery((SelectStatement)s);
            printResults(set);
        } else {
            ActionResult result = con.executeAction(s);
            if(result != null) {
                if(s instanceof InsertStatement) {
                    System.out.println(result.getNumRowsEffected() + " inserted.");
                } else {
                    System.out.println(result.getNumRowsEffected() + " effected.");
                }
            } else {
                System.out.println("ok");
            }
        }
        System.out.println();

        if(timing)
            Helper.printElapsed(System.currentTimeMillis() - startTime);

        if(cat.isInfoEnabled()) {
            cat.info("TOOK: " + (System.currentTimeMillis() - startTime));
        }
    }

    /**
     * Handy method for printing the results of a query
     */
    public static void printResults(TupleIterator set) throws SQLException {
          //  display all rows
        int count = 0;
        while(set.next()) {
            if(count == 0) {
                System.out.println();
                for(int i = 0; i < set.getNumColumns(); i++) {
                    System.out.print(set.getColumnName(i) + "   ");
                }
                System.out.println();
                System.out.println("---------------");
            }

            for(int i = 0; i < set.getNumColumns(); i++) {
                System.out.print(set.getValue(i) + "  ");
            }
            System.out.println();

            count++;
        }

        System.out.println();
        System.out.println(count + " rows displayed.");

        if(explain) {
            System.out.println();
            System.out.println("Plan is:");
            ExecutionPlan plan = set.getExecutionPlan();
            plan.dump(System.out);
        }

        set.close();
    }

    /**
     * Closes this client down.
     */
    public void close(boolean commit) {
        if(con != null) {
            try{
            con.close(commit);
            con = null;
            }catch(Exception e) {
                System.out.print("Failed to close: " + e.getMessage() + " note: does this message matter?");
                //e.printStackTrace();
            }
        }
    }

    /**
     * registered as a shutdown hook, is called to cleanup upon exiting
     */
    class ShutdownThread extends Thread {
        public void run() {
            close(false);
        }
    }

    /**
     * This is a temporary method where I can put default state initialization
     * information.
     */
    public void debugInitialize() throws SQLException, RemoteException {

        /*
        long time = System.currentTimeMillis();
        for(int i = 0; i < 100000; i++) {
            InsertStatement ins = new InsertStatement("TEST",
                new Object[] {"" + i, "'BOB" + i + "'"});
            con.executeAction(ins);
        }
        */
        //System.out.println("TOOK: " + (System.currentTimeMillis() - time));

        /*
        DBConnection con2 = DatabaseNode.getLocalDBConnection("jj", "jj");
        con2.executeAction(new InsertStatement("test", new Object[] {"1", "'bob'"}));


        System.out.println("from t0 (should be empty)");
        printResults(con.executeQuery(new SelectStatement("test")));

        System.out.println("from t1 (should have one)");
        printResults(con2.executeQuery(new SelectStatement("test")));
        con2.executeAction(new CommitStatement());

        System.out.println("from t0 (should have one)");
        printResults(con.executeQuery(new SelectStatement("test")));
        */

    }


    /*********************************************************************
     * Main, spawns a new connection
     */
    public static void main(String[] args) {
        PropertyConfigurator.configure("config/client_log_conf.txt");

        System.out.println("Welcome to F3 Client version " + StartDaemon.FREEBASE_VERSION + "   " + (new Date()));

        String host = "localhost";
        if(args.length > 0) {
            host = args[0];
        }

        SqlPrompt prompt = new SqlPrompt(host, 0, "TEST",
                     "JJ", "PASSWORD");
        prompt.start();
    }
}
