package freebase.engine;

import freebase.util.LongHashMap;

import java.util.HashMap;


/**
 * OLDCLASS
 *  Manages table locks, (actually hopefully all locks).
 * This class works as is, but will require being moved, and resub classes here.<BR><BR>
 *
 * Q:  when a transaction that owned a bunch of locks is long gone, does it need to
 * explicitly remove locks that it owned or should it just wait for other transactions
 * to find and clean out the locks.  My sense is that it should immediately clean up?
 * consider the implication for both main implementations of this class, the block lock
 * and the memory lock
 *
 * It might be cleaner to have one TableLocks object for each table.  This would
 * still be the main lock manager though.
 *
 * When it comes time to subclass this with the block holder lock manager,
 * consider if it is better to aggresively preload locks or only determine
 * their state when required.  Consider the ability to put a flag in the
 * block to determine if the locks have been loaded or not, so all can be done
 * at once.  On another note, how can we be sure the block is fixed?
 *
 * @deprecated
 */
public class LockManager {
    /**
     * map from tablename to LongHashMap for row locks.
     * Could definitely pre-create all of them or something to
     * make it faster.
     */
    HashMap rowLocksByTable = new HashMap();

    //LongHashMap locksByRecId = new LongHashMap();


    /**
     * TODO:  have a map that allows quick removal of transactions
     * In the real db, buffers going out will do this fine.
     */
    //  HashMap transactionToTableRows

    /**
     *  definitely trick wrt getting the locks from the replica when we start up.  Perhaps
     *  via the log?  should all locks be interpreted from the log?
     */
    public LockManager() {
    }
    
    
    public void lockTable(Transaction t, String table, int mode) {
        
    }

    /**
     * Should only be called when a small rollback is happening.  On Commit,
     * call unlockAll(Transaction t), or likewise on a complete rollback.
     *
     * When releasing locks, the simplest design is to notify on the transactions
     * finish lock.  The alternative is to look and see what transactions are exactly
     * waiting for and see if this matches.  On a complete commit or rollback, just
     * loop through all.  This is more concurrent.
     */
    public void unlockRow(Transaction t, String table, long rowId) {

    }

    /**
     * actually, this probably will be the final implementation, though it is
     * called just in case it is useful later.
     */
    public void unlockAll(Transaction t) {
        //  no need to actually do anything since t.isActive() is false anyway, right!?
        //  this may wake up a cleaner thread at some point or something,
        //  perhaps the memory version will be more aggressive about cleanup
        //  but I don't plan on implementing any maps from t -> locks.  Too many!

        //  on the other hand, if there are explicit waits on rowLock objects (not
        //  existing yet) then this would be a place to call notify on them directly.
        //  that implies a simple map of t -> locks being waited on.
        //  even if this class goes to be a single table only (for faster
        //  synchronization) then the map should go on the overal lock manager, as
        //  should this method.
    }

    /**
     * Obtain write lock on the row.  'table' must be all uppercase.
     *
     * Note the simplest design is to explicitly lock the row with a log entry.  That
     * way when the log entry is un-done then unlockRow is called.  Alternatively, the
     * row lock could hold a bit as to whether this is the first time lockRow has been
     * called.  The data updater can then either query or this method can be modified
     * to return boolean as to if this is the first time.  This flag is marked in the
     * row modification log entry, so that when undoing locks can be properly released.
     *
     * And then, consider the implications of this pattern for the table locks.
     *
     * TODO:  should throw deadlock exception
     */
    public void lockRow(Transaction t, String table, long rowId) {
        LongHashMap rowLocks = null;

          //  probably unnecessarily slow, refactor eventually
          //  only because this object holds locks for all tables.
        synchronized(rowLocksByTable) {
            rowLocks = (LongHashMap)rowLocksByTable.get(table);
            if(rowLocks == null) {
                rowLocks = new LongHashMap();
                rowLocksByTable.put(table, rowLocks);
            }
        }

        while(true) {
            Transaction lockHolder = null;

            synchronized(this) {
                lockHolder = (Transaction)rowLocks.get(rowId);

                //  TODO:  should probably be transactionEpoc =

                //  row already part of the transaction
                if(lockHolder == t)
                    return;

                //  make it so TODO:  see the commented stuff
                if(lockHolder == null //  || ! lockHolder.isActive()  or  ! transactionEpoc.isActive()
                        ) {
                    rowLocks.put(rowId, t);
                    return;
                }
            }

            //  TODO:  instead of waiting on the transaction
            //     could instead explictly wait on the lock...
            //     ie, lease a RowLock object that contains the owning transaction
            //     and possibly other information.  lock on that!
            //
            //     OR better, only put in the RowLock object when necessary
            //     normally leaving just the transaction sitting in the hashtable

             //  the following doesn't work anymore.
            //  t.waitForTransactionCompletion(lockHolder);

            //  test for transaction in deadlock state?
        }
    }
}