package freebase.engine;


import freebase.log.LogEntry;
import freebase.log.LogEntryHeader;
import freebase.store.Entity;


/**
 *  The Logical database log.  Uses multiple Entities to manage
 *  online log information.  When something goes offline tell
 *  the log and it will 
 *
 * The logical log should handle all transaction issues, checkpointing.
 * Not sure yet how to manage the low-water mark (oldest begin transaction)
 * and the end of the log.  Somehow they need to be compared so that the
 * potential for a rollback with not enough information is lessened.
 * The system can immediately fail the transaction, thus applying all entries
 *
 *  Should be moved to PhysicalLog:
 *  This log currently uses a modification to the Ping Pong
 *  algorithm discussed in "Transaction Processing".  When syncing
 *  to the log it writes the last page in the two subsequent blocks.
 *  this way if a crash occurs half way between the block write the old
 *  data will at least be in the other block.
 *
 *  maybe it might be faster to just implement the ping pong algorithm but
 *  this should be quite good for now.
 *
 *  the log manages it's own buffer writeouts
 *
 * @deprecated
 */
public class OldLog {
    Object logLock = new Object();
    Object flushLock = new Object();
    
    long lastFlushedLSN = 0;
    long lastLSN;
    
    //  we should have other log entities here
    
    /**
     *  initially the files must all be the same size for ez math.  Later no.
     */
    public OldLog(Entity[] files, long beginningLSN, long lastCheckpoint, int blocksize) {
        //  ok what TODO with all these
        //  walk forward until we find the end of the log
        //  mark it!  assure that we are very aggressive about preloading.
        
        //  TODO:  calculate usable block space from block size - header etc!!!!! :)
    }
    
    
    /**
     *  writes a log entry.  Returns the LSN (long position in virtual log file)
     *  that represents this entry
     */
    public long writeEntry(Transaction t, LogEntry entry) {
        synchronized(logLock) {
            //  do!
            return lastLSN;
        }
    }
    
    /**
     *  read the header info for the log entry and put it into the variable
     *  entry.  From here the caller can use it to get the actual data
     *
     *  TODO:  analyze this method to determine if there is too much fixing
     *      and unfixing while reading the log.  Do this by running
     *      read concurrent transactions on a written record.
     *      Consider fix and hold strategy.  as long as we always fix forward!
     *
     *  @return true if the lsn is a valid log entry
     */
    public boolean readEntry(long lsn, LogEntryHeader entry) {
        return true;
    }
    
    /**
     *  read from the data portion of the log entry marked by lsn
     *  @return amount read
     */
    public int readData(long lsn, int offset, byte[] data) {
        return -1;
    }
    
    /**  this is just a ditty but could be used by the
     *  system when a read query got bounced to the secondary
     *  but the logs are not quite up to date yet.  In reality
     *  shouldn't be too long!
     */  
    public void assureLSN(long lsn) {
    }
    
    public long getMaxLSN() {
        //  always lock to copy long types
        synchronized(logLock) {
            return lastLSN;
        }
    }
    
    /**
     *  assures that the log is flushed up to the point to assure that the lsn given is written
     *  to disk
     */
    public void flush(long lsn, int maxWait) {
        long lsnFlushing = lsn;
        
        synchronized(logLock) {
            //  no worries
            if(lsn <= lastFlushedLSN)
                return;
            
            //  currently ignore maxWait
            if(maxWait != 0) {
                //  technically we should wait
                //  do wait here
                
                //  record where we are actually flushing to
                lsnFlushing = lastLSN;
            }
            
            //  copy the last block in the one after
            //  write out the group (groups if are more than one files touched)
            //  now the blocks should be marked clean and no longer
        }
        
        //  we should not prevent other threads from further writing to the log
        //  so we let go of the lock.  The buffers have already been copied down, maybe even written
        
        
        //  flush the file
        //    TODO:  MUST BE WORKED OUT!!!
        //    they must be one after the other if there are more than one file so that
        //    if a weirdness occurs that causes only the forward section to be written
        //    and if later when eventually reaching that section the system crashes, it
        //    does think that the data is valid.  This assumption needs be thought though 
        //    a little more but at any rate, it is not good the way it is.  Think in terms
        //    of multiple failures
        
        //  about the above:  if a verion boot (birthday like) is used on each bootup and
        //  written with each log block then log blocks can be checked before being used
        //  looks like a good solution
        
        synchronized(logLock) {
            //  did we do the highest flush?
            if(lsnFlushing > lastFlushedLSN)
                lastFlushedLSN = lsnFlushing;
        }
    }
}
