package freebase.engine;

import freebase.store.Block;
import freebase.store.BufferPool;
import freebase.store.TablePageInterface;

import java.io.IOException;
import java.sql.SQLException;

/**
 * OLD CLASS
 *  Represents an object that allows you to get row data
 *  as well as helpers for scanning.  It begins just before the first row
 *  so cursor.next() is required to jumpstart the process
 *
 *  Should the cursor already have the RowInterpreter?
 *  Should it publish the byte arrays out or what?
 *
 *  Current version holds the lock between calls.  This may not be the final implementation
 *
 * OLD CLASS
 */
public class OldTableCursor {
    BufferPool pool;
    TablePageInterface pageInterface;
    
    boolean lockExclusive;
    int entityId;
    long scanFSN;
    
    int curBlock = -1;
    int curRow = -1;
    
      //  current block info
    Block block = null;
    int numRows = 0;
    
    protected OldTableCursor() {
        
    }
    
    /**
     *  This is working under the assumption that
     *  the OldTableCursor objects will be recycled
     */
    protected void resetCursor(RowFormat row, RowInterface rowInterface, int entityId, BufferPool bp, boolean getLocksExclusive) {
        this.entityId = entityId;
        this.pool = bp;
        
        pageInterface = pool.getTablePageInterface(entityId);
        lockExclusive = getLocksExclusive;
    }
    
    /**
     *  releases resources still held by the table cursor.  Starting to like the name
     * "dispose" more, as if you definitely do not plan to use this anymore
     */
    protected void deCommission() {
        if(block != null) {
            if(lockExclusive)
                block.accessLock.releaseLockExclusive();
            else
                block.accessLock.releaseLockShared();
            
            pool.unfix(block);
        }
    }
    
    //  Should use a RowInterface as an adapter
    
    //  does the actual work of performing the next
    //  TODO:  think about this when row migration is active
    //     updating indexes of changing rows is where it gets tricky
    public long next() {
        curRow++;
        
        if(curRow == numRows) {
            curBlock++;
            if(block != null) {
                
                if(lockExclusive)
                    block.accessLock.releaseLockExclusive();
                else
                    block.accessLock.releaseLockShared();
                
                pool.unfix(block);
            }
            
              //  reserve the block
            try{
            System.out.println("fixing " + curBlock);
            block = pool.fix(entityId, curBlock);
            }catch(IOException ioe) {
                //  OH SHIT!  need to figure out what TODO here.
                //  probably try and grab the block from the replica pool
                //  directly.  If that fails, fatal error
                
                //  could be that the disk is out of space :
                //     catch this as a separate exception though
                ioe.printStackTrace();
            }

            if(lockExclusive)
                block.accessLock.getLockExclusive();
            else
                block.accessLock.getLockShared();


            if(! pageInterface.isUsedBlock(block))
                return -1;
            
            numRows = pageInterface.getNumRows(block);
        }
        
        long row = curBlock;
        return row << 8 + curRow;
        //  see if the next row is in the same block... If not
        //  unfix the old one, fetch the new one and start walkin
    }
    
    public void setCursorPos(long rowId) {
        //  unfix the old block
        //  grab the new one
    }
    
    /**
     *  could throw View too old exception
     */
    public Object getValue(int colId) throws SQLException {
        return null;
    }
    
    /**
     *  could throw not serializable update exception
     */
    public void setValue(int colId, Object val) throws SQLException {
        
        //  may not need to actually write all of the data to the blocks
        //  until the next call is issued.  On the other hand, what
        //  difference does it make
    }

    /**
     *  scans forward from the current position until it finds a new spot to
     *  put the row.
     */
    public long newRow() {return -1;}
    
    //  THINK:  how is the caller going to know
    //  when a row was migrated due to growing too large?
    //  is it possible?  does it matter
    //  perhaps a call to didMigrate()


    //  allows the object to release locks held
    //  while the caller does not need them
    public void passivate() {}
    
    //  tells the object to reclaim resources
    public void activate() {}
}

/*
RowInterpreter:
read([0,2,4] -> [] @ blah blah)

yes, a direct transfer function to temp space will be nice

Also, keep in mind that before using a table cursor all desired fields are know
some guarenteed and some optional so they can be passed in initially
It might be helpful to tell the cursor the usage pattern you will be using so that it
can optimize some operations
*/