package freebase.engine;

import freebase.util.ColumnStorageInfo;
import freebase.util.OhShitException;
import freebase.util.SqlTypes;

import java.sql.Types;
import java.util.ArrayList;

/**
 *  holds information about a row format
 */
public class RowFormat {
    /**
     *  currently this is arbitrary.  Get rid of it sometime to see what
     *  assumptions are being made in the code
     */
    final public static int MAX_NUM_COLUMNS = 255;
    
    ArrayList columns = new ArrayList();
    
    
    public RowFormat() {
    }
    
    public int getNumColumns() {
        return columns.size();
    }
    
    /*  TODO:
    public void addField(int type, int size, boolean nullable);
    public void addField(int type, boolean nullable);
    */
    
    public void addColumn(int type, int size, int decimal, boolean nullable) {
        ColumnStorageInfo info = new ColumnStorageInfo(null, type, size, decimal, nullable);
        columns.add(info);
    }
    
    public ColumnStorageInfo getColumn(int num) {
        return (ColumnStorageInfo)columns.get(num);
    }
    
    public void removeColumn(int num) {
        columns.remove(num);
    }
    
    public int lookupField(String field) {
        for(int i = 0; i < columns.size(); i++) {
            ColumnStorageInfo fsi = getColumn(i);

            if(fsi.getName().equals(field))
                return i;
        }

        return -1;
    }
    
    public boolean isVariableWidth(int col) {
        return getColumnWidth(col) < 0;
    }
    
    public boolean isNullable(int col) {
        return ((ColumnStorageInfo)columns.get(col)).isNullable();
    }
    
    /**
     *  @return the exact binary width of the column in bytes
     *  or -1 for a variable width col
     */
    public int getColumnWidth(int col) {
        ColumnStorageInfo colInfo = (ColumnStorageInfo)columns.get(col);
        
        
        switch(colInfo.getType()) {
            case Types.CHAR:          return colInfo.getSize();
            case Types.NUMERIC:       // TODO:  work out the storage format of these guys
            case Types.DECIMAL:       return -2;

            
            case Types.VARCHAR:       
            case Types.LONGVARCHAR:   return -1;
            case Types.BINARY:        return colInfo.getSize();
            case Types.VARBINARY:     
            case Types.LONGVARBINARY: return -1;
         
            case Types.NULL:
                throw new OhShitException("col " + col + "  is type:NULL.  What is that?");
            default:
                return SqlTypes.getWidth(colInfo.getType());
                
        }
    }
    
    /**
     *  returns the number of bytes that designate how large the variable size field is.
     *  sort of like the variable size variable size marker
     */
    public byte getVariableColumnSizeFieldSize(int col) {
        ColumnStorageInfo colInfo = (ColumnStorageInfo)columns.get(col);
        
        return getVariableColumnSizeFieldSize(colInfo.getType());
    }
}
