package freebase.engine;

import freebase.util.DataConverter;
import freebase.util.OhShitException;


/**
 * OLDCLASS
 *  Row interface must be able to handle a couple of important cases:
 *  1.  handle new object creation code from object[]
 *  2.  handle deferring processing multiple byte[] that
 *         make up a single row until the data is actually requested
 *  3.  Handle multiple updates that do not actually take effect on the
 *         underlying system until the row is no longer needed.  Perhaps a
 *         save method will need to be written for when the OldTableCursor is
 *         not in scan mode.  Or even when it is in scan mode but the choice
 *         was made to migrate the row.
 *  4.  Handle dealing with multiple byte[] for chaining, as well as when
 *         saving the row tell the system to try and find new space for it.
 *
 *  The RowInterface class is separate from the OldTableCursor class but they
 *  are very intertwined.  This may be an interface.
 *
 *  :( Currently does not support row chaining, only possibly row migration
 */
public class RowInterface {
    /**
     *  a placeholder for a true "null" value, as opposed to a "not here"
     *  value.  Of course, in this case NULL_VALUE.equals(null) is false :)
     */
    final public static Object NULL_VALUE = new Object();
    
    //  must reset each row
    int[]   fieldOffset = new int[RowFormat.MAX_NUM_COLUMNS];
    int[] fieldLen      = new int[RowFormat.MAX_NUM_COLUMNS];  //  REALLY NEEDED?
    Object[] newValues = new Object[RowFormat.MAX_NUM_COLUMNS];
    
    byte[] dataSrc;
    int offset;
    int len;
    
    //  format specific 
    RowFormat format;
    int firstVariableCol;
    
    /**
     *  tells the row interface that it is 
     */
    public RowInterface() {
    }
    
    /**
     *  tells this RowInterface to start using the given
     *  format to interpret data
     */
    public void setFormat(RowFormat format) {
        this.format = format;
        
          //  do best to optimize for constant size fields in the front
        int totalOffset = 0;
        int i = 0;
        for(i = 0; i < format.getNumColumns(); i++) {
            fieldOffset[i] = totalOffset;
            fieldLen[i] = (short)format.getColumnWidth(i);
            if(fieldLen[i] == -1) {
                break;  //  not enuf info to go further
            }
            
            totalOffset += fieldLen[i];
        }
        
        firstVariableCol = i;
    }
    
    public void deCommission() {
        //  do nothing for now
    }
    
    //  this gives the row knowledge that it is working on a
    //  new row and the initial data chunk it has available.
    //  
    //  one possiblity is that it parses the whole thing
    //  and returns the number of the highest complete
    //  col number that it can currently return without
    //  needing more data.  This may be expensive though, but
    //  probably fine considering.  May want to separate the 
    //  newRow message with the actual data content message
    public int reset(byte[] data, int offset, int len, boolean hasMoreSegments) { 
        if(hasMoreSegments)
            System.out.println("WARNING:  we don't support chaining yet!!!");
            
          //  record row info
        this.dataSrc = data;
        this.offset = offset;
        this.len = len;
        
          //  clear old values pending
        for(int i = 0; i < format.getNumColumns(); i++) {
            newValues[i] = null;  //  unchanged
        }
        
        //  note, length of last field is not redundant because of possible trailing null values
        
          // by definition, this has already been set correctly
        int totalOffset = fieldOffset[firstVariableCol];
        
          //  parse the positions of all of the fields
        for(int i = firstVariableCol; i < format.getNumColumns(); i++) {
            fieldOffset[firstVariableCol] = totalOffset;
            
            //  TODO:  deal with end nulls here
            
            if(format.isVariableWidth(i)) {
                
                byte lenFieldSize = format.getVariableColumnSizeFieldSize(i);
                
                int colLen = 0;
                switch(lenFieldSize) {
                    case 1:  colLen = dataSrc[totalOffset];     break;
                    case 2:  colLen = DataConverter.readShort(dataSrc, totalOffset);  break;
                    case 4:  colLen = DataConverter.readInt(dataSrc, totalOffset);    break;
                    default:
                        throw new OhShitException("Currently do not support parsing size: " + lenFieldSize);
                }
                
                //  check for null value, represented by -1 length.  
                if(colLen == -1) {
                    fieldLen[i] = lenFieldSize;
                    if(! format.isNullable(i))
                        System.out.println("WARNING:  found a null value in a column that shouldn't be null");
                } else {
                    //  else treat the number (negatives masked to +) as len
                    if(colLen < 0) {
                        switch(lenFieldSize) {
                            case 1:  colLen = colLen & 0x000000ff;  break;
                            case 2:  colLen = colLen & 0x0000ffff;  break;
                            case 4:
                                throw new OhShitException("2 gig limit on fields enforced");
                        }
                    }
                    
                    fieldLen[i] = lenFieldSize + colLen;
                }
            } else { //  is a fixed width column
                //  we should not have to deal with nulls here. 
                //  if so, need to figure out what is going on fast!!!
                
                fieldLen[i] = format.getColumnWidth(i);
            }
            
            totalOffset += fieldLen[i];
        }
        
        return format.getNumColumns();  //  means nothing for now
    }
    
    /**
     *  save any changes out to the same segment (only one supported now)
     */
    public void save() {
        save(dataSrc, offset);
    }
    
    /**
     *  save the row to a different spot.
     *  the length will be the amount that was specified.  It is assumed that
     *  the space has been properly prepped to write to, ie all other rows have
     *  been moved and such
     */
    public void save(byte[] seg, int segOffset) {
        //  ?
    }
    
    public Object getValue(int col) {
        //  ah finally
        return null;
    }

    /**
     *  @return the difference in size
     *  0 if the same, negative if shrank, positive if grew
     */
    public int setValue(int col, Object val) {
        return -1;
    }
}
