/**
 * Copyright 2004 JJ Furman
 */
package freebase.engine;

import freebase.util.StringUtil;

import java.util.ArrayList;

/**
 * Encapsulates the list of scan regions an index scan will go through.  This set is kept in sorted order,
 * and contains handy methods for keeping a minimal set.
 *
 * TODO: could explicitly support the [1,2] vs (1,2] and [1,2) and (1,2) but for now, let the higher filter do
 *   the work.  Pretty minor optimization in the scheme of things though more elegant
 *
 * @author JJ Furman
 */
public class ScanRegionList {
    //  all lists of Comparable[]  (one element for each index value)

      //  null means minValue
    ArrayList mins = new ArrayList();

      //  null means maxValue, ie open ended
    ArrayList maxs = new ArrayList();

    public Comparable[] getMin(int i) {
        return (Comparable[])mins.get(i);
    }

    public Comparable[] getMax(int i) {
        return (Comparable[])maxs.get(i);
    }

    public int getNumScanRegions() {
        return mins.size();
    }

    /**
     * Currently doesn't expand these if its is a multi key index
     */
    public void addScanRegion(Comparable min, Comparable max) {
        mins.add(new Comparable[] { min });
        maxs.add(new Comparable[] { max });
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("Scan:");
        for(int i = 0; i < mins.size(); i++) {
            sb.append(" [");
            Comparable[] min = getMin(i);
            Comparable[] max = getMax(i);

            if(min.length == 1) {
                sb.append(min[0]);
                sb.append(',');
                sb.append(max[0]);
            } else {
                StringUtil.writeToBuffer(sb, min);
                sb.append(',');
                StringUtil.writeToBuffer(sb, max);
            }
            sb.append(']');
        }

        return sb.toString();
    }
}
