/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 24, 2002
 * Time: 4:56:53 PM
 */
package freebase.jdbc;

import freebase.server.DBConnection;
import freebase.server.DatabaseNode;

import java.rmi.RemoteException;
import java.sql.*;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * JDBC driver.  Urls are of the form:
 *
 * jdbc:freebase://{ipaddr}<BR>
 *   -or-<BR>
 * jdbc:freebase://{ipaddr}:{port}<BR>
 *   -or-<BR>
 * jdbc:freebase:local<BR><BR>
 *
 * TODO:  write a Thread that periodically pings, probably one for all would be fine
 */
public class F3Driver implements Driver {

      //  register this driver with the authorities
    static {
        try{
        DriverManager.registerDriver(new F3Driver());
        }catch(SQLException e) {
              //  TODO:  log failure, not this crap
            e.printStackTrace();
        }
    }

    /**
     * Connect to the database via the URL passed.
     */
    public Connection connect(String url, Properties info) throws SQLException {
        //System.out.println("connecting '" + url + "'  props: " + info);

        try{
            if(! acceptsURL(url)) {
                return null;
            }

            String user = info.getProperty("user");
            String password = info.getProperty("password");

            StringTokenizer st = new StringTokenizer(url, ":");
            st.nextToken();
            st.nextToken();
            String where = st.nextToken();

            DBConnection physicalCon = null;
            if(where.equals("local")) {
                if(st.hasMoreTokens()) {
                    throw new SQLException("URL: '" + url + "' should not contain anything after local for doing a local connection");
                }

                physicalCon = DatabaseNode.getLocalDBConnection(user, password);
            } else if(where.startsWith("//")) {
                String ip = where.substring(2);
                int port = -1;
                if(st.hasMoreTokens()) {
                    port = Integer.parseInt(st.nextToken());
                }
                physicalCon = DatabaseNode.getRemoteDBConnection(ip, port, "currently not specified in Driver.connect()", user, password);
            } else {
                throw new SQLException("Malformed URL: " + url);
            }

            if(physicalCon == null)
                return null;

            return new F3Connection(physicalCon, user, password, info);
        }catch(SQLException e) {
            throw e;
        }catch(Throwable t) {
            throw new SQLException("Failed to obtain connection: " + t.getMessage());
        }
    }

    public boolean acceptsURL(String url) throws SQLException {
        if(! url.startsWith("jdbc:freebase:")) {
            return false;
        }

        return true;
    }

    /**
     * TODO:  What does this do?
     */
    public DriverPropertyInfo[] getPropertyInfo(String url, Properties info)
            throws SQLException {
        return new DriverPropertyInfo[0];
    }

    public int getMajorVersion() {
        return 1;
    }

    public int getMinorVersion() {
        return 0;
    }

    /**
     * Whatever... Is any JDBC driver truely compliant?
     */
    public boolean jdbcCompliant() {
        return true;
    }

    /***************************************************************
     * Static Helper Methods
     */

    /**
     * Converts RemoteException into SQLException
     */
    public static void convertRemoteException(RemoteException re) throws SQLException {
        throw new SQLException("Network Error: " + re.getMessage());
    }
}
