/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 25, 2002
 * Time: 9:21:38 PM
 */
package freebase.jdbc;

import freebase.api.ColumnInfo;
import freebase.api.TupleIterator;
import freebase.util.SqlTypes;

import java.sql.ResultSetMetaData;
import java.sql.SQLException;

public class F3ResultSetMetaData implements ResultSetMetaData {
    F3ResultSet source;
    TupleIterator serverResult;

    protected F3ResultSetMetaData(F3ResultSet source, TupleIterator serverObject) {
        this.source = source;
        this.serverResult = serverObject;
    }

    public int getColumnCount() throws SQLException {
        return serverResult.getNumColumns();
    }

    /**
     * returns the
     */
    private ColumnInfo getColumnInfo(int pos) throws SQLException {
        if(pos < 1 || pos > serverResult.getNumColumns()) {
            throw new SQLException("Column pos : " + pos + " out of range [1," + serverResult.getNumColumns() +"]");
        }

        return serverResult.getColumnInfo(pos - 1);
    }

    public boolean isAutoIncrement(int column) throws SQLException {
        return false;
    }

    public boolean isCaseSensitive(int column) throws SQLException {
        return false;
    }

    /**
     * Should return true for all columns except maybe certain BLOB types
     */
    public boolean isSearchable(int column) throws SQLException {
        return true;
    }

    /**
     * Currently returns dummy value.
     */
    public boolean isCurrency(int column) throws SQLException {
        return false;
    }

    public int isNullable(int column) throws SQLException {
        return getColumnInfo(column).isNullable() ? 1 : 0;
    }

    /**
     * Currently returns dummy value.
     */
    public boolean isSigned(int column) throws SQLException {
        return false;
    }

    public int getColumnDisplaySize(int column) throws SQLException {
        return getColumnInfo(column).getDisplaySize();
    }

    public String getColumnLabel(int column) throws SQLException {
        return getColumnInfo(column).getName();
    }

    public String getColumnName(int column) throws SQLException {
        return getColumnInfo(column).getName();
    }

    public String getSchemaName(int column) throws SQLException {
        return getColumnInfo(column).getSchemaName();
    }

    /**
     * return total number of digits allowed, ie 2.34 is 3
     * @param column
     * @return
     * @throws SQLException
     */
    public int getPrecision(int column) throws SQLException {
        return getColumnInfo(column).getSize();
    }

    /**
     * return number of digits on right of decimal, ie 2.34 is 2
     * @param column
     * @return
     * @throws SQLException
     */
    public int getScale(int column) throws SQLException {
        return getColumnInfo(column).getScale();
    }

    public String getTableName(int column) throws SQLException {
        return getColumnInfo(column).getTableName();
    }

    /**
     * Currently returns dummy value.
     */
    public String getCatalogName(int column) throws SQLException {
        return "";
    }

    public int getColumnType(int column) throws SQLException {
        return getColumnInfo(column).getType();
    }


    public String getColumnTypeName(int column) throws SQLException {
        return SqlTypes.getNameForType(getColumnType(column));
    }

    /**
     * Currently returns dummy value.
     */
    public boolean isReadOnly(int column) throws SQLException {
        return false;
    }

    /**
     * Currently returns dummy value.
     */
    public boolean isWritable(int column) throws SQLException {
        return false;
    }

    /**
     * Currently returns dummy value.
     */
    public boolean isDefinitelyWritable(int column) throws SQLException {
        return false;
    }

    public String getColumnClassName(int column) throws SQLException {
        return SqlTypes.getJavaClassForType(getColumnType(column), getScale(column));
    }
}
