package freebase.lock;

import freebase.api.DeadlockableResource;

/**
 * Each actual wait should have it's own since other waiters are not effected when there is
 * a rivalry between two parties for a resource.  They can still wait themselves.  This is
 * so that selective deadlock notification can be done.
 *
 * NOTE:  since there is already a synchronized body where they are allocated
 *        these could easily be pooled
 */
public class DeadlockDetectorDelegate implements DeadlockableResource {
    boolean isInDeadlock = false;

    Object assigned;

    public DeadlockDetectorDelegate(Object weAssignedTo) {
        setAssigned(weAssignedTo);
    }

    public void setAssigned(Object weAssignedTo) {
        isInDeadlock = false;
        assigned = weAssignedTo;
    }

    public boolean isInDeadlock() {
        return isInDeadlock;
    }

    public void notifyOfDeadlock() {
        //System.out.println("NOTIFYING OF DEADLOCK: " + assigned);
        isInDeadlock = true;

        synchronized(assigned) {
            assigned.notifyAll();
        }
    }
}

