/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 24, 2001
 * Time: 11:23:05 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.lock;

import freebase.api.DeadlockDetector;
import freebase.api.DeadlockableResource;
import freebase.engine.Transaction;
import org.apache.log4j.Category;

import java.util.*;

/**
 * Uses an external thread to periodically check for deadlocks.  This is a pretty
 * poor performance implementation and I hope a better one gets written later.  The
 * goal of this implementation was correctness first.  There is one side effect, being
 * that if a transaction (T1) has a row share and a row exclusive lock, and another
 * transaction (T2) tries to grab it each lock type will have an entry in the detector.
 * I don't know at this point if that produces the correct behavior but it is left in
 * for now.<BR><BR>
 *
 * TODO:  pool ArrayList objects!!  a synch body already in place, size 5 because that is the total
 *        max that might ever be used... a smaller number (duh, like 1... 3 used now) would totally do!
 */
public class LocalDeadlockDetector extends Thread implements DeadlockDetector {
    static Category cat = Category.getInstance(LocalDeadlockDetector.class.getName());

    /**
     * Hashed on a, contains ArrayList of b
     */
    HashMap waitingEdges = new HashMap();

    /**
     * resources contains ArrayList of
     */
    HashMap resources = new HashMap();

    /**
     * Newly added things are the only thing that can be potentially in deadlock
     */
    ArrayList potentialDeadlocks = new ArrayList();

      //  used only to transfer information between methods
    ArrayList deadlockedResources = new ArrayList();

    /**
     * Start the deadlock detector going.
     */
    public LocalDeadlockDetector() {
        super("Local Deadlock Detector Thread");

          //  deadlock runs as a separate thread
        start();
    }

    /**
     * Add all of the edges in the set to the deadlock graph, a little bit
     * faster maybe.  If it really is, it might be better to add a group of sets
     */
    public void addWaitingEdges(Transaction a, HashSet set, DeadlockableResource res) {
        //System.out.println("addWaitingEdges: " + a + " to " + set + " resource: " + res);
        synchronized(waitingEdges) {
            boolean haveAddedOne = false;

            ArrayList waitList = (ArrayList)waitingEdges.get(a);
            if(waitList == null) {
                waitList = new ArrayList();
                waitingEdges.put(a, waitList);
            }

            Iterator it = set.iterator();
            while(it.hasNext()) {
                Object targetT = it.next();
                if(targetT.equals(a)) {  //  deals with the case that a is included in the set
                    continue;
                }
                haveAddedOne = true;
                waitList.add(targetT);
                potentialDeadlocks.add(targetT);
            }

              //  if a was the only targetT, then we shouldn't even bother whatsoever
            if(! haveAddedOne)
                return;

            ArrayList resList = (ArrayList)resources.get(a);
            if(resList == null) {
                resList = new ArrayList();
                resources.put(a, resList);
            }
            resList.add(res);
        }
    }

    public void addWaitingEdge(Transaction a, Transaction b,
                               DeadlockableResource res) {
        //System.out.println("addWaitingEdge: " + a + " to " + b + " resource: " + res);
        synchronized(waitingEdges) {
            ArrayList waitList = (ArrayList)waitingEdges.get(a);
            if(waitList == null) {
                waitList = new ArrayList(3);
                waitingEdges.put(a, waitList);
            }

            if(waitList.contains(b)) {
                System.out.println("DEBUG WARNING:  Detector already has deadlock edge: " + a + "->" + b);
            }

            waitList.add(b);

            ArrayList resList = (ArrayList)resources.get(a);
            if(resList == null) {
                resList = new ArrayList(3);
                resources.put(a, resList);
            }
            resList.add(res);

            potentialDeadlocks.add(b);

              //  let the deadlock detector do it's dirty work
              //  TODO:  don't notify, it will wake up soon enough
            //waitingEdges.notify();
        }
    }

    public void removeWaitingEdge(Transaction a, Transaction b,
                                  DeadlockableResource res) {
        //System.out.println("removeWaitingEdge: " + a + " to " + b + " resource: " + res);
        synchronized(waitingEdges) {
            ArrayList waitList = (ArrayList)waitingEdges.get(a);
            if(waitList != null) {  //  after a deadlock it will be
                waitList.remove(b);

                  //  remove if empty
                if(waitList.isEmpty()) {
                    waitingEdges.remove(a);
                }
            }

            ArrayList resList = (ArrayList)resources.get(a);
            if(resList != null) {
                resList.remove(res);

                  //  remove if empty
                if(resList.isEmpty()) {
                    resources.remove(a);
                }
            }
        }

        //dumpDetectorState();
    }

    /**
     * called by table locks to remove all edges
     */
    public void removeAllWaitingEdges(Transaction a) {
        //System.out.println("removeAllWaitingEdges: " + a);
        synchronized(waitingEdges) {
            waitingEdges.remove(a);
            resources.remove(a);
        }

        //dumpDetectorState();
    }

    /**
     * Debugging method, can be executed by calling "SYSTEM DDSTATE"
     */
    public void dumpDetectorState() {
        System.out.println("Deadlock Detector State:");
        synchronized(waitingEdges) {
            Iterator it = waitingEdges.keySet().iterator();
            while(it.hasNext()) {
                Object a = it.next();

                System.out.println("  Transaction " + a + " is waiting on ");
                ArrayList list = (ArrayList)waitingEdges.get(a);
                for(int i = 0; i < list.size(); i++) {
                    System.out.println("      " + list.get(0));
                }
            }

            it = resources.keySet().iterator();
            while(it.hasNext()) {
                Object a = it.next();

                System.out.println("  Transaction " + a + " has resources: ");
                ArrayList list = (ArrayList)resources.get(a);
                for(int i = 0; i < list.size(); i++) {
                    System.out.println("      " + list.get(0));
                }
            }
        }
    }

    /**
     * The thread that does the actual deadlock detection.
     *  TODO:  this needs to be rewritten to be more efficient
     *  right now it continues to recheck everything and it could cache
     * that information
     */
    public void run() {
        while(true) {
            synchronized(waitingEdges) {
                try{
                    waitingEdges.wait(1500);  //  TODO:  put this value in a config file
                }catch(InterruptedException ie) {  //  Deadlock thread interrupted! oh no!
                    System.out.println("Critical error:  Deadlock detector thread interrupted");
                    ie.printStackTrace();
                }

                if(potentialDeadlocks.size() > 0) {
                    //System.out.println("DEADLOCK DETECTOR RUNNING!");
                    for(int i = 0; i < potentialDeadlocks.size(); i++) {
                        detectDeadlock(potentialDeadlocks.get(i));
                    }

                    potentialDeadlocks.clear();
                }
            }

              //  if there were any deadlocks detected, remove them now
              //  this prevents a subtle deadlock (yup) with the tableLockLatch.
              //  (or whoever it is we are waking up)
              //  This could also be prevented by making that lock finer grained
              //  and not blocking on the tableLockLatch while in wait
            if(deadlockedResources.size() > 0) {
                for(int i = 0; i < deadlockedResources.size(); i++) {
                    DeadlockableResource dr = (DeadlockableResource)
                        deadlockedResources.get(i);
                    //System.out.println("resource in deadlock: " + dr);

                    //  note that it may be common to notify the same object multiple times in a row
                    //  because of the way the resources are added.  Possible to detect that here
                    dr.notifyOfDeadlock();
                }

                deadlockedResources.clear();
            }
        }
    }

    /**
     * This algorithm is very slow, but intentionally simple so
     * that there is no mystery.  I can imagine a number of optimizations
     * to this but I don't think it really matters at this point.
     * <p>
     * in the case of a -> b -> c -> d -> c
     * c and d are in deadlock.  If the above is the order that they are put on the deadlock waits,
     * then b will be the first thing called in detect deadlock.  Before this method would loop forever.
     * Now it explicitly finds the loop between c & d, but making the code a little more confusing.
     * bottom line is that this code could be simplified a little, but works now so I'm not changing it. 
     */
    private void detectDeadlock(Object n) {
        LinkedList q = new LinkedList();

        Set allSeen = new HashSet();
        allSeen.add(n);
        q.add(n);

        while(! q.isEmpty()) {
            Object cur = q.removeFirst();

            ArrayList next = (ArrayList)waitingEdges.get(cur);
            //System.out.println("DEQUEUED: " + cur + " next were " + next);
            if(next == null)
                continue;

            for(int i = 0; i < next.size(); i++) {
                Object node = next.get(i);
                if(n == node) {
                    markInDeadlock(n);
                    return;
                } else if(allSeen.contains(node)) {
                    //System.out.println("DEADLOCK FOUND w/ new CODE (TODO REMOVE)");  //  TODO:  remove
                    //  admittedly confusing code, but works.  to get this to occur, call
                    //  the unit test ConcurrencyTest.testOutOfOrderDeadlock

                    markInDeadlock(node);
                    return;
                }

                q.addLast(node);

                  //  this way n does not have to be part of the deadlock as originally written (wrongly)
                allSeen.add(node);
            }
        }
    }

    /**
     * All of the waits that this transaction has (multiple possible due currently
     * only to multiple additions via table)
     */
    private void markInDeadlock(Object offendingTransaction) {
        ArrayList list = (ArrayList)resources.get(offendingTransaction);
        if(list == null) {
            cat.error("Deadlock detected but nothing to release " + offendingTransaction);
            return;
        }

        //System.out.println(resources.get(offendingTransaction).toString() + " " + waitingEdges.get(offendingTransaction).toString());
        //System.out.println("ADDING : " + list.size() + " resources in deadlock");
        for(int i = 0; i < list.size(); i++) {
            deadlockedResources.add(list.get(i));
        }

          //  we don't want to see it no mo
        resources.remove(offendingTransaction);
        waitingEdges.remove(offendingTransaction);
    }
}
