/*
 * Copyright JJ Furman
 * User: user
 * Date: Mar 17, 2002
 * Time: 3:29:06 PM
 */
package freebase.log;

import freebase.log.data.LockTableEntry;
import freebase.log.tm.*;
import freebase.memorydb.log.*;

import java.sql.SQLException;

/**
 * This is the main log entry domain for the database
 */
public class DefaultLogEntryDomain implements LogEntryDomain, DefaultLogEntryCodes {


    public short getMinCode() {
        return 0;
    }

    public short getMaxCode() {
        return 4095;
    }

    public String getDomainName() {
        return "Default Log Entry Domain";
    }

    /**
     * TODO:  fill this in
     */
    public LogEntry createEmptyLogEntry(short code) throws SQLException {
        switch(code) {
            case ABSTRACT_LOG_CODE:   throw new SQLException("log entry found with zero code (none)");

            /**
             * Transaction Manager Codes
             */
            case BEGIN_TRANSACTION_LOG_CODE:
                return new BeginTransactionLogEntry();
            case COMMIT_TRANSACTION_LOG_CODE:
                return new CommitLogEntry();
            case ROLLBACK_LOG_CODE:
                return new RollbackLogEntry();
            case SAVEPOINT_LOG_CODE:
                return new SavepointLogEntry();
            case ROLLBACK_COMPLETE_LOG_CODE:
                return new RollbackCompleteLogEntry();
            case CHECKPOINT_START_LOG_CODE:
                return new CheckpointStartLogEntry();
            case CHECKPOINT_END_LOG_CODE:
                return new CheckpointEndLogEntry();

            case UNDO_LOG_CODE:
                return new UndoLogEntry(this);
                
            /**
             * Basic Locks
             */
            case LOCK_TABLE_LOG_CODE:
                return new LockTableEntry();

            /**
             * Memory DB
             */
            case MEMORY_ROW_UPDATE_LOG_CODE:  return new MemoryRowUpdateLogEntry();
            case MEMORY_STORAGE_LOG_CODE:     return new MemoryStorageLogEntry();
            case MEMORY_INDEX_LOG_CODE:       return new MemoryIndexLogEntry();
            case MEMORY_SEQUENCE_INC_LOG_CODE:return new MemorySequenceIncLogEntry();
            case MEMORY_SEQUENCE_STRUCTURE_LOG_CODE: return new MemorySequenceStructureLogEntry();


            default:
                throw new SQLException("log entry type code not recognized: " + code);
        }
    }


}
