package freebase.log;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 *  perhaps this class contains all of the ids for all of the entries?
 *  perhaps this class needs to be moved to the store directory?
 *
 *  all subclasses of this should be created in package:<BR><BR>
 *
 * It might be nice to have a debugging mode that counts the number of bytes to guarentee
 * that the log entry does not write more data than it is supposed to.<BR><BR>
 *
 * TODO:  get rid of constants pattern
 *
 * When implementing a new log entry, you must write these methods for every layer in the inheritance hierarchy:
 *     int getSize();
 *     short getCode();
 *     String toString();
 *     void readIn(DataInput in) throws IOException;
 *     void writeOut(LogOutput out) throws IOException;
 */
public abstract class LogEntry implements DefaultLogEntryCodes {

    /**
     *
     */
    public short getCode() {
        return ABSTRACT_LOG_CODE;
    }

    /**
     *  @return the size of this log entry in bytes when written out.  I am not using
     * the serializable mechanism because I need to know the size ahead of time, and I don't
     * want any extra baggage <BR><RB>
     *
     * All LogEntries should call super.getSize() to include all of the parents used space.  In fact
     * maybe I should even make this not abstract so that all have to call super.getSize()
     */
    public abstract int getSize();


    /**
     * All log entries must be able to read themselves in from a stream.
     * @param in
     * @throws IOException
     */
    public abstract void readIn(DataInput in) throws IOException;

    /**
     * Enables a log entry to write itself out to a stream (serialized)
     * @param out
     * @throws IOException
     */
    public abstract void writeOut(DataOutput out) throws IOException;

    /**
     *  copies the data into the log.  This way the log entry itself
     *  does not technically need to be a copy of the object.  It could
     *  point to a temporary fixed page.
     *
     *  @return the num bytes remaining in the log entry
     *
    abstract int copyInto(byte[] dest, int offset, int max);

    /*

    public static LogEntry extractLogEntry(byte[] data, int offset, int len) {
        //  read the first byte and create the appropriate log entry from
        //  this.  Should be a registered list or something
        return null;
    }

    public static LogEntry extractLogEntry() {
        return null;
        //  not sure yet how all of this is going to fit together.
    }
     */
}
