/*
 * Copyright JJ Furman
 * User: user
 * Date: Mar 17, 2002
 * Time: 3:19:56 PM
 */
package freebase.log;

import java.sql.SQLException;

/**
 * A log entry domain is a group of log entry types that are used by a particular set of libraries in
 * the database.  Since there is no way to predict the log entries that will get created in the future
 * this is just a way of organizing them so that they can be added later.<BR><RB>
 *
 * Log entry codes are encoded as a signed short.  All log entry codes from -4096 to 4095 are reserved
 * for this project to assign for internal log entries.
 */
public interface LogEntryDomain {
    /**
     * For getting the supported code range of the domain
     */
    short getMinCode();

    /**
     * For getting the supported code range of the domain
     */
    short getMaxCode();

    /**
     * Just a name
     */
    String getDomainName();

    /**
     * To return null if not supported by this domain
     */
    LogEntry createEmptyLogEntry(short code) throws SQLException;
}
