package freebase.log;


import freebase.engine.Transaction;

import java.io.Serializable;

/**
 * This class may look weird but it's because it is not externalized yet.  I
 * guess the thought was to make it.
 *
 * 0 (zero) is equivalent to null in LSN.  The first legal LSN is 1.<BR><BR>
 *
 * Note that this may eventually just be an interface, to deal with such things as not parsing the log entry
 * until it is necessary and skipping large log entries.
 */
public class LogEntryHeader implements Serializable {
          long LSN;
          long prevLSN;  //  not useful yet, could be an int for relative addressing
transient long nextLSN;  //  not really logged, but dynamically created for the caller so that they can walk the walk
          long timestamp;
          long transactionId;       //  transaction id of writer
          long prevTransactionLSN;  //  last LSN of

        LogEntry logEntry;

    //static DecimalFormat TODO:  add fixed formatting to output for nice display

    /**
     * Constructor used by the MemoryLogManager
     */
    public LogEntryHeader(long LSN, long prevLSN, long nextLSN, long timestamp,
                          long transactionId, long prevTransactionLSN,
                          LogEntry logEntry) {
        this.LSN = LSN;
        this.prevLSN = prevLSN;
        this.nextLSN = nextLSN;
        this.timestamp = timestamp;
        this.transactionId = transactionId;
        this.prevTransactionLSN = prevTransactionLSN;
        this.logEntry = logEntry;
    }

    /**
     * Another constructor, simpler.
     */
    public LogEntryHeader(long LSN, long prevLSN, long nextLSN, Transaction t, LogEntry logEntry) {
        this.LSN = LSN;
        this.prevLSN = prevLSN;
        this.nextLSN = nextLSN;
        this.timestamp = System.currentTimeMillis();
        if(t == null) {
            this.transactionId = 0;
            this.prevTransactionLSN = 0;
        } else {
            this.transactionId = t.getTransactionId();
            this.prevTransactionLSN = t.getLastLSN();
        }

        this.logEntry = logEntry;
    }

    public long getLSN() {
        return LSN;
    }

    /**
     * NOTE:  currently not used at all
     * @return
     */
    public long getPrevLSN() {
        return prevLSN;
    }

    public long getNextLSN() {
        return nextLSN;
    }

    public long getTimestamp() {
        return timestamp;
    }

    public long getTransactionId() {
        return transactionId;
    }

    public long getPrevTransactionLSN() {
        return prevTransactionLSN;
    }

    public LogEntry getLogEntry() {
        return logEntry;
    }


    public String toString() {
        return "LSN:" + LSN + " tid:" + transactionId + " [" + logEntry + "]";
    }
}