/*
 * Copyright JJ Furman
 * User: user
 * Date: Mar 17, 2002
 * Time: 2:10:37 PM
 */
package freebase.log;

import java.io.DataInput;
import java.io.IOException;
import java.sql.SQLException;

/**
 * A singleton, it's job is to keep track of LogEntry codes, and delegate the job of creating
 * the log entry objects to the various LogEntryDomains
 */
public class LogEntryManager {
    DefaultLogEntryDomain defaultDomain = new DefaultLogEntryDomain();

    /**
     * Given the code determines what kind of logentry to create and parses it in.  It
     * may be good to have a debugging mode that keeps track of the length actually read
     * by the
     */
    public LogEntry readLogEntry(short code, long length, DataInput in) throws SQLException {
        try{
            LogEntry logEntry = allocateLogEntry(code);
            logEntry.readIn(in);
            return logEntry;
        }catch(IOException ioe) {
              //  TODO:  may need to deal with IO exceptions in a more critical way, say failover.
            throw new SQLException(ioe.getMessage());
        }
    }

    /**
     *  Given the log entry code, create an empty one.<BR>
     * TODO:  support other domains for pluggable log entries etc...
     */
    private LogEntry allocateLogEntry(short code) throws SQLException {
        return defaultDomain.createEmptyLogEntry(code);
    }
}
