package freebase.log;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.sql.SQLException;

/**
 * When rolling back, each thing undone is made up of one undo message.
 *
 * Copyright Mar 1, 2004 JJ Furman
 * Time: 2:23:01 PM
 */
public class UndoLogEntry extends LogEntry {
    LogEntry operationUndone;

    LogEntryDomain logDomain;

    /**
     *
     * @param logDomain
     */
    public UndoLogEntry(LogEntryDomain logDomain) {
        this.logDomain = logDomain;
    }

    public UndoLogEntry(LogEntry entryToUndo) {
        operationUndone = entryToUndo;
    }

    public short getCode() {
        return UNDO_LOG_CODE;
    }

    public String toString() {
        return "Undo: " + operationUndone;
    }

    public LogEntry getOperationUndone() {
        return operationUndone;
    }

    /**
     * the code plus the operation data
     * @return
     */
    public int getSize() {
        return 2 + operationUndone.getSize();
    }

    public void readIn(DataInput in) throws IOException {
        short code = in.readShort();

        try{
            operationUndone = logDomain.createEmptyLogEntry(code);
            operationUndone.readIn(in);
        }catch(SQLException  i) {
            throw new IOException(i.getMessage());
        }
    }

    public void writeOut(DataOutput out) throws IOException {
        out.writeShort(operationUndone.getCode());
        operationUndone.writeOut(out);
    }
}
