/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 27, 2001
 * Time: 5:46:30 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.log.data;

import freebase.api.TableLockManager;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 * Lock the given table in the mode specified.  A log entry of this type will
 * only be added the first time it is requested.  That way rolling back past a savepoint
 * will not have problems of undoing locks already obtained.
 */
public class LockTableEntry extends TableLogEntry  {
    byte lockMode;

    /**
     * Should only be called when loading the entry from storage.
     */
    public LockTableEntry() {}

    public LockTableEntry(int tableId, byte lockMode) {
        super(tableId);
        this.lockMode = lockMode;
    }

    public byte getLockMode() {
        return lockMode;
    }

    public String toString() {
        return "Lock Table " + getTableId() + " in mode: " + TableLockManager.tableLockNames[lockMode];
    }

    public int getSize() {
        return super.getSize() + 1;
    }

    public void readIn(DataInput in) throws IOException {
        super.readIn(in);
        lockMode = in.readByte();
    }

    public void writeOut(DataOutput out) throws IOException {
        super.writeOut(out);
        out.writeByte(lockMode);
    }

    public short getCode() {
        return LOCK_TABLE_LOG_CODE;
    }
}
