/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 17, 2001
 * Time: 10:22:28 AM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.log.data;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

public abstract class RowUpdateLogEntry extends TableLogEntry {
    /**
     * if previous != current then this is the first update
     * in the transaction for this row.
     * boolean flag might do fine for most cases.
     * The old value is current replaced when a rollback occurs so that checking who the writer of the
     * data is valid.  (IE, seeing if the row is locked or whatever)  Consider the case when a single transaction
     * updates a row multiple times in a transaction and rolls-back part of the transaction.
     */
    private long previousTransactionId;

    /**
     * Used to construct an old version of the data.<BR>
     * May not be available in disk based implementation, though it should have SOMETHING
     * perhaps at least the previous update to the block or something
     */
    private long prevRowUpdateLSN;

    protected RowUpdateLogEntry() {}

    //  what about the first update to the table?
    //  Should we be conservative and have an explicit entry
    //  for that?  seems silly only because we can pack the info
    //  in much less space here.
    public RowUpdateLogEntry(long previousTransactionId, long prevRowUpdateLSN, int tableId) {
        super(tableId);
        this.previousTransactionId = previousTransactionId;
        this.prevRowUpdateLSN = prevRowUpdateLSN;
    }

    public long getPreviousTransactionId() {
        return previousTransactionId;
    }

    public long getPrevRowUpdateLSN() {
        return prevRowUpdateLSN;
    }

    /**********************
     * IO related methods
     */

    public int getSize() {
        return super.getSize() + 16;
    }

    public void readIn(DataInput in) throws IOException {
        super.readIn(in);

        previousTransactionId = in.readLong();
        prevRowUpdateLSN = in.readLong();
    }

    public void writeOut(DataOutput out) throws IOException {
        super.writeOut(out);

        out.writeLong(previousTransactionId);
        out.writeLong(prevRowUpdateLSN);
    }

    public String toString() {
        return "prev t:" + previousTransactionId + "  prev LSN:" + prevRowUpdateLSN;
    }
}
