package freebase.log.tm;

import freebase.engine.Transaction;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.ArrayList;

/**
 * Copyright Mar 1, 2004 JJ Furman
 * Time: 9:20:31 AM
 */
public class CheckpointStartLogEntry extends TMLogEntry {
    /**
     * list of all active transaction ID's.  We will need to roll these back if we don't see
     * a commit upon recovery.
     */
    long[] activeTransactions;

    /**
     * If we don't see this tells us how far back we have to go to find the most recent
     * transaction log entry.  If we don't see any more log entries for this transaction
     * this helps us find the it.
     * TODO:  read and write these out
     */
    long[] lastLSNPerTransaction;

    //  TODO:  do we need to know which transactions are in the prepared state, either externally driven or internally
    //  TODO:  actually, more critical for externally monitored transaction

    public CheckpointStartLogEntry() {
    }

    public CheckpointStartLogEntry(ArrayList activeTrans) {
        activeTransactions = new long[activeTrans.size()];
        lastLSNPerTransaction = new long[activeTrans.size()];

        for(int i = 0; i < activeTransactions.length; i++) {
            Transaction t = (Transaction)activeTrans.get(i);

            activeTransactions[i] = t.getTransactionId();
            lastLSNPerTransaction[i] = t.getLastLSN();
        }
    }

    /**
     * super  + 4 bytes for the length field + 8 bytes per active transaction id
     * @return
     */
    public int getSize() {
        return super.getSize() + 4 + activeTransactions.length * 8;
    }

    public void readIn(DataInput in) throws IOException {
        super.readIn(in);

        int len = in.readInt();
        activeTransactions = new long[len];
        for(int i = 0; i < len; i++) {
            activeTransactions[i] = in.readLong();
        }
    }

    public void writeOut(DataOutput out) throws IOException {
        super.writeOut(out);

        out.writeInt(activeTransactions.length);
        for(int i = 0; i < activeTransactions.length; i++) {
            out.writeLong(activeTransactions[i]);
        }
    }

    public short getCode() {
        return CHECKPOINT_START_LOG_CODE;
    }

    public String toString() {
        return "Checkpoint Start";
    }
}
