/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 12, 2001
 * Time: 7:33:10 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.memorydb;

import freebase.api.LogManager;
import freebase.engine.Transaction;
import freebase.log.LogEntry;
import freebase.log.LogEntryHeader;
import freebase.log.LogEntryManager;
import org.apache.log4j.Category;

import java.io.PrintStream;
import java.sql.SQLException;
import java.util.ArrayList;

/**
 * Memory Log Manager, a very simple initial log manager that can service
 * all of the basic requirements of a log system except durability.
 *
 * TODO:  support some kind of heuristic for moving hard LSN up when soft is holding too much data
 */
public class MemoryLogManager implements LogManager {
    /**
     * The number of entries we should wait until we actually purge
     */
    final public int PURGE_INTERVAL = 100;

    static Category cat = Category.getInstance(MemoryLogManager.class.getName());

    long lowWaterMarkLSN = 0;

    /**
     * (LSN - basePos) -> LogEntryHeader.  When empty, the next lsn is basePos!
     */
    LogList logEntries = new LogList();

      //  the first entry is basePos
    long basePos = 0;

    PrintStream debugOut = null;

    public MemoryLogManager() {
        /*  DEBUG HERE   
        try{
            OutputStream out = new FileOutputStream("data/log.txt", false);
            out = new BufferedOutputStream(out, 1024 * 1024 * 10);
            debugOut = new PrintStream(out, false);

            Runtime.getRuntime().addShutdownHook(new CloseDebugFile());
        }catch(Exception e) {
            e.printStackTrace();
        }
        // */
    }

    public long writeEntry(Transaction t, LogEntry entry, boolean flush) {
        synchronized(this) {
            long lsn = logEntries.size() + basePos;

            if(cat.isInfoEnabled()) {
                cat.info("writeLogEntry " + lsn + " (" + t + "," + entry + "," + flush +")");
            }

            LogEntryHeader header = new LogEntryHeader(lsn, lsn - 1, lsn + 1, t, entry);
            logEntries.add(header);

            if(debugOut != null) {
                debugOut.println(header);
            }
              //  ignore flush since we don't flush anything...

            return lsn;
        }
    }

    public LogEntryHeader getLogEntry(long lsn) throws SQLException {
        synchronized(this) {
            int pos = (int)(lsn - basePos);
            if(pos < 0 || pos >= logEntries.size()) {
                return null;
            }
            return (LogEntryHeader)logEntries.get(pos);
        }
    }

    /**
     * Gets rid of all log entries below softSLN, as long as the new softLSN is sufficiently larger
     * than the last softLSN we responded to.
     * @param softLSN is lower than hardLSN.  We would LIKE to have softLSN but we require hardLSN,
     *      to prevent snapshot too old message
     */
    public void raiseLowWaterMark(long hardLSN, long softLSN) {
        //if(true) return;  //  disables this process

       // System.out.println("Raise : " + softLSN + "   vs  " + hardLSN + "  size: " + logEntries.size() + " base: " + basePos);

       // System.out.println("raise " + softLSN + "  log size: " + logEntries.size());
        synchronized(this) {
              //  if ok to purge all
            if(softLSN == 0) {
                if(logEntries.size() > PURGE_INTERVAL) {
                    basePos += logEntries.size();
                    logEntries.clear();
                    //logEntries.trimToSize();  <-  something like this might be good
                }
                //System.out.println("PURGE ALL");
            } else {
                int numToPurge = (int)(softLSN - basePos);
                if(numToPurge > PURGE_INTERVAL) {
                    logEntries.purgeFirst(numToPurge);
                    basePos = softLSN;
                   // System.out.println("PURGE " + softLSN);
                }
            }
        }

        /*
        if(true) return;
        //System.out.println("raiseLowWaterMark  hard:" + hardLSN + "  soft: " + softLSN);
        if(hardLSN == 0) {
            synchronized(this) {
                basePos += logEntries.size();
                logEntries.clear();
            }
        }

        //System.out.println("MemoryLogManager: raiseLowWaterMark(" + newLowWaterMarkLSN + ")  not implemented");
        if(true) return;
/*
        // TODO:  this is wrong wrong.  need to record first as offset,
        //  also should do this in increments of 100 or something
        synchronized(logEntries) {
            for(long i = lowWaterMarkLSN; i < hardLSN; i++) {
                logEntries.remove(i);
                System.out.println(logEntries.size());
            }
        }

        ArrayList list;
        list.subList()
        */
    }

    /**
     * Current does nothing with the logEntryManager because this thing never reads messages from the
     * disk or network
     */
    public void setLogEntryManager(LogEntryManager logEntryManager) {
    }

    /**
     * Shutdown task to flush out the debug log.  Only created if the log is printing to a debug file.
     */
    class CloseDebugFile extends Thread {
        public void run() {
            System.out.println("Closing Log Debug File...");
            debugOut.close();
        }
    }
}

/**
 * We explicitly subclass so that we can have access to removeRange.  DUH!
 */
class LogList extends ArrayList { 
    public void purgeFirst(int firstN) {
        removeRange(0, firstN);
    }
}