/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 11, 2001
 * Time: 3:51:48 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.memorydb;

/**
 * Holds all of the rowdata.  Also maybe more.  If it never does, get rid
 * of this tripe.<BR><BR>
 *
 * Eventually this will need to have a dirty flag for writing out to disk on
 * checkpoint.  Checkpoints will be like mini level n backups
 */
public class MemoryRowData {
    /**
     * Transaction ID of last writer. lastWriter may be null if the most recent write was a while ago.
     * Ie, below the lowest water mark of view times.
     * TODO:  Note: Technically this should be directly derivable from curStateLSN, and only null legal when
     *       curStateLSN is not reachable either.  On the other hand, that requires a log lookup for each
     *       read.
     */
    protected long lastWriterTid = 0;

    /**
     * The tid of the transaction who has a temporary lock on this.  I really wanted to avoid this but.
     * One possible alternative is to lock the row lock explicitly.  This makes much more log entries but
     * removes some space requirements in the table since lastWriterTid would then have this value always
     * <P>
     * Another decent solution is to store all rowid's pending lock tid
     * in a separate table.  They should always be few, so this should work ok.  Both the proper lock
     * and unlock should remove this entry.  It must be guarenteed that the entry gets removed despite
     * any error condition.  Having a flag as a hint here is optional, but would spare the common case of
     * there being no lock pending, (thus saving a lock and hashlookup when going to lock)
     *
     * TODO:  this kind of data won't be required if the block lock is held until it is decided if the row
     *      will be written or not.
     */
    protected long pendingLockTid;

    /**
     * Current data
     */
    private Object[] data = null;

    //  or use:  experimental, not too great
  //  private byte[] bytes;
//    static ThreadLocal byteReader = new ThreadLocal();  BAD IDEA, try with this subclassing  


    //SoftReference cache;

    /**
     * The LSN of the log entry that most recently established this value.
     */
    private long curStateLSN = 0;

    /**
     * Constructed at insert time
     */
    public MemoryRowData(long lastWriter, Object[] data, long curStateLSN) {
        this.lastWriterTid = lastWriter;
        this.pendingLockTid = lastWriter;
        setData(data);
        this.curStateLSN = curStateLSN;
    }

    /**
     * dangerous, but ok as long as this is only read from!
     */
    public Object[] getData() {
        /*
        if(bytes == null) {
            return null;
        }

        //Object[] data = (Object[])cache.get();
        //if(data != null) { return data; }
        ByteArrayDataInput reader = (ByteArrayDataInput)byteReader.get();
        if(reader == null) {
            reader =  new ByteArrayDataInput();
            byteReader.set(reader);
        }
        reader.setBytes(bytes);

        try {
            return ObjectIO.readObjects(reader);
        }catch(IOException ioe) {
            throw new OhShitException("bad format", ioe);
        }
        */

        return data;
    }

    /**
     * Private because only used locally.  Don't change this without thinkin.
     * Was put in to make it easier to swap encoding techniques
     */
    private void setData(Object[] d) {
        /*
        if(d == null) {
            bytes = null;
            return;
        }
       // cache = new SoftReference(d);
        bytes = new byte[ObjectIO.getSize(d)];
        try {
            ObjectIO.writeObjects(new ByteArrayDataOutput(bytes), d);
        }catch(IOException ioe) {
            throw new OhShitException("bad format", ioe);
        }
        */
        data = d;
    }

    public long getRecentTransactionId() {
        return lastWriterTid;
    }

    public long getCurStateLSN() {
        return curStateLSN;
    }

    /**
     * Called during an explicit unlock, the data is left unchanged and the last writer is returned to be
     * what it was.
     * @param tid
     */
    public void setlastWriterTid(long tid) {
        synchronized(this) {
            this.lastWriterTid = tid;
        }
    }

    public boolean isDeleted() {
        return data == null;
    }

/*

    public void setNewState(Object[] data, long curStateLSN) {
        synchronized(this) {
            this.data = data;
            this.curStateLSN = curStateLSN;
        }
    }
    */

    /**
     * Set all three characteristics of the row data object
     */
    public void setNewState(long tid, Object[] data, long curStateLSN) {
        synchronized(this) {
            setData(data);
            this.curStateLSN = curStateLSN;
            this.lastWriterTid = tid;
            this.pendingLockTid = tid;
        }
    }
}
