/**
 * Copyright 2004 JJ Furman
 * Time: 6:35:53 PM
 */
package freebase.memorydb;

import freebase.api.LogEntryProcessor;
import freebase.api.SequenceStorage;
import freebase.engine.Transaction;
import freebase.engine.TransactionManager;
import freebase.log.LogEntry;
import freebase.memorydb.log.MemorySequenceIncLogEntry;

import java.math.BigInteger;
import java.sql.SQLException;

/**
 * Single memory implementation.  Locking is done for one reason:  to ensure that a concurrently
 * executing checkpoint either atomically forces this value to be written, or that the corresponding log
 * message is after the start checkpoint.   IE:  if the log message was written, then a checkpoint is started,
 * then the checkpoint process MUST write out the current value... and it will because it must grab the local
 * lock first.<BR>
 */
public class MemorySequenceStorage implements SequenceStorage {
    TransactionManager tm;
    int entityId;
    long lastWriterLSN = 0;
    BigInteger value;


    public MemorySequenceStorage(TransactionManager tm, int id, BigInteger nextVal) {
        this.tm = tm;
        entityId = id;
        value = nextVal;
    }

    public int getSequenceId() {
        return entityId;
    }

    //  TODO:  handle checkpointing by checking that a value needs to be written at all. (lsn of l

    synchronized public BigInteger getValue() throws SQLException {
        return value;
    }

    synchronized public void setValue(BigInteger val, boolean flush) throws SQLException {
        this.value = val;

          //  flush!
        lastWriterLSN = tm.getLog().writeEntry(null, new MemorySequenceIncLogEntry(entityId, val), flush);
    }


    public void drop() throws SQLException {
        //  do nothing for now, just get garbage collected
    }

    /**
     * Processes the log entry by setting the next value available to the connections.
     * @param applyMode
     * @param lsn
     * @param t
     * @param logEntry
     * @throws SQLException
     */
    public void processLogEntry(int applyMode, long lsn, Transaction t, LogEntry logEntry)
            throws SQLException {
        MemorySequenceIncLogEntry inc = (MemorySequenceIncLogEntry)logEntry;
        if(applyMode == LogEntryProcessor.REDO_ENTRY) {
            value = inc.getNextValue();
        }

          //  increment log entries are not reversible
    }
}
