/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 13, 2001
 * Time: 6:10:32 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.memorydb.log;

import freebase.log.data.RowUpdateLogEntry;
import freebase.util.ObjectIO;
import freebase.util.StringUtil;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 *  Represents a change to a row.
 */
public class MemoryRowUpdateLogEntry extends RowUpdateLogEntry {
    private long rowId;

    private Object[] oldData;
    private Object[] newData;

    /**
     * Should only be called when loading from storage.
     */
    public MemoryRowUpdateLogEntry() {}

    public MemoryRowUpdateLogEntry(long previousTransactionId, long prevRowUpdateLSN, int tableId, long rowId, Object[] oldData, Object[] newData) {
        super(previousTransactionId, prevRowUpdateLSN, tableId);
        this.rowId = rowId;
        this.oldData = oldData;
        this.newData = newData;
    }


    public long getRowId() {
        return rowId;
    }

    public Object[] getOldData() {
        return oldData;
    }

    public Object[] getNewData() {
        return newData;
    }

    /**
     * For printing basic info on this guy
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        if(oldData == null) {
            sb.append("Insert Into ");
        } else if(newData == null) {
            sb.append("Delete From ");
        } else {
            sb.append("Update");
        }

        sb.append(" tableId: ");
        sb.append(getTableId());
        sb.append(" rowId: ");
        sb.append(rowId);
        sb.append("  newdata: ");
        StringUtil.writeToBuffer(sb, newData);
        sb.append("  olddata: ");
        StringUtil.writeToBuffer(sb, oldData);
        sb.append("  ");
        sb.append(super.toString());
        return sb.toString();
    }

    /*******************************
     * IO methods
     */
    public short getCode() {
        return MEMORY_ROW_UPDATE_LOG_CODE;
    }

    /**
     * The size is... 8 for the row id plus for each data it is...
     *    2 bytes for numEntries, then one byte for each entry for the type code (all in a row),
     *    then a variable size for each one
     */
    public int getSize() {
        return super.getSize() + 8 + ObjectIO.getSize(oldData) + ObjectIO.getSize(newData);
    }

    public void readIn(DataInput in) throws IOException {
        super.readIn(in);

        rowId = in.readLong();

        oldData = ObjectIO.readObjects(in);
        newData = ObjectIO.readObjects(in);
    }

    public void writeOut(DataOutput out) throws IOException {
        super.writeOut(out);

        out.writeLong(rowId);

        ObjectIO.writeObjects(out, oldData);
        ObjectIO.writeObjects(out, newData);
    }
}
