/**
 * Copyright 2004 JJ Furman
 * Time: 6:45:11 PM
 */
package freebase.memorydb.log;

import freebase.log.LogEntry;
import freebase.util.ObjectIO;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.math.BigInteger;

/**
 * Is written to the log when a new sequence value is needed and there are no more left in the cache.
 * TODO:  maybe call this SequenceInc for greater clarity?
 */
public class MemorySequenceIncLogEntry extends LogEntry {
    /**
     * The sequence id
     */
    int entityId;

    /**
     * What is the next value someone else should get?
     * byte[] representation of the BigInteger
     */
    BigInteger nextVal;

    /**
     * constructor for loading from a
     */
    public MemorySequenceIncLogEntry() {}

    public MemorySequenceIncLogEntry(int entityId, BigInteger nextVal) {
        this.entityId = entityId;
        this.nextVal = nextVal;
    }

    public short getCode() {
        return MEMORY_SEQUENCE_INC_LOG_CODE;
    }

    public int getEntityId() {
        return entityId;
    }

    public BigInteger getNextValue() {
        return nextVal;
    }

    public int getSize() {
        return 5 + ObjectIO.getBigIntLength(nextVal);
    }

    public void readIn(DataInput in) throws IOException {
        entityId = in.readInt();
        nextVal = ObjectIO.readBigInteger(in);
    }

    public void writeOut(DataOutput out) throws IOException {
        out.writeInt(entityId);
        ObjectIO.writeBigInteger(out, nextVal);
    }

    public String toString() {
        return "Seq Update (" + entityId + ") nextval: " + nextVal;
    }
}
