/**
 * Copyright 2004 JJ Furman
 * Time: 8:52:52 AM
 */
package freebase.memorydb.log;

import freebase.log.LogEntry;
import freebase.util.ObjectIO;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.math.BigInteger;

/**
 * Log entry representing the creation and deletion of sequence storage.
 */
public class MemorySequenceStructureLogEntry extends LogEntry {
    final public static byte CREATE_STORAGE = 10;
    final public static byte DELETE_STORAGE = 11;

    final public static String[] text = {"CREATE", "DELETE"};

    int entityId;
    byte opCode;

    /**
     * to save one lousy flush :(  oh well, decisions must be made.
     */
    BigInteger nextVal;

    public MemorySequenceStructureLogEntry() {}

    public MemorySequenceStructureLogEntry(int entityId, byte opCode, BigInteger nextVal) {
        this.entityId = entityId;
        this.opCode = opCode;
        this.nextVal = nextVal;
    }

    public int getEntityId() {
        return entityId;
    }

    /**
     * return
     * @return
     */
    public byte getOpCode() {
        return opCode;
    }

    public BigInteger getNextVal() {
        return nextVal;
    }

    public short getCode() {
        return MEMORY_SEQUENCE_STRUCTURE_LOG_CODE;
    }

    public int getSize() {
        return 6 + ObjectIO.getBigIntLength(nextVal);
    }

    public void readIn(DataInput in) throws IOException {
        entityId = in.readInt();
        opCode = in.readByte();
        nextVal = ObjectIO.readBigInteger(in);
    }

    public void writeOut(DataOutput out) throws IOException {
        out.writeInt(entityId);
        out.writeByte(opCode);
        ObjectIO.writeBigInteger(out, nextVal);
    }

    public String toString() {
        return "Seq Structure Change: " + text[opCode - CREATE_STORAGE] + " (" + entityId + ") nextval:" + nextVal;
    }
}
