/*
 * Copyright JJ Furman
 * User: user
 * Date: Mar 20, 2002
 * Time: 10:00:15 AM
 */
package freebase.memorydb.log;

import freebase.log.LogEntry;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 * Journals a change in storage allocation
 * TODO:  maybe implement a StorageLogEntry
 */
public class MemoryStorageLogEntry extends LogEntry {
    /**
     * op codes
     */
    final public static byte CREATE_STORAGE = 0;
    final public static byte DELETE_STORAGE = 1;
    final public static byte TRUNCATE_STORAGE = 2;

    /**
     * to what codes
     */
    final public static byte TABLE_STORAGE = 100;
    final public static byte INDEX_STORAGE = 101;

    /**
     * storage for what entity was created?
     */
    int entityId;

    /**
     * The following stuff could easily be stored in one byte with masks but I'm too lazy
     */

    /**
     * one of CREATE_STORAGE or DELETE_STORAGE
     */
    byte operation;

    /**
     * one of TABLE_STORAGE or INDEX_STORAGE
     */
    byte storageType;

    /**
     * hack!
     */
    boolean isUnique = false;

    /**
     * Used in replaying from log
     */
    public MemoryStorageLogEntry() {}

    /**
     * @param entityId
     * @param operation
     * @param storageType
     * @param isUnique only used for index space.  Later this will be replaced with general params!!!
     */
    public MemoryStorageLogEntry(int entityId, byte operation, byte storageType, boolean isUnique) {
        this.entityId = entityId;

        this.operation = operation;
        this.storageType = storageType;
        this.isUnique = isUnique;
    }

    public MemoryStorageLogEntry(int entityId, byte operation, byte storageType) {
        this(entityId, operation, storageType, false);
    }

    public int getEntityId() {
        return entityId;
    }

    public byte getOperation() {
        return operation;
    }

    public byte getStorageType() {
        return storageType;
    }

    public boolean isUnique() {
        return isUnique;
    }

    public short getCode() {
        return MEMORY_STORAGE_LOG_CODE;
    }

    public int getSize() {
        return 7;
    }

    public void readIn(DataInput in) throws IOException {
        entityId = in.readInt();

        operation = in.readByte();
        storageType = in.readByte();
        if(in.readByte() != 0) {
            isUnique = true;
        }
    }

    public void writeOut(DataOutput out) throws IOException {
        out.writeInt(entityId);

        out.writeByte(operation);
        out.writeByte(storageType);
        out.writeByte(isUnique ? (byte)1 : 0);
    }


    public String toString() {
        StringBuffer sb = new StringBuffer();

        switch(operation) {
            case CREATE_STORAGE:   sb.append("CREATE ");  break;
            case DELETE_STORAGE:   sb.append("DELETE ");  break;
            case TRUNCATE_STORAGE: sb.append("TRUNCATE ");  break;
            default:  sb.append("UNKNOWN CODE:" + operation);
        }

        if(storageType == TABLE_STORAGE) {
            sb.append("TABLE ");
        } else {
            if(isUnique()) {
                sb.append("UNIQUE ");
            }
            sb.append("INDEX ");
        }

        sb.append(entityId);
        return sb.toString();
    }
}
