/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 29, 2002
 * Time: 1:36:46 PM
 */
package freebase.server;

import java.io.IOException;
import java.net.*;
import java.nio.channels.ServerSocketChannel;
import java.rmi.server.RMISocketFactory;
//import java.nio.channels.ServerSocketChannel;

/**
 * Wraps the default socket factory just to record all sockets that are
 * created.  This is experimental code.
 */
public class DBSocketFactory extends RMISocketFactory {
    /*  Commented out to compile with JDK1.3  */
    RMISocketFactory def;

    public DBSocketFactory(RMISocketFactory def) {
        this.def = def;
    }

    public Socket createSocket(String host, int port)
            throws IOException {
        System.out.println("DBSocketFactory.createSocket: " +host + ":" + port);
        return def.createSocket(host, port);
    }

    public ServerSocket createServerSocket(int port)
            throws IOException {
        System.out.println("DBSocketFactory.createServerSocket: " + port);
        return new ServerSocketWrapper(def.createServerSocket(port));
    }

}


class ServerSocketWrapper extends ServerSocket {
    ServerSocket main;

    public ServerSocketWrapper(ServerSocket main) throws IOException {
        this.main = main;
    }

    public Socket accept() throws IOException {
        Socket s = main.accept();
        System.out.println("Accepted a socket from: " + s.getInetAddress());
        return s;
    }

    public void bind(SocketAddress endpoint) throws IOException {
        main.bind(endpoint);
    }

    public void bind(SocketAddress endpoint, int backlog) throws IOException {
        main.bind(endpoint, backlog);
    }

    protected Object clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException("Doesn't work here");
    }

    public void close() throws IOException {
        main.close();
    }

    public boolean equals(Object obj) {
        return main.equals(obj);
    }

    public ServerSocketChannel getChannel() {
        return main.getChannel();
    }

    public InetAddress getInetAddress() {
        return main.getInetAddress();
    }

    public int getLocalPort() {
        return main.getLocalPort();
    }

    public SocketAddress getLocalSocketAddress() {
        return main.getLocalSocketAddress();
    }

    public int getReceiveBufferSize()
            throws SocketException {
        return main.getReceiveBufferSize();
    }

    public boolean getReuseAddress() throws SocketException {
        return main.getReuseAddress();
    }

    public int getSoTimeout() throws IOException {
        return main.getSoTimeout();
    }

    public int hashCode() {
        return main.hashCode();
    }

    public boolean isBound() {
        return main.isBound();
    }

    public boolean isClosed() {
        return main.isClosed();
    }

    public String toString() {
        return main.toString();
    }

    public void setSoTimeout(int timeout) throws SocketException {
        main.setSoTimeout(timeout);
    }

    public void setReuseAddress(boolean on) throws SocketException {
        main.setReuseAddress(on);
    }

    public void setReceiveBufferSize(int size) throws SocketException {
        main.setReceiveBufferSize(size);
    }
}

