/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 10, 2002
 * Time: 1:32:00 PM
 */
package freebase.server;

import java.io.PrintStream;
import java.io.Serializable;
import java.util.ArrayList;

/**
 * An execution plan describes how your data will be obtained.  Each object is really a step
 * in the plan which forms an entire plan via the tree.
 */
public class ExecutionPlan implements Serializable {
    String stepInfo;
    ArrayList subPlans = new ArrayList();

    public ExecutionPlan(String info) {
        stepInfo = info;
    }

    public ExecutionPlan(String info, ExecutionPlan subPlan) {
        this(info);

        addSubPlan(subPlan);
    }

    public void dump(PrintStream out) {
        printContents(out, "");
    }

    protected void printContents(PrintStream out, String prefix) {
        String subPrefix = null;

        out.println(prefix + stepInfo);
        for(int i = 0; i < subPlans.size(); i++) {
            if(subPrefix == null)
                subPrefix = prefix + "    ";

            ((ExecutionPlan)subPlans.get(i)).printContents(out, subPrefix);
        }
    }

    public void addSubPlan(ExecutionPlan plan) {
        subPlans.add(plan);
    }
}
