/*
 * Copyright JJ Furman
 * User: user
 * Date: May 1, 2002
 * Time: 10:19:15 AM
 */
package freebase.server;

import freebase.api.ColumnInfo;
import freebase.api.TupleIterator;

import java.sql.SQLException;

/**
 * A result set returned to a query made on a local connection.
 */
public class LocalQueryResult implements TupleIterator {
    TupleIterator wrap;
    DBConnectionImpl con;

    public LocalQueryResult(DBConnectionImpl con,  TupleIterator wrap) {
        this.wrap = wrap;
        this.con = con;
    }

    /**********************************************************
     * Standard method pass throughs
     */

    /**
     * Only the close method is exceptional.  It removes this guy from the active results
     */
    public void close() throws SQLException {
        con.enterActiveState();
        try{
            con.removeActiveResult(wrap);
            wrap.close();
        } finally {
            con.exitActiveState();
        }
    }

    public void deleteTuple() throws SQLException {
        con.enterActiveState();
        try{
        wrap.deleteTuple();
        } finally {
            con.exitActiveState();
        }
    }

    public String getColumnName(int i) throws SQLException {
        con.enterActiveState();
        try{
            return wrap.getColumnName(i);
        } finally {
            con.exitActiveState();
        }
    }

    public ColumnInfo getColumnInfo(int i) throws SQLException {
        con.enterActiveState();
        try{
            return wrap.getColumnInfo(i);
        } finally {
            con.exitActiveState();
        }
    }

    public ExecutionPlan getExecutionPlan() throws SQLException {
        con.enterActiveState();
        try{
            return wrap.getExecutionPlan();
        } finally {
            con.exitActiveState();
        }
    }

    public int getNumColumns() throws SQLException {
        con.enterActiveState();
        try{
            return wrap.getNumColumns();
        } finally {
            con.exitActiveState();
        }
    }

    public Comparable getValue(int i) throws SQLException {
        con.enterActiveState();
        try{
            return wrap.getValue(i);
        } finally {
            con.exitActiveState();
        }
    }

    public boolean isUpdateable() throws SQLException {
        con.enterActiveState();
        try{
            return wrap.isUpdateable();
        } finally {
            con.exitActiveState();
        }
    }

    public boolean next() throws SQLException {
        con.enterActiveState();
        try{
        return wrap.next();
        } finally {
            con.exitActiveState();
        }
    }

    public void reset() throws SQLException {
        con.enterActiveState();
        try{
        wrap.reset();
        } finally {
            con.exitActiveState();
        }
    }

    public void setValue(int i, Comparable value) throws SQLException {
        con.enterActiveState();
        try{
        wrap.setValue(i, value);
        } finally {
            con.exitActiveState();
        }
    }
}
