/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 19, 2001
 * Time: 8:21:10 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.sql;

import freebase.util.OhShitException;

import java.io.Serializable;

public class ColumnRef implements Serializable {
    String schema;
    String table;
    String column;

    /**
     * Assigned later, this field keeps track of which table in the query this is.
     * This should be assigned after all subqueries are all flattened out.
     */
    int tableOffset = -1;

    public ColumnRef(String schema, String table, String column) {
        this.schema = schema;
        this.table = table;
        this.column = column;
    }

    public ColumnRef(String table, String column) {
        this(null, table, column);
    }

    public ColumnRef(String column) {
        this(null, column);
    }

    public String getSchema() {
        return schema;
    }

    public String getTable() {
        return table;
    }

    public String getColumn() {
        return column;
    }


    public int getTableOffset() {
        return tableOffset;
    }

    /**
     * Like equals, but didn't want to interfere
     */
    public boolean isEquivalentTo(ColumnRef other) {
          //  TODO:  make these two asserts!
        if(getTableOffset() == -1) {
            throw new OhShitException("Column " + this + " not verified and bound");
        }

        if(other.getTableOffset() == -1) {
            throw new OhShitException("Column " + other + " not verified and bound");
        }

        if(getTableOffset() == other.getTableOffset() &&
           getColumn().equals(other.getColumn())) {
            return true;
        }

        return false;
    }

    /**
     * Called by QueryAnalysis
     */
    public void setTableOffset(int tableOffset) {
        this.tableOffset = tableOffset;
    }

    public void writeToBuffer(StringBuffer sb) {
        if(schema != null) {
            sb.append(schema);
            sb.append('.');
        }

        if(table != null) {
            sb.append(table);
            sb.append(".");
        }

        sb.append(column);
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        writeToBuffer(sb);
        return sb.toString();
    }
}
