/*
 * Copyright JJ Furman
 * User: user
 * Date: May 17, 2002
 * Time: 10:58:35 AM
 */
package freebase.sql;

import freebase.util.NumberUtil;

import java.math.BigInteger;

/**
 * Statement for creating sequences.
 * TODO:  maybe can make all these longs, see oracles ranges for this stuff
 * TODO:  get rid of AlterSequenceStatement, make this polymorphic.
 *
 *
 */
public class CreateOrAlterSequenceStatement extends DDLStatement {
    boolean isCreate;

    /**
     * Names of sequences are just like table names.  Perhaps TableRef should be called ObjectRef.
     */
    TableRef name;

    BigInteger increment = BigInteger.ONE;
    BigInteger start = BigInteger.ONE;

    BigInteger maxValue;
    BigInteger minValue;

    BigInteger cache = NumberUtil.BIG_INTEGER_TWENTY;

    boolean cycle;
    boolean order;
    boolean flush = true;

    public void setIsCreate(boolean isCreate) {
        this.isCreate = isCreate;
    }

    /**
     * To identify this as a create or alter statement
     * @return
     */
    public boolean isCreate() {
        return isCreate;
    }

    public String getSchemaName(String defaultUser) {
        return name.getSchema(defaultUser);
    }

    public String getName() {
        return name.getTable();
    }

    public BigInteger getIncrement() {
        return increment;
    }

    public BigInteger getStart() {
        return start;
    }

    public BigInteger getMaxValue() {
        return maxValue;
    }

    public BigInteger getMinValue() {
        return minValue;
    }

    public BigInteger getCache() {
        return cache;
    }

    public boolean isCycle() {
        return cycle;
    }

    public boolean isOrder() {
        return order;
    }

    public boolean isFlush() {
        return flush;
    }

    public void setCache(BigInteger cache) {
        if(cache == null) {
            this.cache = BigInteger.ONE;
        } else {
            this.cache = cache;
        }
    }

    public void setCycle(boolean cycle) {
        this.cycle = cycle;
    }

    public void setIncrement(BigInteger increment) {
        this.increment = increment;
    }

    public void setMaxValue(BigInteger maxValue) {
        this.maxValue = maxValue;
    }

    public void setMinValue(BigInteger minValue) {
        this.minValue = minValue;
    }

    public void setName(TableRef name) {
        this.name = name;
    }

    public void setOrder(boolean order) {
        this.order = order;
    }

    public void setStart(BigInteger start) {
        this.start = start;
    }

}
