package freebase.sql;

import freebase.parser.ParseException;
import freebase.sql.expression.ExpressionVisitor;
import freebase.sql.expression.LiteralExpression;

import java.sql.SQLException;
import java.util.ArrayList;


/**
 *  class encapsulates a single insert
 */
public class InsertStatement extends DMLStatement {
    TableRef table;
    
    ArrayList fields = null;
    ArrayList values = new ArrayList();

    /**
     * set when this is an insert into select statement
     */
    SelectStatement selectStatement;

    public InsertStatement() { }

    /**
     * Construct an Insert Statement with String values to be parsed directly
     */
    public InsertStatement(String table, String[] values) throws SQLException {
        this(new TableRef(table), values);
    }

    public InsertStatement(TableRef table, String[] values) throws SQLException {
        try{
        setTable(table);
        for(int i = 0; i < values.length; i++) {
              //  TODO:  wrap the values with a literal expression
            addValue(new LiteralExpression(values[i]));
        }
        }catch(ParseException pe) {
            throw new SQLException("FAILED TO INSERT: " + pe.getMessage());
        }
    }

    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
          //  bind the actual inserted values
        for(int i = 0; i < getNumValues(); i++) {
            curPos = getValue(i).bindLiterals(values, curPos);
        }

        return curPos;
    }

    /**
     * bind sequences for all insert values!
     * @param seqBinder
     * @throws SQLException
     */
    public void bindSequences(ExpressionVisitor seqBinder) throws SQLException {

          //  bind select items
        for(int i = 0; i < getNumValues(); i++) {
            getValue(i).accept(seqBinder);
        }

        if(selectStatement != null) {
            selectStatement.bindSequences(seqBinder);
        }
    }

    public void addField(String field) {
        if(fields == null)
            fields = new ArrayList();
        fields.add(field);
    }
    
    public void addValue(SqlExpression value) {
        values.add(value);
    }
    
    public ArrayList getFields() {
        return fields;
    }

    public String getField(int pos) {
        return (String)fields.get(pos);
    }

    public int getNumFields() {
        return fields.size();
    }
/*
    public ArrayList getValues() {
        return values;
    }*/

    public int getNumValues() {
        return values.size();
    }

    public SqlExpression getValue(int pos) {
        return (SqlExpression)values.get(pos);
    }

    public void setTable(TableRef table) {
        this.table = table;
    }

    public TableRef getTable() {
        return table;
    }

    public SelectStatement getSelectStatement() {
        return selectStatement;
    }

    public void setSelectStatement(SelectStatement selectStatement) {
        this.selectStatement = selectStatement;
    }
}