/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 19, 2001
 * Time: 5:45:40 PM
 * To change template for new interface use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.sql;

import freebase.api.ColumnInfo;
import freebase.api.Tuple;
import freebase.catalog.ColumnStatisticProvider;
import freebase.sql.expression.BasicCondition;
import freebase.sql.expression.ColumnExpression;
import freebase.sql.expression.CompoundCondition;
import freebase.sql.expression.LiteralExpression;
import freebase.sql.plan.QueryAnalysis;
import freebase.util.Bool;

import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;

/**
 * Perhaps add isConstant here
 */
public abstract class SqlCondition extends SqlExpression {

    /**
     * Apply Demorgans law or whatever to negate the query below
     */
    abstract public void negate();

    /**
     * Test the truth of this condition
     */
    abstract public boolean isTrue(Tuple t, int[] offsetMap)
            throws SQLException;

    /**
     * The evaluation of a conditional expression is the truth value of it
     * expressed as a literal.
     */
    public Comparable evaluate(Tuple t, int[] offsetMap) throws SQLException {
        return Bool.getBool(isTrue(t, offsetMap));
    }

    /**
     * All conditions should just collape the name and return boolean ColumnInfo.
     * TODO:  the name should not have any spaces
     */
    public ColumnInfo getColumnInfo(Tuple t, int[] offsetMap) throws SQLException {
        return new ColumnInfo(toString(), null, null, Types.BOOLEAN, true, 0, 0);
    }

    /**
     *  the overall "filtering power" of this condition.  It returns the denominator
     * used to reduce the size of query estimates.  Before this is called, all ColumnRefs must
     * have their tableOffset set.  Larger values means less final results in the query.
     */
    abstract public double getSelectivity(ColumnStatisticProvider[] tables) throws SQLException;

    /**
     * If the column list has only fields from one specific table, then this condition can
     * go into the query analysis as a single table.  Otherwise, it will go in as a post join
     * condition.  If there are no columns then the thing may go in as a condition on all tables.<BR><BR>
     *
     * Should only be called when appropriateSubConditions is true
     * TODO:  may need to consider sequences columns for this purpose
     */
    public void analyzeColumnList(QueryAnalysis qa, ArrayList columns, int curPos) {
        int numNewColumns = columns.size() - curPos;

        if(numNewColumns == 0) {
            //  TODO:  what about a < random() or something
            //  for now, add both to the table we know and the table
            qa.addConstantGlobalCondition(this);
            return;
        }

        int tableOffset = ((ColumnRef)columns.get(curPos)).getTableOffset();
        for(int i = curPos + 1; i < columns.size(); i++) {
            int newTableOffset = ((ColumnRef)columns.get(i)).getTableOffset();

            if(tableOffset != newTableOffset) {
                //  for now, abort... add the condition above the joins
                qa.addPostJoinCondition(this);
                return;
            }
        }

          //  since all table offsets are the same, this can be a single table condition
        qa.addSingleTableCondition(tableOffset, this);
    }

    /***********************************************************************
     * Static helpers for common internals.  TODO:  move these eventually to a helper
     */

    /**
     * make COLUMN=VALUE condition
     * @param column
     * @param value
     * @return
     */
    public static BasicCondition fieldEqualsLiteral(String column, Comparable value) {
        BasicCondition cond = new BasicCondition();
        cond.setLeftExpr(new ColumnExpression(column));
        cond.setRightExpr(new LiteralExpression(value, false));
        cond.setOperator(BasicCondition.EQUALS);
        return cond;
    }

    public static CompoundCondition andCondition(SqlCondition a, SqlCondition b) {
        CompoundCondition cond = new CompoundCondition(CompoundCondition.AND);
        cond.addSubExpr(b);
        return cond;
    }
}
