package freebase.sql;

import freebase.sql.expression.ExpressionVisitor;

import java.io.Serializable;
import java.sql.SQLException;

/**
 *  The root class for any object that represents an sql statement.
 * TODO:  all subclass of this should toUpperCase all appropriate strings as they
 *         come in
 */
public abstract class SqlStatement implements Serializable {
    /**
     * Called on a Prepared Statement just before execution.  Returns the highest value in the
     * array used.  The caller should verify that all of them are used.
     * TODO:  these could possibly go into DML depending upon where it is appropriate to use ? etc
     * Ie, can you say "create sequence ? start ?"  I don't think so :P
     */
    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        throw new SQLException("Statement '" + this + "' not a preparable statement");
    }

    //  TODO:  add sequence and variable binding pass!!!

    /**
     * Allows the sequence binder passed in (in the form of a visitor) to
     * visit all the appropriate parts of this statement
     * @param seqBinder the visitor that does the actual work
     * @throws SQLException
     */
    public void bindSequences(ExpressionVisitor seqBinder)
            throws SQLException {
        //  do nothing by default.
    }
}