/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 19, 2001
 * Time: 6:50:38 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.sql.expression;

import freebase.api.ColumnInfo;
import freebase.api.Tuple;
import freebase.parser.ParseException;
import freebase.sql.SqlExpression;
import freebase.sql.plan.QueryAnalysis;
import freebase.util.Helper;
import freebase.util.NumberUtil;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;


/**
 * Represents a node in a parsed algebraic expression.
 * TODO:  support IntArray, or at least create an array of objects for each type!
 * //  TODO:  add MOD
 */
public class AlgebraicExpression extends SqlExpression {
    final public static int PLUS     = 0;
    final public static int MINUS    = 1;
    final public static int CONCAT   = 2;
    final public static int MULTIPLY = 3;
    final public static int DIVIDE   = 4;

    final public String[] opSymbols = new String[] {"+", "-", "||", "*", "/"};

    /**
     * There should be one less operator than exprs
     */
    ArrayList exprs = new ArrayList();
    ArrayList operators = new ArrayList();  //  TODO:  use IntList here!

    /**
     * Set during the first operator
     */
    boolean isNumeric;

    /**
     * Is constructed with nothing and built up.
     */
    public AlgebraicExpression() { }

    /**
     * Is this the kind of expression that is just one sub expression x
     * or perhaps like (x)
     */
    public boolean isTrivial() {
        return exprs.size() == 1;
    }

    public SqlExpression getSubExpr(int i) {
        return (SqlExpression)exprs.get(i);
    }

    public void addExpression(SqlExpression expr) {
        //System.out.println(exprs.size() + ": " + expr);
        exprs.add(expr);
    }

    /**
     * TODO:  handle mixed operators
     * @param op
     * @throws ParseException
     */
    public void addOperator(int op) throws ParseException {
        if(operators.size() == 0) {
            if(op == CONCAT) {
                isNumeric = false;
            } else {
                isNumeric = true;
            }
        } else {
            if(op == CONCAT) {
                if(isNumeric) {
                    throw new ParseException("operator " + opSymbols[op] + " not a number operator");
                }
            } else {
                if(! isNumeric) {
                    throw new ParseException("operator " + opSymbols[op] + " a number operator");
                }
            }
        }

        //System.out.println(opSymbols[op]);
        operators.add(new Integer(op));
    }

    /**
     * Converts the input object to a BigDecimal, or throws
     * an exception if it is not one.
     * TODO:  sub expressions should already be BigDecimals
     */
    protected BigDecimal getNumber(SqlExpression expr, Tuple t, int[] offsetMap) throws SQLException {
        Object o = expr.evaluate(t, offsetMap);
        if(o == null) {
            return null;
        }

        if(o instanceof BigDecimal) {
            return (BigDecimal)o;
        } else if(o instanceof Float || o instanceof Double) {
            return new BigDecimal(((Number)o).doubleValue());
        }

        try{
            return BigDecimal.valueOf(((Number)o).longValue());
        }catch(ClassCastException cce) {
            throw new SQLException("value of expression [" + expr + "] '" + o + "' is not a number");
        }
    }

    /**
     * Assumed to be concatenate for now
     */
    protected Comparable evaluateString(Tuple t, int[] offsetMap) {
        return "THIS NOT WORKING RIGHT NOW";
    }

    /************************************************
     * interface methods
     */

    /**
     * Visitor Pattern.  Visit us, and if true visit children.
     * @param v
     */
    public void accept(ExpressionVisitor v) throws SQLException {
        if(v.forExpr(this)) {
            for(int i = 0; i < exprs.size(); i++) {
                getSubExpr(i).accept(v);
            }
        }
    }

    public ColumnInfo getColumnInfo(Tuple t, int[] offsetMap) throws SQLException {
        if(isNumeric) {
            return new ColumnInfo(toString(), Types.NUMERIC);
        } else {
            int sum = 0;

            //  concat, so sum up the lengths
            for(int i = 0; i < exprs.size(); i++) {
                SqlExpression x = getSubExpr(i);
                ColumnInfo colInfo = x.getColumnInfo(t, offsetMap);
                sum += colInfo.getDisplaySize();
            }

            //  TODO:  deal with maximum size here!

            return new ColumnInfo(toString(), Types.VARCHAR, sum);
        }
    }

    /**
     * TODO:  simply rewrite this horror
     *   make sure to include booleans, strings etc.  + on strings needs to work etc
     */
    public Comparable evaluate(Tuple t, int[] offsetMap) throws SQLException {
        if(operators.get(0).equals(new Integer(2))) {
            return evaluateString(t, offsetMap);
        }

        BigDecimal cur = getNumber((SqlExpression)exprs.get(0), t, offsetMap);

        for(int i = 0; i < operators.size(); i++) {
            BigDecimal next = getNumber((SqlExpression)exprs.get(i + 1), t, offsetMap);

            int op = ((Integer)operators.get(i)).intValue();

            if(op != CONCAT && (cur == null || next == null)) {
                return null;
            }

            switch(op) {
                case PLUS:     cur = cur.add(next);      break;
                case MINUS:    cur = cur.subtract(next); break;
                case MULTIPLY: cur = cur.multiply(next); break;
                case DIVIDE:   cur = NumberUtil.divide(cur, next);   break;
                case CONCAT:  throw new SQLException("need to support this one!");
            }
        }

        return cur;
    }

    /**
     * For all of the raw expressions, just use the same algorithm as buildColumnList used
     */
    public void performAnalysis(QueryAnalysis qa, boolean seekingEquiJoins,
                                ArrayList columns, ArrayList groupFunctions) throws SQLException {
        for(int i = 0; i < exprs.size(); i++) {
            getSubExpr(i).performAnalysis(qa, seekingEquiJoins, columns, groupFunctions);
        }
    }

    /**
     * Replace any expressions with group by
     */
    public void performHavingAnalysis(QueryAnalysis qa, ArrayList groupFunctions,
                                      ArrayList groupByClauses) throws SQLException {
        for(int i = 0; i < exprs.size(); i++) {
            SqlExpression oldX = (SqlExpression)exprs.get(i);
            SqlExpression newX = findGroupByItem(oldX, qa, groupFunctions, groupByClauses);

            if(oldX != newX) {
                exprs.set(i, newX);
            }
        }
    }

    public void verifyColumns(QueryAnalysis qa) throws SQLException {
        for(int i = 0; i < exprs.size(); i++) {
            getSubExpr(i).verifyColumns(qa);
        }
    }

    /*
    public void buildColumnList(ArrayList columnList) {
        for(int i = 0; i < exprs.size(); i++) {
            getSubExpr(i).buildColumnList(columnList);
        }
    }*/

    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        for(int i = 0; i < exprs.size(); i++) {
            curPos = getSubExpr(i).bindLiterals(values, curPos);
        }

        return curPos;
    }

    public void writeToBuffer(StringBuffer sb) {
        sb.append('(');
        for(int i = 0; i < operators.size(); i++) {
            SqlExpression x = (SqlExpression)exprs.get(i);
            Integer op = (Integer)operators.get(i);

            x.writeToBuffer(sb);
            sb.append(opSymbols[op.intValue()]);
        }

        SqlExpression x = (SqlExpression)exprs.get(operators.size());
        x.writeToBuffer(sb);
        sb.append(')');
    }

    /**
     * Note hashCode is not implemented yet so no hashing can be done with these.  Maybe later.
     */
    public boolean equals(Object obj) {
        //System.out.println("comparing Algebraic (" + this + ") to (" + obj + ")");
        if(! (obj instanceof AlgebraicExpression))
            return false;

        AlgebraicExpression other = (AlgebraicExpression)obj;
        if(! Helper.areListsEqual(operators, other.operators)) {
            return false;
        }

        if(! Helper.areListsEqual(exprs, other.exprs)) {
            return false;
        }

        return true;
    }
}
