/*
 * Copyright JJ Furman
 * User: user
 * Date: May 13, 2002
 * Time: 7:19:39 PM
 */
package freebase.sql.expression;

import freebase.api.ColumnInfo;
import freebase.api.Tuple;
import freebase.sql.SqlExpression;
import freebase.sql.plan.QueryAnalysis;

import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;

/**
 * Represents a late binding literal found in a prepared statement.
 */
public class BindableLiteral extends SqlExpression {
    boolean isBound = false;
    Comparable value = null;

    public void writeToBuffer(StringBuffer sb) {
        sb.append("?");
    }

    /**
     * Visitor Pattern.  Visit us, and if true visit children.
     * @param v
     */
    public void accept(ExpressionVisitor v) throws SQLException {
        v.forBindableLiteral(this);

          //  no children :)
    }

    public void verifyColumns(QueryAnalysis qa) throws SQLException {

    }

    public void performHavingAnalysis(QueryAnalysis qa, ArrayList groupFunctions,
                                      ArrayList groupByClauses) throws SQLException {
    }

    public void performAnalysis(QueryAnalysis qa, boolean seekingEquiJoins,
                                ArrayList columns, ArrayList groupFunctions) throws SQLException {
    }

    /**
     * TODO:  fill in the information more correctly
     */
    public ColumnInfo getColumnInfo(Tuple t, int[] offsetMap) throws SQLException {
        if(! isBound) {
            throw new SQLException("Literal not bound.");
        }

        if(value == null) {
            return new ColumnInfo("BindableLiteral", Types.NULL);
        }

        if(value instanceof Number) {
            return new ColumnInfo("BindableLiteral", Types.NUMERIC);
        }

        if(value instanceof String) {
            return new ColumnInfo("BindableLiteral", Types.VARCHAR, 4000);
        }

        throw new SQLException("Info on Literal of class " + value.getClass() + "  value : " +value + " not completed");
    }

    /**
     * This is it!
     */
    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        if(values.length <= curPos) {
            throw new SQLException("Not enough values to bind to prepared statement!  only " + values.length);
        }

        isBound = true;
        value = values[curPos];
        return curPos + 1;
    }

    public Comparable evaluate(Tuple t, int[] offsetMap) throws SQLException {
        if(! isBound) {
            throw new SQLException("Literal not bound.");
        }

        return value;
    }

    public void buildColumnList(ArrayList columnList) {
    }
}
