/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 19, 2001
 * Time: 7:29:42 PM
 * To change template for new class use
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.sql.expression;

import freebase.api.ColumnInfo;
import freebase.api.Tuple;
import freebase.catalog.SeqValueProvider;
import freebase.sql.ColumnRef;
import freebase.sql.SqlExpression;
import freebase.sql.plan.QueryAnalysis;
import freebase.util.NumberUtil;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;

/**
 * TODO:  deal with sequence references, and other in scope variables.
 */
public class ColumnExpression extends SqlExpression {
    ColumnRef column;

    /**
     * our relative offset into the scope of the execution.  It uses double indirection
     * to find the correct value, so technically this is the index into the offsetMap to find
     * our real position in the Tuple being passed in during evaluation.
     */
    int relativeOffset = -1;

    /**
     *  If this is a sequence, not a column, this represents the sequence.
     */
    SeqValueProvider boundSequence;

    /**
     * true if nextval
     */
    boolean isCurval = false;


    public ColumnExpression(ColumnRef column) {
        this.column = column;
    }

    public ColumnExpression(String column) {
        this.column = new ColumnRef(null, null, column);
    }

    /**
     * Returns the value of the the column.  It will convert all numbers
     * to BigDecimal.
     */
    public Comparable evaluate(Tuple t, int[] offsetMap) throws SQLException {
        //SqlValue sqlVal = t.getValue(offsetMap[ourRelColumnNum]);
        //Object val = sqlVal.getValue();

          //  if a sequence
        if(boundSequence != null) {
              //  TODO: get rid of this hack.  Should not be required to make bigDecimals!
            if(isCurval) {
                return new BigDecimal(boundSequence.getCurValue());
            } else {
                return new BigDecimal(boundSequence.getNextValue());
            }
        }

          //  no tuple we are depending on so must be a bunk reference.  (inserts and updates only)
        if(t == null) {
            throw new SQLException("Column or variable reference '" + column + "' not valid in this context");
        }

        Comparable val = t.getValue(offsetMap[relativeOffset]);

        if(val instanceof Number) {
            return NumberUtil.convertToBigDecimal((Number)val);
        }

        return val;
    }

    public ColumnInfo getColumnInfo(Tuple t, int[] offsetMap) throws SQLException {
        if(boundSequence != null) {
            return new ColumnInfo(column.toString(), Types.NUMERIC);
        }

        return t.getColumnInfo(offsetMap[relativeOffset]);
    }

    /*******************************************************
     * analysis methods
     */

    /**
     * called from the visitor.
     * @param s
     * @param isCurval
     */
    public void bindToSequence(SeqValueProvider s, boolean isCurval) {
        boundSequence = s;
        this.isCurval = isCurval;
    }

    /**
     * Called just before giving up on an unfound column.
     * @return
     */
    public boolean isBoundToSequence() {
        return boundSequence != null;
    }

    /**
     * Visitor Pattern.  Visit us, and if true visit children.
     * @param v
     */
    public void accept(ExpressionVisitor v) throws SQLException {
        v.forColumn(this);

          //  no children :)
    }

    /**
     * Determines what column this is anyway!  If it is a sequence or other value, then
     * by all means bind the value
     */
    public void performAnalysis(QueryAnalysis qa, boolean appropriateSubConditions,
                                ArrayList columns, ArrayList groupFunctions) throws SQLException {
        //  TODO:  deal with sequence values and scoped variables
        //System.out.println("relative offset " + relativeOffset);
        qa.assignTableOffset(this);
        columns.add(column);
    }

    /**
     * throw exception, since the above expression or condition should have already found us
     */
    public void performHavingAnalysis(QueryAnalysis qa, ArrayList groupFunctions,
                                      ArrayList groupByClauses) throws SQLException {
        throw new SQLException("Column " + getColumn() + " not a legal group by item");
    }


    public void setRelativeOffset(int relativeOffset) {
        //System.out.println("")
        this.relativeOffset = relativeOffset;
    }

    /**
     * Verifies that this column is valid, and assigns it a tableOffset index
     */
    public void verifyColumns(QueryAnalysis qa) throws SQLException {
        if(column.getTableOffset() != -1) {
            System.out.println("NOTE:  as expected, Column " + column + " has been verified twice, probably via a functional or other kind of expression");
        } else {
            qa.assignTableOffset(this);
        }
    }

    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        return curPos;
    }

    /**
     * @deprecated now use the visitor pattern for more reuse in children traversal and centralization of code
     * @param columnList
     */
    public void buildColumnList(ArrayList columnList) {
        relativeOffset = columnList.size();
        columnList.add(column);
    }

    public void writeToBuffer(StringBuffer sb) {
        column.writeToBuffer(sb);
    }

    /**
     * Gets the column that this is refering to.
     */
    public ColumnRef getColumn() {
        return column;
    }

    public boolean equals(Object obj) {
        if(! (obj instanceof ColumnExpression)) {
            return false;
        }

        ColumnExpression other = (ColumnExpression)obj;
        if(! getColumn().isEquivalentTo(other.getColumn())) {
            return false;
        }

        return true;
    }
}
