/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 19, 2001
 * Time: 5:46:02 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.sql.expression;

import freebase.api.Tuple;
import freebase.catalog.ColumnStatisticProvider;
import freebase.sql.SqlCondition;
import freebase.sql.plan.QueryAnalysis;
import freebase.util.OhShitException;

import java.sql.SQLException;
import java.util.ArrayList;

/**
 * TODO:  implement .equals()  for whatever reason, I'm not even sure
 */
public class CompoundCondition extends SqlCondition {
    final public static byte OR = 0;
    final public static byte AND = 1;

    byte compoundOp;
    ArrayList subExprs;

    /*********************************************
     * constructors and their helpers
     */
    public CompoundCondition(byte compoundOp) {
        this.compoundOp = compoundOp;
    }

    public CompoundCondition(byte compoundOp, SqlCondition first) {
        this.compoundOp = compoundOp;

        addSubExpr(first);
    }

    /**********************************************
     * other methods
     */

    public void addSubExpr(SqlCondition subEx) {
        if(subExprs == null)
            subExprs = new ArrayList();
        subExprs.add(subEx);
    }

    public int getNumSubConditions() {
        return subExprs.size();
    }

    public SqlCondition getSubCondition(int i) {
        return (SqlCondition)subExprs.get(i);
    }

    public byte getCompoundOp() {
        return compoundOp;
    }

    /**************************************************
     * Interface methods
     */

    /**
     * Apply DeMorgan's law
     */
    public void negate() {
          //  negate operator
        switch(compoundOp) {
            case AND: compoundOp = OR;  break;
            case OR:  compoundOp = AND; break;
            default:   throw new OhShitException("compound op: " + compoundOp + " not recognized in negate");
        }

        for(int i = 0; i < subExprs.size(); i++) {
            ((SqlCondition)subExprs.get(i)).negate();
        }
    }

    public boolean isTrue(Tuple t, int[] offsetMap) throws SQLException {
        for(int i = 0; i < subExprs.size(); i++) {
            SqlCondition subC = (SqlCondition)subExprs.get(i);
            boolean subTruth = subC.isTrue(t, offsetMap);
            if(compoundOp == AND && ! subTruth)
                return false;
            if(compoundOp == OR && subTruth)
                return true;
        }

        if(compoundOp == AND)
            return true;
        return false;  //  OR
    }

    /**
     * For AND, return the product of the children.  For OR, returns the sum
     * of the inverses... then inversed again.  The simple method.
     */
    public double getSelectivity(ColumnStatisticProvider[] tables) throws SQLException {
        if(compoundOp == AND) {
            double product = 1;
            for(int i = 0; i < getNumSubConditions(); i++) {
                product *= getSubCondition(i).getSelectivity(tables);
            }

            return product;
        } else /* compoundOp == OR */ {
            double inverseTotal = 0;

            for(int i = 0; i < getNumSubConditions(); i++) {
                inverseTotal += 1 / getSubCondition(i).getSelectivity(tables);
            }

            inverseTotal = 1 / inverseTotal;
            if(inverseTotal < 1)
                return 1;  //  ALL rows is all that we can supply, no more ;)
            return inverseTotal;
        }
    }

    /**
     * Visitor Pattern.  Visit us, and if true visit children.
     * @param v
     */
    public void accept(ExpressionVisitor v) throws SQLException {
        if(v.forCond(this)) {
            for(int i = 0; i < getNumSubConditions(); i++) {
                getSubCondition(i).accept(v);
            }
        }
    }

    /**
     * If this is AND, recurse
     */
    public void performAnalysis(QueryAnalysis qa, boolean seekingEquiJoins,
                                ArrayList columns, ArrayList groupFunctions) throws SQLException {
        int curPos = columns.size();

        for(int i = 0; i < getNumSubConditions(); i++) {
            getSubCondition(i).performAnalysis(qa, seekingEquiJoins && compoundOp == AND,
                    columns, groupFunctions);
        }

        if(seekingEquiJoins && compoundOp == OR) {
            analyzeColumnList(qa, columns, curPos);
        }
    }

    /**
     * simply recurse, no need to check since sub conditions cannot possibly be whole group by
     */
    public void performHavingAnalysis(QueryAnalysis qa, ArrayList groupFunctions,
                                      ArrayList groupByClauses) throws SQLException {
        for(int i = 0; i < getNumSubConditions(); i++) {
            getSubCondition(i).performHavingAnalysis(qa, groupFunctions, groupByClauses);
        }
    }

    /*
    public void buildColumnList(ArrayList columnList) {
        for(int i = 0; i < getNumSubConditions(); i++) {
            getSubCondition(i).buildColumnList(columnList);
        }
    }*/

    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        for(int i = 0; i < getNumSubConditions(); i++) {
            curPos = getSubCondition(i).bindLiterals(values, curPos);
        }
        return curPos;
    }

    public void verifyColumns(QueryAnalysis qa) throws SQLException {
        for(int i = 0; i < getNumSubConditions(); i++) {
            getSubCondition(i).verifyColumns(qa);
        }
    }

    /**
     * eventually this will have modes etc
     */
    public void writeToBuffer(StringBuffer sb) {
        if(subExprs == null) {
            return;
        }
        sb.append('(');

        for(int i = 0; i < subExprs.size(); i++) {
            if(i != 0) {
                switch(compoundOp) {
                    case OR:  sb.append(" OR "); break;
                    case AND: sb.append(" AND "); break;
                    default: sb.append(" !!!ILLEGAL OPERATOR!!! "); break;
                }
            }

            ((SqlCondition)subExprs.get(i)).writeToBuffer(sb);
        }

        sb.append(')');
    }

}
