package freebase.sql.expression;

import freebase.sql.SqlCondition;
import freebase.sql.SqlExpression;

import java.sql.SQLException;

/**
 * ExpressionVisitor pattern.  It should be possible to rewrite most of the query analysis using visitor patterns,
 * thus better separating the analysis from the conditions.  Frankly, I'm not sure if one is that much
 * better than the other.
 *
 * TODO:  finish this using polymorphic methods for each basic type.  Currently not done
 * Copyright Mar 14, 2004 JJ Furman
 * Time: 9:31:27 AM
 * TODO:  rename ExpressionVisitor
 * TODO:  add base visitor.  Implement most functions to the one
 * TODO:  move up?
 * TODO:  put implementations in freebase.sql.plan.visitor
 * TODO:  add push and pop to denote levels changing (or just unVisit() :) )
 */
public interface ExpressionVisitor {
    /**
     * Visit the node in question.  very generic, use specific other methods
     * to avoid ugly instanceof pattern
     * @return true if we want to be visited by the children.  (default is true)
     */
    boolean forExpr(SqlExpression exp) throws SQLException;

    boolean forCond(SqlCondition cond) throws SQLException;

    /**
     * Technically, this can be any
     * @param col
     * @return
     */
    boolean forColumn(ColumnExpression col) throws SQLException;

    boolean forFunc(FunctionalExpression func) throws SQLException;

    /**
     * Prepared Statement ?
     * @param bind
     * @return
     */
    boolean forBindableLiteral(BindableLiteral bind) throws SQLException;

    /**
     * Items found in the having clause.
     */
    boolean forGroupByItem(GroupByItem item) throws SQLException;

    /**
     * not sure this does anything anytime at all!
     * @param lit
     * @return
     */
    boolean forLiteralExpression(LiteralExpression lit) throws SQLException;

    boolean forNegative(NegativeExpression neg) throws SQLException;
}
