/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 23, 2002
 * Time: 9:47:33 AM
 */
package freebase.sql.expression;

import freebase.api.ColumnInfo;
import freebase.api.Tuple;
import freebase.sql.SqlExpression;
import freebase.sql.plan.QueryAnalysis;

import java.sql.SQLException;
import java.util.ArrayList;

/**
 * Represents a group by item found in a having clause.  The actual expression is replaced by
 * this because:
 * 1.  The expression is already calculated TODO the group by.<BR>
 * 2.  The sub items may not even be legal, ie ... group by a+b having a+b > count(*)
 */
public class GroupByItem extends SqlExpression {
    int groupByItem;

    /**
     * For finding our value in the tuple, use use this index to lookup our true offset into the tuple
     */
    int relativeOffset = -1;

    public GroupByItem(int i) {
        groupByItem = i;
    }

    public int getGroupByItem() {
        return groupByItem;
    }

    public void accept(ExpressionVisitor v) throws SQLException {
        v.forGroupByItem(this);
          //  no children
    }

    public void writeToBuffer(StringBuffer sb) {
        sb.append("[GROUP BY item #");
        sb.append(groupByItem + 1);
        sb.append("]");
    }

    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        return curPos;
    }

    public Comparable evaluate(Tuple t, int[] offsetMap) throws SQLException {
        return t.getValue(offsetMap[relativeOffset]);
    }

    public ColumnInfo getColumnInfo(Tuple t, int[] offsetMap) throws SQLException {
        return t.getColumnInfo(offsetMap[relativeOffset]);
    }

    public void setRelativeOffset(int relativeOffset) {
        this.relativeOffset = relativeOffset;
    }

    public void buildColumnList(ArrayList columnList) {
        relativeOffset = columnList.size();
        columnList.add(this);
    }

    public void performAnalysis(QueryAnalysis qa, boolean seekingEquiJoins,
                                ArrayList columns, ArrayList groupFunctions) throws SQLException {
        throw new SQLException("performAnalysis should not be being called on GroupByItem");
    }

    public void performHavingAnalysis(QueryAnalysis qa, ArrayList groupFunctions,
                                      ArrayList groupByClauses) throws SQLException {
        throw new SQLException("performHavingAnalysis should not be being called on GroupByItem");
    }

    public void verifyColumns(QueryAnalysis qa) throws SQLException {
        throw new SQLException("verifyColumns should not be being called on GroupByItem");
    }

    public boolean equals(Object obj) {
        if(! (obj instanceof GroupByItem)) {
            return false;
        }

        GroupByItem gby = (GroupByItem)obj;
        return gby.getGroupByItem() == getGroupByItem();
    }
}
