package freebase.sql.expression;

import freebase.api.Tuple;
import freebase.catalog.ColumnStatisticProvider;
import freebase.sql.SqlCondition;
import freebase.sql.SqlExpression;
import freebase.sql.plan.QueryAnalysis;
import freebase.util.SqlTypes;

import java.sql.SQLException;
import java.util.ArrayList;

/**
 * TODO:  currently this is not used, but I have a sense it would be easier optimize if this was.
 * Eventually think about using this.  For now the condition a in (x,y,z) becomes a=x or a=y or a=z.
 * TODO:  when changing this, rewrite the nice handy static metho here
 *
 * select * from tab where name IN (blah, blah, blah).  This is essentially a big a=b or a=c or a=d
 * Copyright 2004 JJ Furman
 * Time: 8:06:14 PM
 */
public class InListCondition extends SqlCondition {
    boolean isNegated = false;

    SqlExpression left;

    ArrayList inList = new ArrayList();

    public InListCondition(SqlExpression left, ArrayList inList, boolean negate) {
        this.left = left;
        this.inList = inList;
        this.isNegated = negate;
    }

    public void negate() {
        isNegated = ! isNegated;
    }

    public boolean isTrue(Tuple t, int[] offsetMap)
            throws SQLException {
        boolean truth = false;

        Comparable leftValue = left.evaluate(t, offsetMap);
        if(leftValue != null) {
            for(int i = 0; i < inList.size(); i++) {
                Comparable inValue = getInExpr(i).evaluate(t, offsetMap);
                if(inValue != null && SqlTypes.compare(leftValue, inValue) == 0) {
                    truth = true;
                    break;
                }
            }
        }

        return isNegated ? ! truth : truth;
    }

    public SqlExpression getInExpr(int pos) {
        return (SqlExpression)inList.get(pos);
    }

    public double getSelectivity(ColumnStatisticProvider[] tables) throws SQLException {
        return .2;  //  TODO:  very arbitrary HACK!  Have no idea if this is even called
    }

    public void accept(ExpressionVisitor v) throws SQLException {
        if(v.forCond(this)) {
            left.accept(v);

            for(int i = 0; i < inList.size(); i++) {
                getInExpr(i).accept(v);
            }
        }
    }

    public void writeToBuffer(StringBuffer sb) {
        left.writeToBuffer(sb);
        sb.append(" IN (");
        for(int i = 0; i < inList.size(); i++) {
            if(i != 0) {
                sb.append(",");
            }
            getInExpr(i).writeToBuffer(sb);
        }
        sb.append(")");
    }

    /*
    public void buildColumnList(ArrayList columnList) {
        left.buildColumnList(columnList);

        for(int i = 0; i < inList.size(); i++) {
            getInExpr(i).buildColumnList(columnList);
        }
    }*/

    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        curPos = left.bindLiterals(values, curPos);
        for(int i = 0; i < inList.size(); i++) {
            curPos = getInExpr(i).bindLiterals(values, curPos);
        }
        return curPos;
    }

    public void performAnalysis(QueryAnalysis qa, boolean seekingEquiJoins,
                                ArrayList columns, ArrayList groupFunctions) throws SQLException {
        int curPos = columns.size();

        left.performAnalysis(qa, seekingEquiJoins, columns, groupFunctions);

        for(int i = 0; i < inList.size(); i++) {
            getInExpr(i).performAnalysis(qa, seekingEquiJoins, columns, groupFunctions);
        }

        if(seekingEquiJoins) {
            analyzeColumnList(qa, columns, curPos);
        }
    }

    public void performHavingAnalysis(QueryAnalysis qa, ArrayList groupFunctions,
                                      ArrayList groupByClauses) throws SQLException {
        left.performHavingAnalysis(qa, groupFunctions, groupByClauses);

        for(int i = 0; i < inList.size(); i++) {
            getInExpr(i).performHavingAnalysis(qa, groupFunctions, groupByClauses);
        }
    }

    public void verifyColumns(QueryAnalysis qa) throws SQLException {
        left.verifyColumns(qa);

        for(int i = 0; i < inList.size(); i++) {
            getInExpr(i).verifyColumns(qa);
        }
    }

/*

    public void writeToBuffer(StringBuffer sb) {
        left.writeToBuffer(sb);
        sb.append(opSymbols[operator]);
        if(right != null)
            right.writeToBuffer(sb);
    }*/
}
