/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 19, 2001
 * Time: 7:28:11 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.sql.expression;

import freebase.api.ColumnInfo;
import freebase.api.Tuple;
import freebase.parser.ParseException;
import freebase.sql.SqlExpression;
import freebase.sql.plan.QueryAnalysis;
import freebase.util.Helper;
import freebase.util.NumberUtil;
import freebase.util.SqlTypes;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;

/**
 * Eventually this should contain more than just two types of values, indeed perhaps
 * boolean, date etc?
 */
public class LiteralExpression extends SqlExpression {
    boolean isNumeric = false;
    String stringValue = null;
    BigDecimal numericValue = null;

    /**
     * Value is to be parsed.
     */
    public LiteralExpression(String value) throws freebase.parser.ParseException {
        try{
        if(value == null || "null".equalsIgnoreCase(value)) {
            return;  //  for now, just leaving it null makes it so
        } if(value.charAt(0) != '\'') {
            numericValue = new BigDecimal(value);
            isNumeric = true;
        } else {
            stringValue = SqlTypes.deSqlize(value);
        }
        }catch(Exception e) {
            throw new ParseException("Value [" + value + "] not valid!");
        }
    }

    public LiteralExpression(Comparable val, boolean ignoredUsedForParanoia) {
        if(val instanceof String) {
            stringValue = (String)val;
        } else if(val instanceof Number) {
            isNumeric = true;
            numericValue = NumberUtil.convertToBigDecimal((Number)val);
        }
    }

    public Comparable evaluate(Tuple t, int[] offsetMap) {
        return getValue();
    }

    /**
     * Accept visitor.
     * @param v
     */
    public void accept(ExpressionVisitor v) throws SQLException {
        v.forLiteralExpression(this);
          //  no children
    }

    /**
     * No unbound literal here!  :)
     */
    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        return curPos;
    }

    /**
     * What this is depends upon the type.  NULL values
     */
    public ColumnInfo getColumnInfo(Tuple t, int[] offsetMap) throws SQLException {
        if(numericValue == null && stringValue == null) {
            return new ColumnInfo("NULL", Types.NULL);
        } else if(numericValue != null) {
            return new ColumnInfo(numericValue.toString(), Types.NUMERIC);
        } else if(stringValue != null) {
            return new ColumnInfo(stringValue, Types.VARCHAR);
        }

        throw new SQLException("getColumnInfo not supported with this data type");
    }

    public Comparable getValue() {
        if(numericValue != null)
            return numericValue;
        return stringValue;
    }

    public void performAnalysis(QueryAnalysis qa, boolean appropriateSubConditions,
                                ArrayList columns, ArrayList groupFunctions) throws SQLException {
        return;
    }

    public void performHavingAnalysis(QueryAnalysis qa, ArrayList groupFunctions,
                                      ArrayList groupByClauses) throws SQLException {
        return;
    }

    public void verifyColumns(QueryAnalysis qa) throws SQLException {
        return;
    }

    public boolean isNumeric() {
        return isNumeric;
    }

    public boolean isNull() {
        return getValue() == null;
    }

    public void buildColumnList(ArrayList columnList) {
        return;
    }

    public void writeToBuffer(StringBuffer sb) {
        sb.append(evaluate(null, null));
    }

    public boolean equals(Object obj) {
        if(! (obj instanceof LiteralExpression)) {
            return false;
        }

        LiteralExpression other = (LiteralExpression)obj;
        if(numericValue != null) {
            if(! numericValue.equals(other.numericValue)) {
                return false;
            }
        } else {
            if(! Helper.areEqual(stringValue, other.stringValue)) {
                return false;
            }
        }

        return true;
    }
}
