/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 19, 2001
 * Time: 7:16:20 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.sql.expression;

import freebase.api.ColumnInfo;
import freebase.api.Tuple;
import freebase.sql.SqlExpression;
import freebase.sql.plan.QueryAnalysis;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.ArrayList;

public class NegativeExpression extends SqlExpression {
    SqlExpression expr;

    public NegativeExpression(SqlExpression expr) {
        this.expr = expr;
    }

    /**
     *  TODO: This implementation is hoaky and slow.  It should use custom introspection
     * to derive the type.
     */
    public Comparable evaluate(Tuple t, int[] offsetMap) throws SQLException {
        Comparable val = expr.evaluate(t, offsetMap);
        if(val == null)
            return null;

        if(val instanceof BigDecimal) {
            return ((BigDecimal)val).negate();
        } else if(val instanceof Float) {
            return new Float(-((Float)val).floatValue());
        } else if(val instanceof Double) {
            return new Double(-((Double)val).doubleValue());
        } else if(val instanceof Long) {
            return new Long(-((Long)val).longValue());
        } else if(val instanceof Integer) {
            return new Integer(-((Integer)val).intValue());
        } else if(val instanceof Short) {
            return new Short((short) -((Short)val).shortValue());
        } else if(val instanceof Byte) {
            return new Byte((byte) -((Byte)val).byteValue());
        } else {
            throw new SQLException("value of expression '" + expr + "' not a BigDecimal but: " + val.getClass());
        }
    }

    public void accept(ExpressionVisitor v) throws SQLException {
        if(v.forNegative(this)) {
            expr.accept(v);
        }
    }

    public int bindLiterals(Comparable[] values, int curPos) throws SQLException {
        return expr.bindLiterals(values, curPos);
    }

    /**
     * The information about this column is not effected by a negation
     */
    public ColumnInfo getColumnInfo(Tuple t, int[] offsetMap) throws SQLException {
        return expr.getColumnInfo(t, offsetMap);
    }

    public void performAnalysis(QueryAnalysis qa, boolean appropriateSubConditions,
                                ArrayList columns, ArrayList groupFunctions) throws SQLException {
        expr.performAnalysis(qa, appropriateSubConditions, columns, groupFunctions);
    }

    /**
     *
     */
    public void performHavingAnalysis(QueryAnalysis qa, ArrayList groupFunctions,
                                      ArrayList groupByClauses) throws SQLException {
        expr = findGroupByItem(expr, qa, groupFunctions, groupByClauses);
    }

    public void verifyColumns(QueryAnalysis qa) throws SQLException {
        expr.verifyColumns(qa);
    }

    /*
    public void buildColumnList(ArrayList columnList) {
        expr.buildColumnList(columnList);
    }*/

    public void writeToBuffer(StringBuffer sb) {
        sb.append('-');
        expr.writeToBuffer(sb);
    }


    public boolean equals(Object obj) {
        if(! (obj instanceof NegativeExpression)) {
            return false;
        }

        NegativeExpression other = (NegativeExpression)obj;
        return expr.equals(other.expr);
    }
}
