/*
 * Copyright JJ Furman
 * User: user
 * Date: May 10, 2002
 * Time: 12:03:29 PM
 */
package freebase.sql.function.aggregation;

import freebase.sql.function.Function;
import freebase.sql.function.FunctionFactory;
import freebase.util.NumberUtil;

import java.math.BigDecimal;
import java.sql.SQLException;

public class AvgFunctionFactory implements FunctionFactory {
    public Function getFunction() {
        return new AvgFunction();
    }
}

/**
 * The avg is just the sum / count
 */
class AvgFunction extends SumFunction {
    long count;

    /**
     * Note sure if f(null) is always null, but for now
     * I assume not!
     */
    public void addValue(Comparable val) throws SQLException {
        if(val == null) {
            return;
        }

        super.addValue(val);

        count++;
    }
    
    public Comparable getResult() throws SQLException {
        Comparable c = super.getResult();

        if(c == null) {
            return null;
        }

        if(c instanceof BigDecimal) {
            BigDecimal bigD = (BigDecimal)c;

            return NumberUtil.divide(bigD, BigDecimal.valueOf(count));
        } else {
            throw new SQLException("Currently do not support anything than BigDecimal in the top of the avg function.  TODO: " + c);
        }
    }

    public void reset() throws SQLException {
        super.reset();

        count = 0;
    }

    /**
     * Only count accepts full tuples for now.
     */
    public boolean acceptsFullTuples() throws SQLException {
        return false;
    }
}