package freebase.sql.function.aggregation;

import freebase.sql.function.AggregationFunction;
import freebase.sql.function.Function;
import freebase.sql.function.FunctionFactory;

import java.sql.SQLException;

/**
 * CHOOSE_MAX(a,b) finds the highest value of a, and returns the corresponding b
 * This is VERY useful for implementing ordered logs and grabbing the most recent
 * version.
 * 
 * TODO:  implement ChooseMin
 * Copyright JJ Furman,  Jun 8, 2005  9:47:04 PM
 */
public class ChooseMaxFunctionFactory implements FunctionFactory {
    public Function getFunction() {
        return new ChooseMaxFunction();
    }
}


class ChooseMaxFunction extends AggregationFunction {
    Comparable max = null;
    Comparable maxElem = null;

    /**
     * Note sure if f(null) is always null, but for now
     * I assume not!
     */
    public void addValue(Comparable val) throws SQLException {
        throw new SQLException("Illegal to get just one element here-> internal error");
    }

    public void addValues(Comparable[] val) throws SQLException {
        if(val[0] == null) {
            return;
        }

        if(max == null) {
            max = val[0];
        } else {
            if(max.compareTo(val[0]) < 0) {
                max = val[0];
                maxElem = val[1];
            }
        }
    }

    public Comparable getResult() throws SQLException {
        return maxElem;
    }

    public void reset() throws SQLException {
        max = null;
    }

    /**
     * Only count accepts full tuples for now.
     */
    public boolean acceptsFullTuples() throws SQLException {
        return false;
    }

    public boolean supportsValues(int number) throws SQLException {
        return number == 2;
    }
}

