/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 17, 2002
 * Time: 3:22:44 PM
 */
package freebase.sql.function.aggregation;

import freebase.sql.function.AggregationFunction;
import freebase.sql.function.Function;
import freebase.sql.function.FunctionFactory;

import java.sql.SQLException;

/**
 * Creates CountFunctions.  Needs to create a new one each time.
 */
public class CountFunctionFactory implements FunctionFactory {
    public Function getFunction() {
        return new CountFunction();
    }
}

/**
 * Keeps track of counts.  Pretty Straightforward.
 */
class CountFunction extends AggregationFunction {
    long count = 0;

    /**
     * Note sure if f(null) is always null, but for now
     * I assume not!
     */
    public void addValue(Comparable val) throws SQLException {
        if(val != null)
            count++;
    }

    public Comparable getResult() throws SQLException {
        return new Long(count);
    }

    public void reset() throws SQLException {
        count = 0;
    }

    public boolean acceptsFullTuples() throws SQLException {
        return true;
    }
}
