/**
 * Copyright 2004 JJ Furman
 * Time: 8:27:18 AM
 */
package freebase.sql.function.aggregation;

import freebase.sql.function.AggregationFunction;
import freebase.sql.function.Function;
import freebase.sql.function.FunctionFactory;

import java.sql.SQLException;

/**
 * GROUP_CONCAT function
 * Take a list of values in a group and turn them into a list like
 * BingoGame1      b1,o71,n32  etc
 * The purpose of this is to be able to quickly probe a debug a database without having
 * to resort to lots of nonesense.  The idea came from Rob Warnock.
 * TODO:  support setting separator char, options for null being missing etc
 * TODO:  limit the buffer, say 2K to 64K?  (append "...");
 */
public class GroupConcatFunctionFactory implements FunctionFactory {
    public Function getFunction() {
        return new GroupConcatFunction();
    }
}


class GroupConcatFunction extends AggregationFunction {
    StringBuffer sb = new StringBuffer();
    boolean first = true;

    /**
     * Note sure if f(null) is always null, but for now
     * I assume not!
     */
    public void addValue(Comparable val) throws SQLException {
        if(first) {
            first = false;
        } else {
            sb.append(',');
        }

        sb.append(val);
    }

    public Comparable getResult() throws SQLException {
        return sb.toString();
    }

    public void reset() throws SQLException {
        sb = new StringBuffer();
        first = true;
    }

    /**
     * Only count accepts full tuples for now.
     */
    public boolean acceptsFullTuples() throws SQLException {
        return false;
    }
}

