/*
 * Copyright JJ Furman
 * User: user
 * Date: May 15, 2002
 * Time: 1:08:42 PM
 */
package freebase.sql.function.standard;

import freebase.api.ColumnInfo;
import freebase.sql.function.StandardFunction;

import java.math.BigDecimal;
import java.sql.SQLException;

/**
 * Returns the absolute value of the input
 */
public class AbsFunction extends StandardFunction {
    public ColumnInfo getColumnInfo(ColumnInfo[] inputs) throws SQLException {
        if(inputs.length != 1) {
            throw new SQLException("ABS function only takes one argument");
        }

        return inputs[0];
    }

    public Comparable evaluate(Comparable[] input) throws SQLException {
          //  validate inputs
        if(input.length != 1) {
            throw new SQLException("ABS function only takes one argument");
        }

        if(input[0] == null) {
            return null;
        }

        if(! (input[0] instanceof Number)) {
            throw new SQLException("ABS function only takes a numeric argument");
        }
        Number n = (Number)input[0];

        if(n instanceof Float) {
            return new Float(Math.abs(n.floatValue()));
        }

        if(n instanceof Double) {
            return new Double(Math.abs(n.doubleValue()));
        }

        if(n instanceof Byte) {
            return new Byte((byte)Math.abs(n.byteValue()));
        }

        if(n instanceof Short) {
            return new Short((short)Math.abs(n.shortValue()));
        }

        if(n instanceof Integer) {
            return new Integer(Math.abs(n.intValue()));
        }

        if(n instanceof Long) {
            return new Long(Math.abs(n.longValue()));
        }

        if(n instanceof BigDecimal) {
            return ((BigDecimal)n).abs();
        }

        throw new SQLException("Class of input " + n + "  (" + n.getClass() + ") not recognized");
    }
}