/**
 * Copyright 2004 JJ Furman
 * Time: 7:08:38 PM
 */
package freebase.sql.function.standard;

import freebase.api.ColumnInfo;
import freebase.sql.function.Function;
import freebase.sql.function.FunctionFactory;
import freebase.sql.function.StandardFunction;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;

/**
 * This function returns the current time.  In the default system, this is mapped to SYSDATE().
 * It will return the same time for the entire query.
 */
public class CurrentTime implements FunctionFactory {
    public Function getFunction() {
        return new CurrentTimeInstance();
    }
}

class CurrentTimeInstance extends StandardFunction {
    Timestamp time = new Timestamp(System.currentTimeMillis());

    public ColumnInfo getColumnInfo(ColumnInfo[] inputs) throws SQLException {
        if(inputs.length > 0) {
            throw new SQLException("Current time function (SYSDATE) does not take arguments");
        }
        return new ColumnInfo("SYSDATE()", Types.TIMESTAMP);
    }

    public Comparable evaluate(Comparable[] input) throws SQLException {
        return time;
    }
}