/**
 * Copyright 2004 JJ Furman
 * Time: 6:45:07 AM
 */
package freebase.sql.plan.visitor;

import freebase.sql.SqlCondition;
import freebase.sql.SqlExpression;
import freebase.sql.expression.*;

import java.util.ArrayList;

/**
 * Encapsulates an ArrayList of ColumnRef, and the visitor code to grab them from a parse tree.
 * TODO:  maybe rename ColumnListAndBuilder ?  (longish)
 * Builds the list of columns used in this expression by
 * appending them to the end of this list.  This
 * will be used later to evaluate the expression.  When evaluated
 * a corresponding int[] offset map will be supplied so that the column
 * can find its value in the current tuple.
 */
public class ColumnList extends ArrayList implements ExpressionVisitor {
    public boolean forColumn(ColumnExpression col) {
        if(col.isBoundToSequence()) {
            return true;
        }

        col.setRelativeOffset(size());
        add(col.getColumn());
        return true;
    }

    public boolean forGroupByItem(GroupByItem item) {
        item.setRelativeOffset(size());
        add(item);
        return true;
    }

    public boolean forFunc(FunctionalExpression func) {
        if(func.getFunction().isAggregation()) {
            func.setRelativeOffset(size());
            add(func);
            return false;
        } else {
            return true;
        }
    }

    /*******************************************
     * The rest are boilerplate
     */

    /**
     *
     * @param exp
     * @return
     */
    public boolean forExpr(SqlExpression exp) {
        return true;
    }

    public boolean forCond(SqlCondition cond) {
        return true;
    }

    public boolean forBindableLiteral(BindableLiteral bind) {
        return true;
    }

    public boolean forLiteralExpression(LiteralExpression lit) {
        return true;
    }

    public boolean forNegative(NegativeExpression neg) {
        return true;
    }
}
