/**
 * Copyright 2004 JJ Furman
 * Time: 3:00:57 PM
 */
package freebase.sql.plan.visitor;

import freebase.catalog.Catalog;
import freebase.catalog.Schema;
import freebase.catalog.SeqValueProvider;
import freebase.catalog.Sequence;
import freebase.engine.Transaction;
import freebase.sql.expression.ColumnExpression;

import java.sql.SQLException;

/**
 * A visitor that binds certain columns as sequences before (or after) they are assumed to be columns.
 * TODO:  deal with sequence binding and use locks!
 */
public class SequenceBinder extends VisitorBase {
    Transaction t;
    Catalog cat;

    public SequenceBinder(Transaction t, Catalog cat) {
        this.t = t;
        this.cat = cat;
    }

    public boolean forColumn(ColumnExpression col) throws SQLException {
        boolean isNextVal = col.getColumn().getColumn().equals("NEXTVAL");
        boolean isCurVal = col.getColumn().getColumn().equals("CURRVAL");

        if(isNextVal || isCurVal) {
            String schema = col.getColumn().getSchema();
            if(schema == null) {
                schema = t.getUser();
            }
            Schema s = cat.getSchema(schema);
            Sequence seq = s.getSequence(col.getColumn().getTable(), false);  //  expect null if not there
            if(seq != null) {
                col.bindToSequence(new SeqValueProvider(t, seq), isCurVal);

                //System.out.println("Potential sequence value: " + col.getColumn());
            }
        }

        return true;
    }
}
