package freebase.sql.query;

import freebase.api.ColumnInfo;
import freebase.api.TupleIterator;
import freebase.server.ExecutionPlan;
import freebase.sql.SqlCondition;
import freebase.util.OhShitException;

import java.sql.SQLException;

/**
 * An optimization that return only the metadata but no data.  This is used when the query
 * processor finds deterministicly false clauses like 1=0
 * Copyright 2004 JJ Furman
 * Time: 9:27:57 PM
 */
public class NoRowsMetaOnly implements TupleIterator {
    TupleIterator source;

    SqlCondition falseCondition;

    public NoRowsMetaOnly(TupleIterator source, SqlCondition breakingCondition) {
        this.source = source;
        this.falseCondition = breakingCondition;
    }

    public void reset() throws SQLException {
    }

    public boolean next() throws SQLException {
        return false;
    }

    public void close() throws SQLException {
        source.close();
    }

    public ExecutionPlan getExecutionPlan() throws SQLException {
        return new ExecutionPlan("No Rows Filter (condition false: " + falseCondition.toString() + ")", source.getExecutionPlan());
    }

    public Comparable getValue(int i) throws SQLException {
        return null;
    }

    public void setValue(int i, Comparable value) throws SQLException {
    }

    public int getNumColumns() throws SQLException {
        return source.getNumColumns();
    }

    public String getColumnName(int i) throws SQLException {
        return source.getColumnName(i);
    }

    public ColumnInfo getColumnInfo(int i) throws SQLException {
        return source.getColumnInfo(i);
    }

    public boolean isUpdateable() throws SQLException {
        return source.isUpdateable();
    }

    public void deleteTuple() throws SQLException {
        throw new OhShitException("It can't be possible to delete when there is no tuple :P");
    }

}
