/*
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: Sep 10, 2001
 * Time: 10:00:24 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.sql.query;

import freebase.api.ColumnInfo;
import freebase.api.LockableTupleIterator;
import freebase.api.TupleIterator;
import freebase.api.ViewInconsistentException;
import freebase.server.ExecutionPlan;
import freebase.sql.SqlCondition;

import java.sql.SQLException;

/**
 * Selects only a subset of tuples.  This is currently required on top of an updating view, even if there is no
 * real filter.
 * <p>
 * NOT TO BE CONFUSED WITH: FilterTupleIterator
 */
public class RowFilterIterator implements TupleIterator {
    //  MODE:  SELECT  -> simple query
    //  UPDATE   ->  double query
    //  SELECT_FOR_UPDATE  ->  locks the row explicitly in the log
    //       (ie, since no

    TupleIterator it;
    SqlCondition condition;
    int[] offsetMap;
    boolean forUpdate;

    /**
     * Construct a new filter iterator.  Condition = null means all rows.
     * TODO:  can we get the forUpdate flag from below?  I think so.
     * TODO:  change param it to be of a specific type, like lockable tuple iterator or something:
     *      methods like lockRowTemp, unlockRowTemp, lockRowPerm etc
     * TODO:  change param forUpdate to have read, write, read_for_update
     */
    public RowFilterIterator(TupleIterator it, SqlCondition c, int[] offsetMap, boolean forUpdate) throws SQLException {
        this.it = it;
        this.condition = c;
        this.offsetMap = offsetMap;
        this.forUpdate = forUpdate;
    }

    /**
     * Reset the underlying iterator
     */
    public void reset() throws SQLException {
        it.reset();
    }

    /**
     * Find next until the tuple matches condition.
     */
    public boolean next() throws SQLException {
        while(it.next()) {
            if(condition == null || condition.isTrue(it, offsetMap)) {
                if(! forUpdate) {
                    return true;
                } else {
                    //  TODO:  Should we lock the row if the data
                    //    is not going to be part of our query?
                    //  TEST ORACLE!!!

                    /**
                     * maybe:  query if row is already locked
                     * if so, then no need to query again at all
                     * lock row (this tells the underneath system that we can read the most current state)
                     * if the condition is no longer true, then release the lock
                     */

                      //  will throw some exception if is no longer there!  (minor View or major Can'tSerialize)
                    ((LockableTupleIterator)it).lockRow();

                    //  TODO:  check the view here (or have the view checked inside lock already)!
                    //    if done inside lock already, then it can just go ahead and writeout the lock message too

                      //  still needed even in the case of dirty read mode... it still may have changed!
                    //System.out.print("checking:");
                    if(condition == null || condition.isTrue(it, offsetMap)) {
                        //System.out.println(" ok");
                          //  todo:  writeout lock to log if serializable
                        return true;
                    }
                    //System.out.println(" failed");
                    ((LockableTupleIterator)it).unlockRow();

                      //  Since the criteria we want is no longer valid, the data must have changed.
                      //  thus, we don't have the most recent copy.  We *may* have read behind a more recent
                      //  version of another row (the change may have not even been there yet when we were)
                    throw new ViewInconsistentException();

                        //  release the lock -> we know this is safe because
                        //  we didn't have the lock before, cuz if we did
                        //  we would not have seen different data the second
                        //  time through the expression

                    //  TODO:  if "select for update", make lock permanent by writing a log entry
                }
            }
        }

        return false;
    }

    public Comparable getValue(int i) throws SQLException {
        return it.getValue(i);
    }

    public void setValue(int i, Comparable value) throws SQLException {
        it.setValue(i, value);
    }

    public void deleteTuple() throws SQLException {
        it.deleteTuple();
    }

    public void close() throws SQLException {
        it.close();
    }

    public int getNumColumns() throws SQLException {
        return it.getNumColumns();
    }

    public String getColumnName(int i) throws SQLException {
        return it.getColumnName(i);
    }

    public ColumnInfo getColumnInfo(int i) throws SQLException {
        return it.getColumnInfo(i);
    }

    public boolean isUpdateable() throws SQLException {
        return it.isUpdateable();
    }

    public ExecutionPlan getExecutionPlan() throws SQLException {
        return new ExecutionPlan("Row Selection [" + condition + "]", it.getExecutionPlan());
    }
}
