/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 23, 2002
 * Time: 5:26:43 PM
 */
package freebase.sql.query;

import freebase.util.OhShitException;

import java.util.ArrayList;
import java.util.Collections;

/**
 * Current implementation just uses a simple memory structure, but a future implementation
 * will need to use a memory structure.<BR><BR>
 *
 * If the number of values was large but the number of distinct values was small it might
 * be more efficient to use a tree, since only one value would need to reside in memory at
 * a time.
 *
 * NOTE:  currently, nulls are not accepted as values.
 */
public class SingleValueUniqueness {
    ArrayList data = new ArrayList();

    boolean hasBeenSorted = false;

    int pos = 0;

    Object curVal = null;

    /**
     * Do Nothing for now.
     */
    public SingleValueUniqueness() { }

    /**
     * Reset this space for fresh reuse
     */
    public void reset() {
        data.clear();
        hasBeenSorted = false;
        pos = 0;
        curVal = null;
    }

    /**
     *  TODO:  make the tests assertions
     */
    public void addValue(Comparable c) {
        if(c == null) {
            throw new OhShitException("null not a legal value to add");
        }

        if(hasBeenSorted)
            throw new OhShitException("after a getNextUnique is called, addValue cannot be called unless reset is called first!");

        data.add(c);
    }

    /**
     * Return true if there is more data
     */
    public boolean nextUnique() {
        if(! hasBeenSorted) {
            Collections.sort(data);
            hasBeenSorted = true;
        }

        while(pos < data.size()) {
            Object nextVal = data.get(pos++);
            if(! nextVal.equals(curVal)) {
                curVal = nextVal;
                return true;
            }
        }

        return false;
    }

    /**
     * This implementation returns these values in sorted order, but it
     * is not clear to me that it would really matter.  This will return null
     * whe
     */
    public Comparable getValue() {
        return (Comparable)curVal;
    }

    public void close() {
        data = null;
    }
}
