package freebase.store;

import java.io.IOException;

/**
 *  Represents an abstract storage entity, linearly addressable and blocked.
 * This is like a file.
 */
abstract public class Entity {
    /**
     *  read one
     *  TODO:  maybe not create an array each time here :)
     *  be only part of the Block!
     */
    void read(int fromPage, Block b) throws IOException {
        read(fromPage, new Block[] {b});
    }
    
    /**
     *  write one.  Is obligated to set the dirty flag to false if
     *  it was successful
     */
    void write(Block b) throws IOException {
        write(new Block[] {b});
    }
    
    /**
     *  read many
     */
    abstract void read(int fromPage, Block[] b) throws IOException;
    
    /**
     *  write many
     */
    abstract void write(Block[] b) throws IOException;
    
    /**
     *  returns the size in the number of bytes
     */
    abstract int getSize() throws IOException;
    
    /**
     *  makes sure that all of the writes are secured on durable storage
     */
    abstract void sync() throws IOException;
    
    /**
     *  requests that the system grow by at least numBlocks number of
     *  additional blocks.  Not sure if this method needs to be published to 
     *  the outside since
     */
    abstract void grow(int numBlocks, Block bufferBlockSpace) throws IOException;
    
    /**
     *  allocates a new page dedicated to the database entity.
     *  takes the Block object passed in and initializes it structures
     *  so that.  Not sure this is correct so it is taken out
     */
    //  abstract void newBlock(Block b) throws IOException;
}