package freebase.store;


import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

/**
 *  Entity manager keeps track of the object level storage abstraction
 *  layer.  Each entity represents a table or index and is addressable
 *  within that context.
 */
public class EntityManager {
    BufferPool theOnlyPool;
    ArrayList entities = new ArrayList();
    
    int blockSize;
    File homeDir;
    
    /**
     *  This class will be changing a lot over time, especially
     *  the way that it maps entities onto actual files.  This will
     *  eventually be done with tablespaces (or disk sets) but for
     *  now will be ok this way.
     */
    public EntityManager(String directory, int blockSize, int numBlocks) throws IOException {
        this.blockSize = blockSize;
        
        //  TODO:  remove this when the bug in array list is fixed
        for(int i = 0; i < 50; i++) {
            entities.add(null);
        }
        
        try{
            homeDir = new File(directory);
            String[] files = homeDir.list();
            
            for(int i = 0; i < files.length; i++) {
                // System.out.println("found: " + files[i]);
                int entityId = Integer.parseInt(files[i].substring(6, 8));
                entities.ensureCapacity(1);
                entities.set(entityId, new BasicFileEntity(directory + "/" + files[i], blockSize, entityId));
            }
            
        }catch(IOException e) {
            
        }
        
        
        theOnlyPool = new BufferPool(numBlocks, blockSize, this);
    }

    public Entity getEntity(int entityId) {
        return (Entity)entities.get(entityId);
    }
    
    public BufferPool getBufferPoolForEntity(int entityId) {
        return theOnlyPool;
    }
    
    public int newEntity() {
        return -1;
    }
    
    public int getNumEntities() {
        return entities.size();
    }
}
