/*
 * Copyright JJ Furman
 * User: user
 * Date: May 1, 2002
 * Time: 2:31:39 PM
 */
package freebase.test;

import freebase.StartDaemon;
import freebase.util.Helper;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Random;


/**
 * Repeats transactions over and over again.  For now does it all in one thread,
 * but eventually this will be a test harness for lots of concurrent threads.<BR><BR>
 *
 * Final test should vary individual statement executions with periodic whole transactions
 * to test more view oriented stuff.  Also, do some queries, preferably with zero prefetch and
 * delay.
 *
 * OLD NOTES!  ha
 */
public class BasicRepeatTest {
    final static int numTests = 1000;
    final static int range = 10;
    final static int PAUSE = 100;

    static Random r = new Random(5);

    static boolean printOutput = false;
      //  when slow motion is true, the system pauses between each statement a little
    static boolean slowMotion = false;

    public static void main(String[] args) throws Throwable {
        StartDaemon.startDatabaseServer().createTestSchema();


        Connection con = JDBCTest.getF3Connection();
        con.setAutoCommit(false);

        int i = 0;
        while(true) {
            long startTime = System.currentTimeMillis();
            doOneBatch(con);
            System.out.print("#" + (++i) + ": ");
            Helper.printElapsed(System.currentTimeMillis() - startTime);

            Helper.sleep(500);
        }

        //con.close();

        //
    }

    public static void doOneBatch(Connection con) {
        for(int i = 0; i < numTests; i++) {
            doSomeWork(con);
        }
    }

    /**
     * Does a unit of work, sometimes
     */
    public static void doSomeWork(Connection con) {
        try{
            doOneOperation(con);
            doOneOperation(con);
            doOneOperation(con);
            doOneOperation(con);

            if(r.nextBoolean()) {
                debug("COMMIT");
                con.commit();
            } else {
                debug("ROLLBACK");
                con.rollback();
            }
        }catch(SQLException s) {
            s.printStackTrace();
        }
    }

    /**
     * either does an insert, delete or update
     */
    public static void doOneOperation(Connection con) throws SQLException {
        Statement st = con.createStatement();
        int op = r.nextInt(3);
        int val = r.nextInt(range);

        String statement = null;
        switch(op) {
            case 0: statement = "insert into test values (" + val + ", 'val" + val + "')"; break;
            case 1: statement = "delete from test where id=" + val; break;
            case 2: statement = "update test set name='up " + val + "' where id=" + val; break;
        }

        if(statement != null) {
            debug(statement);
            st.execute(statement);
        }

        st.close();

        if(slowMotion) {
            Helper.sleep(PAUSE);
        }
    }

    public static void debug(String message) {
        if(printOutput) {
            System.out.println(message);
        }
    }
}
