/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 24, 2002
 * Time: 5:14:44 PM
 */
package freebase.test;

import freebase.StartDaemon;
import freebase.util.Helper;

import java.sql.*;

/**
 * Both a place for initial testing of JDBC code and a place to store
 */
public class JDBCTest {
    /**
     *
     */
    public static void main(String[] args) throws Throwable {
        //Props.setInstance(new Props(StartDaemon.DEFAULTS_FILENAME, "config/durable.prop"));
        StartDaemon.startDatabaseServer().createTestSchema();
        //Connection con = getRemoteF3Connection();
        Connection con = getLocalF3Connection();

        con.setAutoCommit(false);
        long time = System.currentTimeMillis();

        //testBatch(con);
        testSelfInsert(con);

        time = System.currentTimeMillis() - time;
        System.out.println("Took: " + time);
        System.gc();
        System.out.println("Used memory: " + (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / 1024 + "K");
        con.close();
        System.exit(0);
    }

    public static void allocateExtraConnections() throws Throwable {
        Connection con = getLocalF3Connection();
        con = getLocalF3Connection();
        con = getLocalF3Connection();
        con = getLocalF3Connection();
    }

    public static void testSelfInsert(Connection c) throws SQLException {
        Statement st = c.createStatement();
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //20
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //40
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //80
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //160
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //320
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //640
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //  1280
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //  2560
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //  5120
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //  10240
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //  20480
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //  40960
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  //  81920
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  // 163840 total
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  // 327680 total
        st.executeUpdate("insert into nums select a+b, a+b+1 from nums");  // 655360 total  50M
        //st.executeUpdate("insert into nums select * from nums");  // 1.2M total   850M
        //st.executeUpdate("insert into nums select * from nums");  // 2.4M total   200M

        st.close();
        c.commit();

    }

    /**
     * Uses a table:  create table speed_insert (a int, b int, c int, d varchar(255), e varchar(255));
     * @param c
     */
    public static void testBatch(Connection c) throws Throwable {
        PreparedStatement ps = c.prepareStatement("insert into speed_insert values (?,?,?,?,?)");

        for(int i = 0; i < 1000; i++) {
            for(int j = 0; j < 300; j++) {
                ps.setInt(1, i);
                ps.setInt(2, j);
                ps.setInt(3, 99999);
                ps.setString(4, "01234567890123456789" + i);
                ps.setString(5, "01234567890123456789" + j);
                ps.addBatch();
            }
            ps.executeBatch();
        }

        ps.close();
        c.commit();
    }

    public static void someCrazyTest() throws Throwable {
        //continueTest();

          //  create the local database
        //StartDaemon.startDatabaseServer().createTestSchema();

        //  choose the database here
        Connection con = getRemoteF3Connection();

        try{
            PreparedStatement ps = con.prepareStatement("select ? from nums");
            ps.setInt(1, -1);
            ResultSet rs = ps.executeQuery();

            printMetaData(rs);
            printResults(rs);

            //Statement st = con.createStatement();
            //ResultSet rs = st.executeQuery("select * from nums where id=?");
            //ResultSet rs = st.executeQuery("select distinct a, count(*) from test2 group by a order by count(*)+a");

            long startTime = System.currentTimeMillis();

            /*
            System.out.println("Fetch is " + rs.getFetchSize());
            rs.setFetchSize(400);
            long count = 0;
            long total = 0;
            while(rs.next()) {
                count++;
                total += rs.getInt(1);
                rs.getObject(2);
                //System.out.println(rs.getObject(1) + "    " + rs.getObject(2));
            }
*/

            rs.close();
            ps.close();

            Helper.printElapsed(System.currentTimeMillis() - startTime);
            //System.out.println("Num Rows: " + count + "  total: " + total);
        }finally{
            con.close();
        }

        //System.exit(0);
    }


    public static void printResults(ResultSet rs) throws SQLException {
        rs.setFetchSize(400);

        ResultSetMetaData meta = rs.getMetaData();
        for(int i = 0; i < meta.getColumnCount(); i++) {
            System.out.print(meta.getColumnName(i + 1) + "     ");
        }
        System.out.println();

        while(rs.next()) {
            for(int i = 0; i < meta.getColumnCount(); i++) {
                System.out.print(rs.getObject(i + 1) + "     ");
            }

            System.out.println();
        }
    }

    public static void printMetaData(ResultSet rs) throws SQLException {
        ResultSetMetaData meta = rs.getMetaData();
        for(int i = 0; i < meta.getColumnCount(); i++) {
            System.out.println("TYPE @" + i + ":  " + meta.getColumnType(i+1) + " prec: "+ meta.getPrecision(i+1) +  " scale : " + meta.getScale(i+1));
            System.out.println("    CAT:'" +  meta.isNullable(i+1) + "'  '" + meta.getColumnLabel(i+1) + "' '" + meta.getTableName(i+1) + "' '" +meta.isSearchable(i+1) + "'");
        }
    }

    /********************************************************************************
     * A bunch of methods for grabbing connections to specific databases
     */

    /**
     * Grab a connection to local oracle database
     */
    public static Connection getOracleConnection() throws Throwable {
        Class.forName("oracle.jdbc.OracleDriver");
        return DriverManager.getConnection("jdbc:oracle:thin:jj/password@192.168.123.123:1521:TEST");
    }

    public static Connection getF3Connection() throws Throwable {
        Class.forName("freebase.jdbc.F3Driver");
        return DriverManager.getConnection("jdbc:freebase://localhost", "JJ", "yeah right");
    }

    public static Connection getLocalF3Connection() throws Throwable {
        Class.forName("freebase.jdbc.F3Driver");
        return DriverManager.getConnection("jdbc:freebase:local", "JJ", "yeah right");
    }

    public static Connection getRemoteF3Connection(String host) throws Throwable {
        return DriverManager.getConnection("jdbc:freebase://" + host, "JJ", "yeah right");
    }

    public static Connection getRemoteF3Connection() throws Throwable {
        //
        Class.forName("freebase.jdbc.F3Driver");
        //return DriverManager.getConnection("jdbc:freebase://192.168.123.123", "JJ", "yeah right");
        return getRemoteF3Connection("192.168.123.130");
    }
}
