/*
 * Copyright JJ Furman
 * User: user
 * Date: Mar 15, 2002
 * Time: 8:48:31 AM
 */
package freebase.test;

import freebase.util.DataConverter;
import sun.misc.Unsafe;
import sun.reflect.Reflection;

import java.nio.ByteBuffer;


/**
 * We may eventually need to use the Unsafe access mechanism
 */
public class ScratchEntry {
    final public static int NUM_TESTS = 10000000;
    int a = 500000;
    int b = 6;

    /**
     * Space to put random tests
     */
    public static void main(String[] args) {
        interruptTest();
    }


    /*********************************************************************88888
     * some random tests
     */

    public static void unsafeTest() {
        Unsafe unsafe = Unsafe.getUnsafe();
        System.out.println(System.getProperty("user.home"));

        System.out.println(speedUnsafeArray(unsafe));
        System.out.println(speedJavaArray());


        byte[] javaBuf = new byte[8192];
        javaBuf[0] = 0;
        javaBuf[1] = 1;
        javaBuf[2] = 2;
        javaBuf[3] = 3;

        for(int i = 0; i < 10; i++) {
            System.out.println("offset " + i + " " + unsafe.getByte(javaBuf, i + 12));
        }

        //unsafe.
        System.out.println(unsafe.arrayBaseOffset(byte[].class));
        System.out.println(unsafe.arrayIndexScale(byte[].class));
    }

    public static long speedUnsafeArray(Unsafe unsafe) {
        byte[] javaBuf = new byte[8192];
        unsafe.putLong(javaBuf, 12, 0);

        long sum = 0;

        long time = System.currentTimeMillis();
        for(int i = 0; i < NUM_TESTS; i++) {
            //sum += unsafe.getLong(javaBuf, 12 + (i % 4));
            sum += getLong(unsafe, javaBuf, i % 4);
            //System.out.println(x +   " "  + y);
        }

        System.out.println("sum is " + sum);
        return System.currentTimeMillis() - time;
    }

    public static long getLong(Unsafe unsafe, byte[] a, int pos) {
        return unsafe.getLong(a, 12 + pos);
    }

    public static long speedByteBuffer() {
        ByteBuffer buf = ByteBuffer.allocateDirect(8192);
        buf.putLong(0, -1);

        long sum = 0;
        long time = System.currentTimeMillis();
        for(int i = 0; i < NUM_TESTS; i++) {
            sum += buf.getLong(i % 4);
            //System.out.println(x +   " "  + y);
        }

        System.out.println("sum is " + sum);
        return System.currentTimeMillis() - time;
    }


    public static long speedJavaArray() {
        byte[] javaBuf = new byte[8192];
        DataConverter.writeLong(javaBuf, 0, -1);
        long sum = 0;

        long time = System.currentTimeMillis();
        for(int i = 0; i < NUM_TESTS; i++) {
            sum += DataConverter.readLong(javaBuf, i % 4);
            //System.out.println(x +   " "  + y);
        }

        System.out.println("sum is " + sum);
        return System.currentTimeMillis() - time;
    }


    /**
     * A neat internal sun thing
     */
    public static void testReflection() {
        int i = 0;
        boolean notDone = true;
        while(notDone) {
            Class c = Reflection.getCallerClass(i);

            System.out.println(c);
            if(c == null)
                notDone = false;
            i++;
        }
    }


    static Object lock = new Object();
    public static void interruptTest() {
        Thread.currentThread().interrupt();

        synchronized(lock) {
            try{
            lock.wait();
            }catch(InterruptedException ie) {
                ie.printStackTrace();
            }
        }
    }
}

//class WorkerThread