/*
 * Created by IntelliJ IDEA.
 * User: administrator
 * Date: Nov 24, 2001
 * Time: 9:54:55 PM
 * To change template for new class use
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.test;

import freebase.api.TupleIterator;
import freebase.parser.SqlParser;
import freebase.server.ActionResult;
import freebase.server.DBConnection;
import freebase.server.DatabaseNode;
import freebase.sql.SelectStatement;
import freebase.sql.SqlStatement;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.HashMap;

/**
 * Encapsulates an execution of a script.  Is also the access for
 * getting the execution results
 */
public class ScriptExecution {
    HashMap connections = new HashMap();

    File script;

    public ScriptExecution(File script) {
        this.script = script;
    }

    /**
     * Executes the script given in a new database.
     * TODO:  deal with deadlock timeout
     */
    public void execute() throws Throwable {
        ConnectionThread cur = null;

          //  init
        DatabaseNode dbNode = new DatabaseNode();
        dbNode.createTestSchema();

          //  open file
        BufferedReader in = new BufferedReader(
              new InputStreamReader(new FileInputStream(script)));

        SqlParser parser = new SqlParser(in);


        Object nextCommand = null;
        while((nextCommand = parser.testScriptFileCommand()) != null) {
            if(nextCommand instanceof String) {
                cur = (ConnectionThread)connections.get(nextCommand);
                if(cur == null) {
                    cur = new ConnectionThread(DatabaseNode.getLocalDBConnection("JJ", ""));
                    cur.start();
                    connections.put(nextCommand, cur);
                }
            } else if(nextCommand instanceof SqlStatement) {
                cur.processStatement((SqlStatement)nextCommand);
            } else {
                System.out.println("Could not process script command:  '" + nextCommand + "'");
            }
        }
    }
}


/**
 * Represents on thread of execution on a transaction
 */
class ConnectionThread extends Thread {
    DBConnection con;

    Object conLock = new Object();

    SqlStatement nextStatement;

    SQLException  lastException;
    TupleIterator lastQueryResults;
    ActionResult  lastActionResult;


    public ConnectionThread(DBConnection con) {
        this.con = con;
    }

    public void processStatement(SqlStatement st) {
        synchronized(conLock) {
            while(nextStatement != null) {
                try{
                conLock.wait();  //  with time limit?
                }catch(InterruptedException ie) {ie.printStackTrace();}
            }

            nextStatement = st;
            conLock.notify();

              //  definitely with time limit,
              //  long enough to be sure it got in!
            try{
            conLock.wait(2000);
            }catch(InterruptedException ie) {ie.printStackTrace();}
        }
    }

    /**
     * This is the thread that does the actual work to the connection, locking
     * and all
     */
    public void run() {
        SqlStatement toProcess = null;
        while(true) {
            synchronized(conLock) {
                while(nextStatement == null) {
                    try{
                    conLock.wait();
                    }catch(InterruptedException ie) {ie.printStackTrace();}
                }

                toProcess = nextStatement;
                sendToConnection(toProcess);

                nextStatement = null;
                conLock.notify();
            }
        }
    }

    private void sendToConnection(SqlStatement st) {
        System.out.println(this + " is processing " + st);
        try{
            if(st instanceof SelectStatement) {
                lastQueryResults = con.executeQuery((SelectStatement)st);
            } else {
                lastActionResult = con.executeAction(st);
            }
        }catch(SQLException s) {
            lastException = s;
        }catch(RemoteException e) {
            e.printStackTrace();
        }
    }
}