package freebase.test.regression;

import freebase.test.JDBCTest;
import freebase.test.TestFailedException;
import freebase.test.TestSession;

import java.sql.Connection;
import java.util.ArrayList;

/**
 * Subclass of Regression test that handles DB Specific session and cleanup handling.  Just create them
 * via the createSession API and this will automatically clean them up for you.
 *
 * TODO:  rename this to MultiSessionTest
 */
public abstract class MultiSessionTest extends RegressionTest {
    TestSession cleanupSession;


    public MultiSessionTest() {
          //  create a session but don't put it in the sessions arraylist
        cleanupSession = _createSession();
    }


    /*******************************************************************
     *
     */

    ArrayList sessions = new ArrayList();

    /**
     * just creates the session.  Generally shouldn't call this, so its private for now.
     */
    private TestSession _createSession() {
        try{
            Connection con = JDBCTest.getLocalF3Connection();
            con.setAutoCommit(false);

            return new TestSession(con);
        }catch(Throwable s) {
            throw new TestFailedException("Test failed in creating a session: " + s.getMessage());
        }
    }

    /**
     * Allocates a session
     */
    public TestSession createSession() {
        TestSession s = _createSession();
        sessions.add(s);
        return s;
    }

    /**
     * Closes all connections obtained.  Could eventually leave them and just pool
     */
    public void teardown() throws Throwable {
          //  close all
        for(int i = 0; i < sessions.size(); i++) {
            ((TestSession)sessions.get(i)).close();
        }

        sessions.clear();

        //Helper.sleep(10000000);
        afterEachTestCleanup(cleanupSession);

        if(cleanupSession.isClosed()) {
            throw new TestFailedException("testSpecificCleanup should not close cleanupSession!");
        }
    }

    /**
     * don't close the cleanup session!!!
     * @param cleanupSession
     */
    public abstract void afterEachTestCleanup(TestSession cleanupSession);
}
