package freebase.test.regression;

import freebase.test.TestSession;
import freebase.util.ErrorCode;


/**
 * Test Unique keys.  TODO:  lots more!!!!!
 */
public class UniqueKeyTest extends MultiSessionTest {

      //  some tests can be repeated
    int NUM_REPEATS = 0;

    /**
     * do some basic single session stuff
     */
    public void testBasics() {
        TestSession a = createSession();

        a.execute("update names set id=2 where id=1");
        a.checkException();
        a.execute("update names set id=2 where id=2 or id=3");
        a.checkException();

        a.execute("insert into names values (6)");
        a.ensureIsNotBlocked();
        a.execute("insert into names values (1)");
        a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);

        a.execute("savepoint pre_insert_5");
        a.execute("insert into names values (5)");
        a.execute("insert into names values (5)");
        a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
        a.execute("rollback to pre_insert_5");

        a.execute("insert into names values (5)");
        a.execute("insert into names values (5)");
        a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);

        a.execute("insert into names values (6)");
        a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
    }

    /**
     * consider getting rid of this
     *
    public void testHammer() {
        TestSession a = createSession();
        TestSession b = createSession();

        a.execute("insert into names values (1, 'ff')");

        for(int i = 0; i < 30; i++) {
            //Helper.sleep(1);
            b.execute("select count(*) from names");
            b.checkSingleReturnValue(4);
        }
    }*/

    public void testInsert() {
        //  duh!
    }

    public void testDelete() {
        TestSession a = createSession();
        TestSession b = createSession();

          //  delete, then change mind
        a.execute("delete from names where id=2");
            b.execute("insert into names values (5)");
            b.ensureIsNotBlocked();
            b.execute("insert into names values (2)");
            b.ensureIsBlocked();
        a.rollback();
            b.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
            b.ensureIsNotBlocked();
            b.rollback();

          //  delete, then commit (allow b to proceed)
        a.execute("delete from names where id=2");
            b.execute("insert into names values (5)");
            b.ensureIsNotBlocked();
            b.execute("insert into names values (2)");
            b.ensureIsBlocked();
        a.commit();
            b.checkNoException();
            b.rollback();
    }

    public void testShift() {
        TestSession a = createSession();
        TestSession b = createSession();

        for(int i = 0; i < 3; i++) {
            a.execute("update names set id=id+1");
                b.execute("update names set id=10 where id=4");
            a.execute("select sum(id) from names");
            a.checkValues(0, new int[] {14}, 0);
            a.execute("update names set id=id-1");
            a.ensureIsNotBlocked();
                b.ensureIsBlocked();
            a.rollback();
                b.checkUpdateCount(1);
                b.execute("select * from names where id > 6");
                b.checkCount(1);
                b.rollback();
        }

        a.execute("update names set id=id+1");
            b.execute("update names set id=5 where id=3");
        a.commit();
            b.ensureIsNotBlocked();
             //  TODO:  this fails specifically because our MVC works different than oracles (inferior for the moment)
            //b.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);

    }

    public void testDeleteExisting() {
        TestSession a = createSession();
        TestSession b = createSession();

        for(int i = 0; i < NUM_REPEATS; i++) {
              //  delete w/ rollback hole
            a.execute("delete from names where id=2");
                b.execute("insert into names values (6)");
                b.execute("insert into names values (2)");
            a.rollback();
                b.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
                b.rollback();
        }

          //  delete don't rollback hole
        a.execute("delete from names where id=2");
            b.execute("insert into names values (6)");
            b.execute("insert into names values (2)");
        a.commit();
            b.execute("select sum(id) from names");
            b.checkValues(0, new int[] {16}, 0);
            b.commit();
    }

    public void testTricky() {
        TestSession a = createSession();
        TestSession b = createSession();

        for(int i = 0; i < NUM_REPEATS; i++) {
              //  reinsert after another has inserted and waited
            a.execute("insert into names values (6)");
                b.execute("insert into names values (6)");
            a.execute("insert into names values (6)");
            a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
            a.rollback();
                b.execute("select count(*) from names where id > 5)");
                b.checkCount(1);
                b.rollback();
        }

          //  tricky ordering problem:
        a.execute("select * from NAMES");
        a.checkCount(4);
        a.execute("insert into NAMES values (6)");
        a.execute("savepoint bob");
        a.execute("insert into NAMES values (8)");
        a.execute("insert into NAMES values (7)");
        a.execute("rollback to bob");
            b.execute("insert into NAMES values (7)");
            b.execute("select * from NAMES");
            b.checkCount(5);
        a.execute("insert into NAMES values (7)"); //  (should block)
        a.ensureIsBlocked();
        b.rollback();
        a.ensureIsNotBlocked();
        a.rollback();
    }

    /**
     * test closing the connecting in various ways
     */
    public void testClosing() {
        TestSession a = createSession();
        TestSession b = createSession();
        TestSession c = createSession();

        a.execute("insert into names values (5, 'You have no chance to survive')");
            b.execute("insert into names values (5, 'You have no chance to survive')");
                c.execute("insert into names values (5, 'DOGS!')");
                c.ensureIsBlocked();
            b.close();    //  <--  testing that this close causes correct behavior (rollback)
        a.rollback();
                c.execute("select * from names where id=5");
                c.checkValues(1, new Object[] {"DOGS!"}, 0);
                c.rollback();

        b = createSession();
        a.execute("insert into names values (77, 'You have no chance to survive')");
            b.execute("insert into names values (77, 'DOGS!')");
        a.close();  //  <--  testing that this close causes correct behavior (rollback)
            b.execute("select * from names where id=77");
            b.checkValues(1, new Object[] {"DOGS!"}, 0);
            b.rollback();

        a = createSession();
        a.execute("insert into names values (5, 'You have no chance to survive')");
            b.execute("insert into names values (5, 'You have no chance to survive')");
                c.execute("insert into names values (5, 'DOGS!')");
        a.commit();
            b.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
                c.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);

        a.execute("insert into names values (50, 'You have no chance to survive')");
            b.execute("insert into names values (50, 'You have no chance to survive')");
                c.execute("insert into names values (50, 'DOGS!')");
        a.rollback();
            b.ensureIsNotBlocked();
                c.ensureIsBlocked();
            b.commit();
                c.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
    }

    /**
     * some of these tests may be redundant, but better to be safe
     */
    public void testUpdateRevert() {
        TestSession a = createSession();
        TestSession b = createSession();

        a.execute("insert into names values (100)");
        a.execute("savepoint THERE");
        a.execute("update names set id=500 where id=2");
        a.execute("rollback to THerE");
            b.execute("insert into NAMES values (500)");
            b.ensureIsNotBlocked();
        a.execute("insert into NAMES values (500)");
            b.commit();
        a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
    }

    /**
     *
     */
    public void testBasicDeadlock() {
        TestSession a = createSession();
        TestSession b = createSession();

        a.execute("insert into names values (100)");
            b.execute("insert into names values (101)");
            b.execute("insert into names values (100)");
        a.execute("insert into names values (101)");

        TestSession.checkOneThrewException(a, b);
    }

    /**
     * in this case an edge gets put in after
     */
    public void testAnotherDeadlock() {
        TestSession a = createSession();
        TestSession b = createSession();
        TestSession c = createSession();

        a.execute("insert into names values (100)");
            b.execute("insert into names values (101)");
            b.execute("insert into names values (100)");
                c.execute("insert into names values (102)");
                c.execute("insert into names values (100)");
        a.rollback();
            b.execute("insert into names values (102)");

        TestSession.checkOneThrewException(b, c);
    }

    /**
     * test uniqueness for keys with multiple values.  Ensure that multi-value keys can go into the index
     * as long as they are not all null
     */
    public void testMultiKeys() {
        TestSession a = createSession();

        a.execute("insert into unique_nums values (1,2)");
        a.execute("insert into unique_nums values (1,1)");
        a.execute("insert into unique_nums values (1,null)");
        a.execute("insert into unique_nums values (null,2)");
        a.execute("insert into unique_nums values (null,2)");
        a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
        a.execute("insert into unique_nums values (1,1)");
        a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);
        a.execute("insert into unique_nums values (null, 1)");
        a.execute("insert into unique_nums values (null,2)");
        a.checkException(ErrorCode.UNIQUE_KEY_VIOLATION);

          //  silly row, I admit, but legal in most db's so we should test it
        a.execute("insert into unique_nums values (null, null)");
        a.execute("insert into unique_nums values (null, null)");
        a.ensureIsNotBlocked();
    }

    /**
     * TODO:  do all the following
    1.  Test null values in the index
    2.  Test multi value unique keys
    3.  Test create index with bad columns (duh :)  )



update test set id=3 where id=1;
commit;

insert into test values (2);
rollback to bob;
				insert into test values (2);
insert into test values (2);  (should block)



     */

    /*********************************************************************
     *
     */

    /**
     * same as ConcurrencyTest for now
     * @param cleanupSession
     */
    public void afterEachTestCleanup(TestSession cleanupSession) {
        cleanupSession.execute("DELETE FROM NAMES");
        cleanupSession.execute("INSERT INTO NAMES VALUES (1, 'JJ')");
        cleanupSession.execute("INSERT INTO NAMES VALUES (2, 'Pranav')");
        cleanupSession.execute("INSERT INTO NAMES VALUES (3, 'Tito')");
        cleanupSession.execute("INSERT INTO NAMES VALUES (4, 'Smoove B')");
        cleanupSession.commit();
    }

    /**
     * main entry point
     */
    public static void main(String[] args) throws Throwable {
        RegressionTest.runTestAndExit(UniqueKeyTest.class);  //, "testShift");
    }
}
