/**
 * Copyright 2004 JJ Furman
 */
package freebase.test.regression;

import freebase.test.TestSession;
import freebase.util.ErrorCode;

/**
 * Contains data validation tests.
 * All Validation tests should go ahead and rollback before proceeding, simplifying everything.  This
 * is done automatically by the MultiSessionTest framework.
 *<p>
 * To test:
 *
 * update nums set a='bob' where 1=0
 * update nums set a=? where ? is illegal values
 * update nums set a=1 where a=?  where ? is illegal values
 * update nums set a=1 where 'bob'=1
 * update
 *
 * delete from nums where b='ddd'
 * delete from nums where b=?
 * delete from nums where 'a' = 2
 *
 * select * from names where id=name  (duh)
 *
 * select * from something where date='2004-24-13'
 * select * from something where '2004-24-13'=date
 * select * from something where '2004-2bad'=date
 * select * from something where '2004-2bad'=date
 *
 * allow:
 * update names set name=5 where id=1;  ???
 *
 * TODO:  create a table with entries of every type!
 *
 * @author JJ Furman
 */
public class ValidationTest extends MultiSessionTest {

    public void testInserts() {
        TestSession a = createSession();

          //  test putting 'bob' into number
        a.execute("insert into nums values ('bob', 1)");
        a.checkException(ErrorCode.EXPRESSION_NOT_NUMBER);

          //  test putting 'bbb' into date
        a.execute("insert into ind_nums (c,d) values ('bbb', 'bbb')");
        a.checkException(ErrorCode.EXPRESSION_NOT_DATE);

          //  test putting null into c
        a.execute("insert into ind_nums (d) values ('2002-10-10')");
        a.checkException(ErrorCode.COLUMN_NOT_NULLABLE);

          //  test dates working
        a.execute("insert into ind_nums (c,d) values ('dummy', '2002-10-10')");
        a.checkNoException();

        //  TODO:  prepared and batch statement
    }

    public void testUpdate() {
        TestSession a = createSession();

        a.execute("update IND_NUMS set c=null");
        a.checkException(ErrorCode.COLUMN_NOT_NULLABLE);

        //  TODO:  prepared and batch statement
    }


    public void testSelect() {
        //  TODO:  implement condition tests, simple at first to get basic pattern
        //  implement join test type comparison etc
    }

    public void afterEachTestCleanup(TestSession cleanupSession) {
        //  since Validation Tests rollback their work, this will
    }

    /**
     * main entry point
     */
    public static void main(String[] args) throws Throwable {
        RegressionTest.runTestAndExit(ValidationTest.class);
    }
}
