/**
 * Copyright JJ Furman
 * Sep 22, 2005  8:38:39 PM
 */
package freebase.test.system;

import freebase.StartDaemon;
import freebase.jdbc.F3Connection;
import freebase.test.JDBCTest;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Basic load test.
 */
public class BasicLoad {
    public static int NUM_THREADS = 300;
    public static int TIME_INTERVAL = 1000;
    public static int NUM_HISTOGRAM_BUCKETS = 24;

    static Object incLock = new Object();
    static int numOps = 0;
    //  first bucket is <=1, next <=2, <=4
    static int[] timeHistogram = new int[NUM_HISTOGRAM_BUCKETS];

    public static void main(String[] args) throws Throwable {
        StartDaemon.startDatabaseServer().createTestSchema();

        for(int i = 0; i < NUM_THREADS; i++) {
            new WorkerThread(i).start();
        }

        while(true) {
            try{
                Thread.sleep(TIME_INTERVAL);
            }catch(InterruptedException ie) { }

            int numCopy;

            synchronized(incLock) {
                numCopy = numOps;
                numOps = 0;
            }

            System.out.println("Did " + numCopy + " in " + TIME_INTERVAL);
            // print histogram.  My this is too much code :(

            for(int i = 0; i < NUM_HISTOGRAM_BUCKETS; i++) {
                if(i != 0) {
                    System.out.print(", ");
                }
                System.out.print(timeHistogram[i]);
            }
            System.out.println();
        }
    }

    static class WorkerThread extends Thread {
        int id;

        public WorkerThread(int id) {
            this.id = id;
        }

        public void run() {
            Connection  c = null;
            try {
                c = (F3Connection)JDBCTest.getRemoteF3Connection();
                c.setAutoCommit(false);
                Statement st = c.createStatement();
                //System.out.print(id + " ");

                //Thread.sleep(20000);
                while(true) {
                    long time = System.currentTimeMillis();
                    try{
                        Thread.sleep(10);
                    }catch(InterruptedException ie) { }
                    ResultSet rs = st.executeQuery("select * from NAMES");
                    while(rs.next()) {
                        for(int i = 0; i < rs.getMetaData().getColumnCount(); i++) {
                            //System.out.println(rs.getObject(i + 1));
                        }
                    }
                    rs.close();
                    time = System.currentTimeMillis() - time;

                    int timeBin = 0;
                    while(time > 1) {
                        timeBin++;
                        time >>= 1;
                    }

                    synchronized(incLock) {
                        numOps++;
                        timeHistogram[timeBin]++;
                    }
                }

                //st.close();
            }catch(Throwable t) {
                t.printStackTrace();
            } finally {
                if(c != null) {
                    try{
                        c.close();
                    }catch(SQLException s) {
                        s.printStackTrace();
                    }
                }
            }
        }
    }
}
