/**
 * Copyright 2004 JJ Furman
 * Time: 12:01:59 AM
 */
package freebase.test.system;

import freebase.StartDaemon;
import freebase.jdbc.F3Connection;
import freebase.test.JDBCTest;
import freebase.test.TestFailedException;
import freebase.util.ErrorCode;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Aggressively inserts into the same unique value over and over!  Tests unique locks pretty well.
 * TODO:  figure out why snapshot old is happening... like ever!
 * TODO:  figure out why view got a null transaction that time
 * TODO:  test plan
 *
 * run server in IDEA in debug mode
 * run test on command line against remote server (change code).
 * get test to fail.  pause it.
 * Execute a query directly on the server to see the data.
 *
 *
 * Alternatively, hestitate dramatically (many minutes) while rolling back to see if that is a problem
 *
 * See about doing an index dump.... visualizing advanced structures can be key!
 */
public class MassiveUniqueInsert {

    public static void main(String[] args) throws Throwable {
        StartDaemon.startDatabaseServer().createTestSchema();

        createTable();

        executeTest(1, 1);
        executeTest(1, 100);
        executeTest(10, 10);
        executeTest(100, 2);

        int count = 0;
        while(true) {
            long time = System.currentTimeMillis();
            executeTest(10, 100);
            System.out.println(" #" + (count++) + "  took " + (System.currentTimeMillis() - time));
            System.gc();
        }
    }

    public static void executeTest(int numCycles, int numThreads) throws Throwable {
        Thread[] threads = new Thread[numThreads];

        for(int i = 0; i < threads.length; i++) {
            threads[i] = new InsertThread(numCycles);
        }

        for(int i = 0; i < threads.length; i++) {
            threads[i].start();
        }

        for(int i = 0; i < threads.length; i++) {
            threads[i].join();
        }

        //  ExternalStats.dumpNumStatementRollbacks();

        Connection c = JDBCTest.getLocalF3Connection();
        Statement st = c.createStatement();
        ResultSet rs = st.executeQuery("select count(*) from parallel");
        while(rs.next()) {
            long count = rs.getLong(1);
            if(count != 1) {
                throw new TestFailedException("OH NO!  count is " + count + " but expected 1");
            }
        }

        rs.close();

        st.execute("truncate table parallel");

        st.close();
        c.close();

        System.out.println("parallel insert test w/ " + numCycles + " values and " + numThreads + " threads passed");

    }

    /**
     *
     */
    public static void createTable() throws Throwable {
        Connection c = JDBCTest.getLocalF3Connection();
        Statement st = c.createStatement();
        st.execute("create table parallel (id int)");
        st.execute("create unique index random_name_here on parallel (id)");
        st.close();
        c.close();
    }

    static class InsertThread extends Thread {
        int numCycles;

        public InsertThread(int numCycles) {
            this.numCycles = numCycles;
        }

        public void run() {
            F3Connection c = null;
            try {
                c = (F3Connection)JDBCTest.getLocalF3Connection();
                c.setAutoCommit(false);
                Statement st = c.createStatement();

                  //  for each number, increment it
                for(int i = 0; i < numCycles; i++) {
                    ResultSet rs = st.executeQuery("select * from parallel where id=0");
                    int count = 0;
                    while(rs.next()) {
                        count++;
                    }
                    rs.close();

                    if(count > 1) {
                        System.out.println("We see more than 1 value!!!");
                        //DebugLog.printLog(c.getPhysicalConnection());
                        rs = st.executeQuery("select * from parallel where id=0");
                        count = 0;
                        while(rs.next()) {
                            count++;
                        }
                        rs.close();

                        if(count > 1) {
                              //  TODO:  STOP THE THREADS OR SOMETHING????  is such a thing possible?
                            System.out.println("STILL!!!");
                        }


                        System.exit(1);
                    }
                    //DebugLog.clear(c.getPhysicalConnection());

                    //  TODO:  try with inserts only, and lots of queries to verify uniqueness
                    try{
                        int numEffected = st.executeUpdate("insert into parallel (id) values (0)");
                    }catch(SQLException s) {
                        if(s.getErrorCode() == ErrorCode.UNIQUE_KEY_VIOLATION) {
                            //System.out.println("acceptable unique key found  threadid:" + threadID);
                        } else {
                            throw s;
                        }
                    }

                    c.commit();
                }

                st.close();
            }catch(Throwable t) {
                t.printStackTrace();
            } finally {
                if(c != null) {
                    try{
                        c.close();
                    }catch(SQLException s) {
                        s.printStackTrace();
                    }
                }
            }
        }
    }
}
