/**
 * Copyright 2004 JJ Furman
 * Time: 9:04:15 PM
 */
package freebase.test.system;

import freebase.StartDaemon;
import freebase.jdbc.F3Connection;
import freebase.server.DBConnectionImpl;
import freebase.test.JDBCTest;
import freebase.test.TestFailedException;
import freebase.util.DebugLog;
import freebase.util.ErrorCode;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;

/**
 * Increments a set of values.  Should test concurrent row locks pretty well.
 * TODO:  come up with a mechanism for reporting problems and generally running these in batch
 */
public class ParallelInc {

    public static void main(String[] args) throws Throwable {
        //Props.setInstance(new Props(StartDaemon.DEFAULTS_FILENAME, "config/durable.prop"));
        StartDaemon.startDatabaseServer().createTestSchema();

        createTable();

        if(false) {
            while(true) {
                executeTest(2, 2);
            }
        }

        executeTest(1, 1);
        executeTest(1, 10);
        executeTest(1, 100);
          //System.exit(0);
        executeTest(10, 1);
        executeTest(10, 2);
        executeTest(10, 5);
        //System.exit(0);
        executeTest(10, 10);

        try{
            while(true) {
                long time = System.currentTimeMillis();
                executeTest(100, 30);  //fails after a while
                //executeTest(1, 200);
                System.out.println("Took: " + (System.currentTimeMillis() - time) + "  " + (new Date()));
                DebugLog.clearAll();
            }
        }catch(Throwable t) {
            t.printStackTrace();
        }

        /*

        Connection c = JDBCTest.getLocalF3Connection();
        Statement st = c.createStatement();
        ResultSet rs = st.executeQuery("select * from parallel where val=29");
        while(rs.next()) {
            System.out.println("rowdata: " + rs.getObject(1) + "   " + rs.getObject(2));
        }
        rs.close();
        st.close();
        c.close();
        */
    }

    public static void executeTest(int numValues, int numThreads) throws Throwable {
        Thread[] threads = new Thread[numThreads];

        for(int i = 0; i < threads.length; i++) {
            threads[i] = new IncThread(numValues, i);
        }

        for(int i = 0; i < threads.length; i++) {
            threads[i].start();
        }

        for(int i = 0; i < threads.length; i++) {
            threads[i].join();
        }

        Connection c = JDBCTest.getLocalF3Connection();
        Statement st = c.createStatement();

        /*  Comment this out when correctness is important!!!!!! */
        ResultSet rs = st.executeQuery("select sum(val), count(*) from parallel");
        while(rs.next()) {
            long sum = rs.getLong(1);

            if(sum != (numValues * numThreads)) {
                throw new TestFailedException("OH NO!  sum is " + sum  + " but expected " + (numValues * numThreads));
            }

            long count = rs.getLong(2);
            if(count != numValues) {
                throw new TestFailedException("OH NO!  count is " + count + " but expected " + numValues);
            }
        }

        rs.close();
        //*/

        F3Connection f3 = (F3Connection)c;
        DBConnectionImpl dbCon = (DBConnectionImpl)f3.getPhysicalConnection();
        //dbCon.getCatalog().getSchema("JJ").getIndex("PARALLEL_ID").getStorage().dumpContents();

        st.execute("truncate table parallel");

        st.close();
        c.close();

        System.out.println("parallel inc test w/ " + numValues + " values and " + numThreads + " threads passed");
    }

    /**
     *
     */
    public static void createTable() throws Throwable {
        Connection c = JDBCTest.getLocalF3Connection();
        Statement st = c.createStatement();
        st.execute("create table parallel (id int, val int)");
        st.execute("create unique index PARALLEL_ID on parallel (id)");
        st.close();
        c.close();
    }

    static class IncThread extends Thread {
        int threadID;
          //  all numbers in a random order
        int[] nums;

        public IncThread(int numValues, int threadID) {
            super("INC_THREAD " + threadID);
            this.threadID = threadID;
            nums = new int[numValues];
            for(int i = 0; i < nums.length; i++) {
                nums[i] = i;
            }

            randomize(nums);
        }

        public void run() {
            Connection c = null;
            try {
                c = JDBCTest.getLocalF3Connection();
                c.setAutoCommit(false);
                Statement st = c.createStatement();

                  //  for each number, increment it
                for(int i = 0; i < nums.length; i++) {
                    /*
                    ResultSet rs = st.executeQuery("select * from parallel where id=" + nums[i]);
                    int count = 0;
                    while(rs.next()) {
                        count++;
                    }
                    rs.close();

                    if(count > 1) {
                        System.out.println("We see more than 1 value!!!");
                        rs = st.executeQuery("select * from parallel where id=" + nums[i]);
                        count = 0;
                        while(rs.next()) {
                            count++;
                        }
                        rs.close();

                        if(count > 1) {
                            System.out.println("STILL!!!");
                        }
                    }
                    */

                    //  TODO:  try with inserts only, and lots of queries to verify uniqueness
                    int numEffected = 0;//st.executeUpdate("update parallel set val = val + 1 where id=" + nums[i]);
                    if(numEffected == 0) {
                        try{
                            st.executeUpdate("insert into parallel (id, val) values (" + nums[i] + ", 0)");
                        }catch(SQLException s) {
                            if(s.getErrorCode() == ErrorCode.UNIQUE_KEY_VIOLATION) {
                                //System.out.println("acceptable unique key found  threadid:" + threadID);
                            } else {
                                System.out.print("inc threadid:" + threadID + " :");
                                throw s;
                            }
                        }

                        numEffected = st.executeUpdate("update parallel set val = val + 1 where id=" + nums[i]);
                        if(numEffected != 1) {
                            throw new Exception("ERROR failed to update inc row value, got " + numEffected + " on thread id: " + threadID);
                        }
                    } else if(numEffected > 1) {
                        throw new Exception("ERROR failed first update, got " + numEffected  + " on thread id: " + threadID);
                    }

                    c.commit();
                }

                st.close();
            }catch(Throwable t) {
                t.printStackTrace();
            } finally {
                if(c != null) {
                    try{
                        c.close();
                    }catch(SQLException s) {
                        s.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * I realize this isn't as good as Knuth can do :P
     */
    public static void randomize(int[] a) {
        for(int i = 0; i < a.length; i++) {
            int j = (int) (Math.random() * (double)(a.length));

            //  swap i & j
            int tmp = a[i];
            a[i] = a[j];
            a[j] = tmp; //a[i]; //  tmp;  a bug that caused weirdness!!!
        }

       // System.out.println();
    }
}
