/**
 * Copyright 2004 JJ Furman
 * Time: 7:38:17 PM
 */
package freebase.tools;

import freebase.parser.ParseException;
import freebase.parser.SqlParser;
import freebase.sql.ColumnDefinition;
import freebase.sql.CreateTableStatement;
import freebase.sql.SqlStatement;
import freebase.util.SqlTypes;
import freebase.util.StringUtil;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;


/**
 * A tool that generates the critical CatalogDD and CatalogHelper classes with all the constants and helper methods
 * TODO:  add delete by condition (indexed columns)
 */
public class GenerateCatalogDD {
    /**
     * Fields:  map from table name to String[]
     */
    HashMap fields = new HashMap();

    ArrayList tables = new ArrayList();


    int maxNameLen = 0;

    public static void main(String[] args) throws Throwable {
        GenerateCatalogDD gen = new GenerateCatalogDD();

        gen.generateCatalogDD();
    }

    /**
     * main method  TODO:  check dates of files
     */
    public void generateCatalogDD() throws IOException, ParseException, SQLException  {
          //  read the create script
        readGenesisScript("catalog/create_system.sql");

        prepareData();

          //  write out the CatalogDD file
        writeCatalogDD("src/freebase/catalog/CatalogDD.java");
        writeCatalogHelper("src/freebase/catalog/CatalogHelper.java");
    }

    /**
     * reads the database catalog genesis script
     * @param filename
     */
    public void readGenesisScript(String filename) throws IOException, ParseException {
          //  parse the create_tables.  For now, this is hard coded as such, later use F3_HOME or whatever
        FileInputStream in = new FileInputStream(filename);
        SqlParser parser = new SqlParser(in);

        ArrayList allStatements = new ArrayList();
        ArrayList allTables = new ArrayList();
        CreateTableStatement tablesCreate = null;
        boolean done = false;
        while(! done) {
            SqlStatement s = parser.TerminatedStatement();

            if(s != null) {
                allStatements.add(s);
                if(s instanceof CreateTableStatement) {
                    CreateTableStatement c = (CreateTableStatement)s;
                    tables.add(c.getTableName());

                    if(c.getTableName().length() > maxNameLen) {
                        maxNameLen = c.getTableName().length();
                    }

                    ColumnDefinition[] columns = new ColumnDefinition[c.getNumColumns()];
                    for(int i = 0; i < c.getNumColumns(); i++) {
                        columns[i] = c.getColumn(i);
                    }

                    fields.put(c.getTableName(), columns);
                    /*
                    if(c.getTableName().equals(DataDictionary.TABLES_TABLE)) {
                        tablesCreate = c;
                    }

                    allTables.add(s);
                    */
                }
            } else {
                done = true;
            }
        }
    }

    /**
     *  make some nice whitespace
     */
    public void prepareData() {
    }

    public void writeHeader(PrintStream out) {
        out.println("/***************************************************/");
        out.println("/*    AUTO GENERATED FILE - DO NOT EDIT DIRECTLY   */");
        out.println("/***************************************************/");

        out.println("package freebase.catalog;");
        out.println();
    }


    public void writeCatalogDD(String filename) throws IOException {
        PrintStream out = new PrintStream(new FileOutputStream(filename), false);

        writeHeader(out);

        out.println("  //  generated by freebase.tools.GenerateCatalogDD during the ant build process");
        out.println("public interface CatalogDD {");

          //  write all the table statics
        for(int i = 0; i < tables.size(); i++) {
            out.print("    final public static String " + tables.get(i) + "_TABLE ");
            for(int j = ((String)tables.get(i)).length(); j < maxNameLen; j++) {
                out.print(' ');
            }

            out.println("= \"" + tables.get(i) + "\";");
        }

          //  write an array for them all
        out.println();
        out.println("      //  in an arbitrary order!  don't depend on this");
        out.println("    final public static String[] ALL_TABLES = {");
        for(int i = 0; i < tables.size(); i++) {
            out.println("        " + tables.get(i) + "_TABLE,");
        }
        out.println("    };");

        out.println();
        out.println("      //  in same arbitrary order!");
        for(int i = 0; i < tables.size(); i++) {
            out.print("    final public static int " + tables.get(i) + "_TABLE_OFFSET ");
            for(int j = getTableName(i).length(); j < maxNameLen; j++) {
                out.print(' ');
            }

            out.println("= " + i + ";");
        }
        out.println();
        out.println("    final public static int NUM_CATALOG_TABLES = " + tables.size() + ";");
        out.println();
        out.println("    ///////////////////////////////////////////////////////////////////////////");
        out.println();
        for(int i = 0; i < tables.size(); i++) {
            out.println("    public static interface " + StringUtil.makeNice(getTableName(i)) + "Columns {");
            ColumnDefinition[] columns = getColumns(i);
            for(int j = 0; j < columns.length; j++) {
                out.println("        final public static int " + columns[j].getFieldName() + "_OFFSET = " + j + ";");
            }

            out.println("    }");
            out.println();
        }


        out.println("}");
        out.close();
    }

    public void writeCatalogHelper(String filename) throws IOException, SQLException {
        PrintStream out = new PrintStream(new FileOutputStream(filename), false);

        writeHeader(out);

        out.println("import java.sql.SQLException;");
        out.println("import freebase.engine.Transaction;");
        out.println();
        out.println("/**");
        out.println(" * generated by freebase.tools.GenerateCatalogDD during the ant build process");
        out.println(" * Helps enforce compile time error checking when working on catalog");
        out.println(" */");
        out.println("public abstract class CatalogHelper implements CatalogDD {");
        out.println("    abstract protected Table getSysTable(int offset);");
        out.println();

        for(int i = 0; i < tables.size(); i++) {
            ColumnDefinition[] columns = getColumns(i);
            out.print("      //  Columns: ");
            for(int j = 0; j < columns.length; j++) {
                if(j != 0) {
                    out.print(", ");
                }
                out.print(columns[j].getFieldName());
            }
            out.println();
            out.println("    public void insertInto" + tables.get(i) + "(Transaction t, ");

            for(int j = 0; j < columns.length; j++) {
                if(j != 0) {
                    out.println(',');
                }

                out.print("                                       ");
                out.print(SqlTypes.getJavaClassForType(columns[j].getSqlType(), columns[j].getScale()));
                out.print(' ');
                out.print(columns[j].getFieldName());
            }

            out.println(") throws SQLException {");

            out.print("        getSysTable(" + tables.get(i) + "_TABLE_OFFSET).insertRow(t, new Comparable[] {");
              //  put data into an array
            for(int j = 0; j < columns.length; j++) {
                if(j != 0) {
                    out.print(',');
                }
                out.print(columns[j].getFieldName());
            }
            out.println("});");

            out.println("    }");
            out.println();
        }

        out.println("}");

        out.close();
    }


    public String getTableName(int i) {
        return (String)tables.get(i);
    }

    public ColumnDefinition[] getColumns(int i) {
        return (ColumnDefinition[])fields.get(getTableName(i));
    }
}
