/*
 * Created by IntelliJ IDEA.
 * User: user
 * Date: Mar 11, 2002
 * Time: 6:17:14 PM
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package freebase.util;

import java.io.IOException;
import java.io.Serializable;

/**
 * An immutable Boolean, just like the Boolean class but is comparable
 */
public class Bool implements Comparable, Serializable {
    public static Bool TRUE = new Bool(true);
    public static Bool FALSE = new Bool(false);

    /**
     * Should always use this function because there are only two values
     */
    public static Bool getBool(boolean value) {
        return value ? TRUE : FALSE;
    }

    /******************************
     *  true or false
     */
    boolean value;

    /**
     * Private because there are only two of these, so they are recycled over and over
     */
    private Bool(boolean value) {
        this.value = value;
    }

    /**
     * Called only when being serialized
     */
    private Bool() {}

    /**
     * Either true or false
     */
    public boolean getValue() {
        return value;
    }

    /**
     * -1 if this is less than the target (false < true)
     */
    public int compareTo(Object o) {
        if(! (o instanceof Bool)) {
            return -1;
        }

        Bool other = (Bool)o;
        if(other.getValue() == this.getValue())
            return 0;

          //  if we are true, then the other is false thus we are greater than it...
        if(getValue())
            return 1;

        return -1;

    }

    public String toString() {
        return value ? "true" : "false";
    }


    ////////////  Serializable methods

    private void writeObject(java.io.ObjectOutputStream out)
         throws IOException {
        out.writeBoolean(value);
    }

    private void readObject(java.io.ObjectInputStream in)
        throws IOException, ClassNotFoundException {
         value = in.readBoolean();
    }
}
