/**
 * Copyright 2004 JJ Furman
 * Time: 8:25:09 AM
 */
package freebase.util;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.IOException;

/**
 * This is a prototype so I'm not sure yet how this class is going to be used.
 * Possibilities
 * 1.  One way is to create one each time needed, not really that bad.
 * 2.  Another way is to have both DataInput and DataOutput be the same class and have specific instances that need it
 * subclass.
 *
 * Either way, for performance (yes!) this class assumes that the caller will not go past EOF (or end of bytes)
 */
public class ByteArrayDataInput implements DataInput {
    byte[] data;

    /**
     * the next byte to return
     */
    int pos = 0;

    public ByteArrayDataInput() {

    }

    public ByteArrayDataInput(byte[] data) {
        this.data = data;
    }

    public byte[] getBytes() {
        return data;
    }

    public void setBytes(byte[] d) {
        this.data = d;
        pos = 0;
    }

    /***********************************************************
     * DataInputStream methods
     *
     */

    public void readFully(byte b[]) throws IOException {
        readFully(b, 0, b.length);
    }

    public void readFully(byte b[], int off, int len) throws IOException {
        System.arraycopy(data, pos, b, off, len);
        pos += len;
    }

    public int skipBytes(int n) throws IOException {
        pos += n;
        return n;
    }

    /**
     * TODO:  remove dependencies on this in here
     * @return
     */
    public int read() {
        return ((int)data[pos++]) & 0xff;
    }

    public final boolean readBoolean() throws IOException {
        int ch = read();
        return (ch != 0);
    }

    public final byte readByte() throws IOException {
        int ch = read();
        return (byte) (ch);
    }

    public final int readUnsignedByte() throws IOException {
        int ch = read();
        return ch;
    }

    public final short readShort() throws IOException {
        int ch1 = read();
        int ch2 = read();
        return (short) ((ch1 << 8) + (ch2 << 0));
    }

    public final int readUnsignedShort() throws IOException {
        int ch1 = read();
        int ch2 = read();
        return (ch1 << 8) + (ch2 << 0);
    }

    public final char readChar() throws IOException {
        int ch1 = read();
        int ch2 = read();
        return (char) ((ch1 << 8) + (ch2 << 0));
    }

    public final int readInt() throws IOException {
        int ch1 = read();
        int ch2 = read();
        int ch3 = read();
        int ch4 = read();
        return ((ch1 << 24) + (ch2 << 16) + (ch3 << 8) + (ch4 << 0));
    }

    private byte readBuffer[] = new byte[8];

    public final long readLong() throws IOException {
        readFully(readBuffer, 0, 8);
        return (((long) readBuffer[0] << 56) +
                ((long) (readBuffer[1] & 255) << 48) +
                ((long) (readBuffer[2] & 255) << 40) +
                ((long) (readBuffer[3] & 255) << 32) +
                ((long) (readBuffer[4] & 255) << 24) +
                ((readBuffer[5] & 255) << 16) +
                ((readBuffer[6] & 255) << 8) +
                ((readBuffer[7] & 255) << 0));
    }

    public final float readFloat() throws IOException {
        return Float.intBitsToFloat(readInt());
    }

    public final double readDouble() throws IOException {
        return Double.longBitsToDouble(readLong());
    }

    private char lineBuffer[];

    public final String readLine() throws IOException {
        char buf[] = lineBuffer;

        if (buf == null) {
            buf = lineBuffer = new char[128];
        }

        int room = buf.length;
        int offset = 0;
        int c;

        loop:	while (true) {
            switch (c = read()) {
                case -1:
                case '\n':
                    break loop;

                case '\r':
                    int c2 = read();
                    break loop;

                default:
                    if (--room < 0) {
                        buf = new char[offset + 128];
                        room = buf.length - offset - 1;
                        System.arraycopy(lineBuffer, 0, buf, 0, offset);
                        lineBuffer = buf;
                    }
                    buf[offset++] = (char) c;
                    break;
            }
        }
        if ((c == -1) && (offset == 0)) {
            return null;
        }
        return String.copyValueOf(buf, 0, offset);
    }

    public final String readUTF() throws IOException {
        return DataInputStream.readUTF(this);
    }
}
