/**
 *  this is in util so that it can be seen from SqlTypes.  Anyway, why not?
 */
package freebase.util;

import freebase.sql.ColumnDefinition;

import java.io.Serializable;

/**
 *  really more like a struct.  These are treated as singletons in the system
 *  so don't go changing them... let the catalog create new ones for changed fields.
 *
 * They are treated as singletons because some query may be using it or
 * something I guess.  Can't remember my own comments.
 *
 *  TODO:  consider how this is redundant with ColumnDefinition
 */
public class ColumnStorageInfo implements Serializable {
    private String name;
    private int type;
    private int size;
    private int decimal;
    private boolean nullable;

    public ColumnStorageInfo(String name, int type, int size, int prec, boolean nullable) {
        this.name = name;
        this.type = type;
        this.size = size;
        this.decimal = decimal;
        this.nullable = nullable;
    }

    public ColumnStorageInfo(ColumnDefinition def) {
        this.name = def.getFieldName();
        this.type = def.getSqlType();
        this.size = def.getSize();
        this.decimal = def.getScale();
        this.nullable = def.isNullable();

        //System.out.println(name + " " + type + " " + size + " " + decimal + " " + nullable);
    }

    public String getName() {
        return name;
    }

    public int getType() {
        return type;
    }

    public int getSize() {
        return size;
    }

    public int getDecimal() {
        return decimal;
    }

    public boolean isNullable() {
        return nullable;
    }

    /**
     * Called when a new row is being inserted.
     * TODO:  although this is not implemented yet, it is here as a placeholder when it does get going!
     */
    public Comparable getDefault() {
        return null;
    }
}