package freebase.util;

/**
 *  converts data to and from byte arrays.
 *
 *  This class is one of the primary reasons java is slower than
 *  C for building a database.  In c you just cast pointers and you are done.
 */
public class DataConverter {
    
    public static short readShort(byte[] data, int pos) {
        int c1 = data[pos] & 0xff;
	    int c2 = data[pos + 1] & 0xff;
	    return (short)((c1 << 8) + (c2 << 0));
    }
    
    public static void writeShort(byte[] data, int pos, short v) {
        data[pos]   = (byte)((v >>>  8) & 0xFF);
	    data[pos+1] = (byte)((v >>>  0) & 0xFF);
    }
    
    public static int readInt(byte[] data, int pos) {
        int c1 = data[pos] & 0xff;
	    int c2 = data[pos + 1] & 0xff;
	    int c3 = data[pos + 2] & 0xff;
	    int c4 = data[pos + 3] & 0xff;
	    return ((c1 << 24) + (c2 << 16) + (c3 << 8) + (c4 << 0));
    }
    

    
    public static void writeInt(byte[] data, int pos, int v) {
        data[pos]   = (byte)((v >>> 24) & 0xFF);
	    data[pos+1] = (byte)((v >>> 16) & 0xFF);
	    data[pos+2] = (byte)((v >>>  8) & 0xFF);
	    data[pos+3] = (byte)((v >>>  0) & 0xFF);
    }
    
    public static long readLong(byte[] data, int pos) {
        return ((long)(readInt(data, pos)) << 32) + (readInt(data, pos + 4) & 0xFFFFFFFFL);
    }
    
    public static void writeLong(byte[] data, int pos, long v) {
        data[pos]   = (byte)((v >>> 56) & 0xFF);
	    data[pos+1] = (byte)((v >>> 48) & 0xFF);
	    data[pos+2] = (byte)((v >>> 40) & 0xFF);
	    data[pos+3] = (byte)((v >>> 32) & 0xFF);
	    data[pos+4] = (byte)((v >>> 24) & 0xFF);
	    data[pos+5] = (byte)((v >>> 16) & 0xFF);
	    data[pos+6] = (byte)((v >>>  8) & 0xFF);
	    data[pos+7] = (byte)((v >>>  0) & 0xFF);
    }
    
    public static String readShortString(byte[] data, int pos) {
        short len = readShort(data, pos);
        return new String(data, pos, len);
    }
    
    //  writes a string that can have a max length of 32k
    public static void writeShortString(byte[] data, int pos, String v) {
        if(v.length() > Short.MAX_VALUE)
            throw new RuntimeException("String longer than max short");
        
        writeShort(data, pos, (short)v.length());
        System.arraycopy(v.getBytes(), 0, data, 2 + pos, v.length());
    }
}