/**
 * Copyright 2004 JJ Furman
 * Time: 4:49:03 PM
 */
package freebase.util;

/**
 * Global properties for the database.  These are used for properties that are accessed frequently, not
 * cached in local places and generally don't change while the db is running.  To fetch a global property it should
 * not be required to lookup in a hashtable.
 * <P>
 * No property should be used before Props has been appropriately loaded and set.
 */
public class GlobalProps {
    final public static String STRING_DATA_CASE_INSENSITIVE = "STRING_DATA_CASE_INSENSITIVE";
    final public static String MAX_CURSORS = "MAX_CURSORS";

    /**
     * On a per table or index basis, the maximum # of outstanding cursors allowed.  This
     * is to prevent crazy runnaway.
     */
    private static int maxCursors = 500;

    /**
     *
     */
    private static boolean stringsCaseInsensitive = true;


    static {
        if(Props.containsKey(STRING_DATA_CASE_INSENSITIVE)) {
            stringsCaseInsensitive = Props.getBoolean(STRING_DATA_CASE_INSENSITIVE);
        }

        if(Props.containsKey(MAX_CURSORS)) {
            maxCursors = Props.getInteger(MAX_CURSORS);
        }
    }


    /**********************************************************************
     *
     */

    public static int getMaxCursors() {
        return maxCursors;
    }

    public static boolean isStringsCaseInsensitive() {
        return stringsCaseInsensitive;
    }
}
