/*
 * Copyright JJ Furman
 * User: user
 * Date: Apr 3, 2002
 * Time: 9:03:14 AM
 */
package freebase.util;

import java.io.PrintStream;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;


/**
 * Basic utility methods with no other place to go
 */
public class Helper {
    static NumberFormat twoDigits = new DecimalFormat("00");
    static NumberFormat threeDigits = new DecimalFormat("000");

    /**
     * Deflects any interrupt by setting the interrupt state.  This means
     * that the next operation to wait or sleep will get the interrupt.
     */
    public static void sleep(long millis) {
        try{
            Thread.sleep(millis);
        }catch(InterruptedException i) {
            System.out.println("Interrupted during sleep");
            Thread.currentThread().interrupt();  //  reinterrupt for future reference
            i.printStackTrace();
        }
    }


    /**
     * Helper method to compare two array lists
     */
    public static boolean areListsEqual(ArrayList a, ArrayList b) {
        if(a == null)
            return b == null;

        if(b == null)
            return a == null;  //  ie, false else true would have already been returned, but who cares

        if(a.size() != b.size())
            return false;

        for(int i = 0; i < a.size(); i++) {
            if(! a.get(i).equals(b.get(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * In this case, null == null
     */
    public static boolean areEqual(Object a, Object b) {
        if(a == null) {
            return b == null;
        }

        return a.equals(b);
    }

    public static void printElapsed(PrintStream out, long time) {
        long millis = time % 1000;
        time = time / 1000;
        long secs = time % 60;
        time = time / 60;
        long mins = time % 60;
        time = time / 60;
        long hours = time;

        out.println("Elapsed: " + twoDigits.format(hours) + ':' + twoDigits.format(mins) + ':' +
                twoDigits.format(secs) + '.' + threeDigits.format(millis));
    }

    /**
     * Prints the time in the format:  'Elapsed: HH:MM:SS:mil'
     */
    public static void printElapsed(long time) {
        printElapsed(System.out, time);
    }
}
