/**
 * Copyright 2004 JJ Furman
 * Time: 8:35:06 PM
 */
package freebase.util;

import java.util.TreeMap;

/**
 * Holds a list of unique values.  As they are removed, it will tell the caller the lowest remaining
 * value.  It is used to keep track of log positions for both active transactions and lowest LSN's of
 * transactions who may still be needed to construct a historical view (soft LSN)<P>
 *
 * Currently this does not deal with it's own concurrency so lock it up!
 *
 * TODO:  look into heap data structure
 * TODO:  look into something that doesn't require objects!
 */
public class LogStop {
    TreeMap logStoppers = new TreeMap();
    long minLogStopper;

    /**
     * add a stop at this position
     * @param stopPos
     */
    public void addStop(long stopPos) {
        Long posObj = new Long(stopPos);
        if(logStoppers.size() == 0) {
            minLogStopper = stopPos;
        }

        logStoppers.put(posObj, this);
    }

    /**
     * remove the stop associated with the position
     * @return true if the min has now changed
     */
    public boolean removeStop(long stopPos) {
        Long posObj = new Long(stopPos);
        logStoppers.remove(posObj);
        if(stopPos == minLogStopper) {
            if(logStoppers.size() == 0) {
                minLogStopper = 0;
                return true;
            } else {
                posObj = (Long)logStoppers.firstKey();
                minLogStopper = posObj.longValue();
                return true;
            }
        }

        return false;
    }

    /**
     * obtain the current minimum.
     * @return 0 if there is no minimum
     */
    public long getMinStop() {
        return minLogStopper;
    }
}
