package freebase.util;


import java.io.IOException;
import java.util.Enumeration;
import java.util.NoSuchElementException;

//  TODO:  eventually this will need to be rewritten :)


/**
 * Note: this class is unsynchronized!
 *
 * TODO:  when rewriting this consider an implementation that creates no new objects in the normal case.
 * Might slow down the code a little bit because you would need to check to see if the object at the location
 * is an entry node or a real val.
 *
 * 
 *
 * long[] ids
 * Object[] data
 * boolean[] isData;  //  optional
 * @author JJ Furman
 */
public class LongHashMap implements Cloneable, java.io.Serializable {
    private transient LongHashMapEntry table[];
    private transient int count;
    private int thresh;
    private float loadFactor;

    public LongHashMap(int initialCapacity, float loadFactor) {
        if ((initialCapacity <= 0) || (loadFactor <= 0.0)) {
            throw new IllegalArgumentException();
        }
        this.loadFactor = loadFactor;
        table = new LongHashMapEntry[initialCapacity];
        thresh = (int) (initialCapacity * loadFactor);
    }

    public LongHashMap(int initialCapacity) {
        this(initialCapacity, 0.75f);
    }

    public LongHashMap() {
        this(101);
    }

    public int size() {
        return count;
    }

    public boolean isEmpty() {
        return count == 0;
    }

    public Enumeration keys() {
        return new LongHashMapEnumerator(table, true);
    }


    public Enumeration elements() {
        return new LongHashMapEnumerator(table, false);
    }

    public boolean contains(Object value) {
        if (value == null) {
            throw new NullPointerException();
        }

        LongHashMapEntry tab[] = table;
        for (int i = tab.length; i-- > 0;) {
            for (LongHashMapEntry e = tab[i]; e != null; e = e.next) {
                if (e.val.equals(value)) {
                    return true;
                }
            }
        }
        return false;
    }


    public boolean containsKey(long key) {
        LongHashMapEntry tab[] = table;
        int hash = (int) (key ^ (key >> 32));
        int index = (hash & 0x7FFFFFFF) % tab.length;
        for (LongHashMapEntry e = tab[index]; e != null; e = e.next) {
            if ((e.hash == hash) && e.key == key) {
                return true;
            }
        }
        return false;
    }


    public Object get(long key) {
        LongHashMapEntry tab[] = table;
        int hash = (int) (key ^ (key >> 32));
        int index = (hash & 0x7FFFFFFF) % tab.length;
        for (LongHashMapEntry e = tab[index]; e != null; e = e.next) {
            if ((e.hash == hash) && e.key == key) {
                return e.val;
            }
        }
        return null;
    }

    /**
     * Rehashes the table.
     */
    protected void rehash() {
        int oldCapacity = table.length;
        LongHashMapEntry oldTable[] = table;

          //  TODO:  this could be more prime :)
        int newCapacity = oldCapacity * 2 + 1;
        LongHashMapEntry newTable[] = new LongHashMapEntry[newCapacity];

        thresh = (int) (newCapacity * loadFactor);
        table = newTable;

        for (int i = oldCapacity; i-- > 0;) {
            for (LongHashMapEntry old = oldTable[i]; old != null;) {
                LongHashMapEntry e = old;
                old = old.next;

                int index = (e.hash & 0x7FFFFFFF) % newCapacity;
                e.next = newTable[index];
                newTable[index] = e;
            }
        }
    }


    public Object put(long key, Object value) {
          // Make sure the val is not null.  3/10/2004: Who Cares?  Commented out
        if (value == null) {
            //throw new NullPointerException();
        }

          //  Makes sure the key is not already in the hashtable.
        LongHashMapEntry tab[] = table;
        int hash = (int) (key ^ (key >> 32));
        int index = (hash & 0x7FFFFFFF) % tab.length;
        for (LongHashMapEntry e = tab[index]; e != null; e = e.next) {
            if ((e.hash == hash) && e.key == key) {
                Object old = e.val;
                e.val = value;
                return old;
            }
        }

        if (count >= thresh) {
            // Rehash the table if the thresh is exceeded
            rehash();
            return put(key, value);
        }

        // Creates the new entry.
        LongHashMapEntry e = new LongHashMapEntry();
        e.hash = hash;
        e.key = key;
        e.val = value;
        e.next = tab[index];
        tab[index] = e;
        count++;
        return null;
    }

    public Object remove(long key) {
        LongHashMapEntry tab[] = table;
        int hash = (int) (key ^ (key >> 32));
        int index = (hash & 0x7FFFFFFF) % tab.length;
        for (LongHashMapEntry e = tab[index], prev = null; e != null; prev = e, e = e.next) {
            if ((e.hash == hash) && e.key == key) {
                if (prev != null) {
                    prev.next = e.next;
                } else {
                    tab[index] = e.next;
                }
                count--;
                return e.val;
            }
        }
        return null;
    }


    public void clear() {
        LongHashMapEntry tab[] = table;
        for (int index = tab.length; --index >= 0;)
            tab[index] = null;
        count = 0;
    }

    public Object clone() {
        try {
            LongHashMap t = (LongHashMap) super.clone();
            t.table = new LongHashMapEntry[table.length];
            for (int i = table.length; i-- > 0;) {
                t.table[i] = (table[i] != null) ? (LongHashMapEntry) table[i].clone() : null;
            }
            return t;
        } catch (CloneNotSupportedException e) {
            // this shouldn't happen, since we are Cloneable
            throw new InternalError();
        }
    }


    public String toString() {
        int max = size() - 1;
        StringBuffer buf = new StringBuffer();
        Enumeration k = keys();
        Enumeration e = elements();
        buf.append("{");

        for (int i = 0; i <= max; i++) {
            String s1 = k.nextElement().toString();
            String s2 = e.nextElement().toString();
            buf.append(s1 + "=" + s2);
            if (i < max) {
                buf.append(", ");
            }
        }
        buf.append("}");
        return buf.toString();
    }


    private void writeObject(java.io.ObjectOutputStream s)
            throws IOException {
        // Write out the length, thresh, loadfactor
        s.defaultWriteObject();

        // Write out length, count of elements and then the key/val objects
        s.writeInt(table.length);
        s.writeInt(count);
        for (int index = table.length - 1; index >= 0; index--) {
            LongHashMapEntry entry = table[index];

            while (entry != null) {
                s.writeLong(entry.key);
                s.writeObject(entry.val);
                entry = entry.next;
            }
        }
    }


    private void readObject(java.io.ObjectInputStream s)
            throws IOException, ClassNotFoundException {
        // Read in the length, thresh, and loadfactor
        s.defaultReadObject();

        // Read the original length of the array and number of elements
        int origlength = s.readInt();
        int elements = s.readInt();

        // Compute new size with a bit of room 5% to grow but
        // No larger than the original size.  Make the length
        // odd if it's large enough, this helps distribute the entries.
        // Guard against the length ending up zero, that's not valid.
        int length = (int) (elements * loadFactor) + (elements / 20) + 3;
        if (length > elements && (length & 1) == 0)
            length--;
        if (origlength > 0 && length > origlength)
            length = origlength;

        table = new LongHashMapEntry[length];
        count = 0;

        // Read the number of elements and then all the key/val objects
        for (; elements > 0; elements--) {
            long key = s.readLong();
            Object value = s.readObject();
            put(key, value);
        }
    }
}


class LongHashMapEnumerator implements Enumeration {
    boolean keys;
    int index;
    LongHashMapEntry table[];
    LongHashMapEntry entry;

    LongHashMapEnumerator(LongHashMapEntry table[], boolean keys) {
        this.table = table;
        this.keys = keys;
        this.index = table.length;
    }

    public boolean hasMoreElements() {
        if (entry != null) {
            return true;
        }
        while (index-- > 0) {
            if ((entry = table[index]) != null) {
                return true;
            }
        }
        return false;
    }

    public Object nextElement() {
        if (entry == null) {
            while ((index-- > 0) && ((entry = table[index]) == null)) ;
        }
        if (entry != null) {
            LongHashMapEntry e = entry;
            entry = e.next;
            return keys ? new Long(e.key) : e.val;
        }
        throw new NoSuchElementException("LongHashMapEnumerator");
    }
}

/**
 * Hashtable collision list.
 */
class LongHashMapEntry {
    int hash;  //  possibly redundant, lets save on space a bit here
    long key;
    Object val;
    LongHashMapEntry next;

    protected Object clone() {
        LongHashMapEntry entry = new LongHashMapEntry();
        entry.hash = hash;
        entry.key = key;
        entry.val = val;
        entry.next = (next != null) ? (LongHashMapEntry) next.clone() : null;
        return entry;
    }
}


/**
 * IntHashMap is (will be) an optimized HaspMap that only uses
 * ints (32 bit) for the key val.  This should eventually save
 * space and time over using Integer types as keys.
 * Initially I will not make a
 * rigorous attempt at supplying all functionality but feel free to
 * add whatever methods you like.  As well, the initial implementation
 * will only be a simple wrapping of HashMap:  in the future we should
 * optimize this by implementing our own fanciness.
 *
 * @author JJ Furman
 *

public class IntHashMap {
 Hashtable hashmap = new Hashtable();

 public Object get(int key) {
 return hashmap.get(new Integer(key));
 }

 public Object put(int key, Object val) {
 return hashmap.put(new Integer(key), val);
 }

 public Object remove(int key) {
 return hashmap.remove(new Integer(key));
 }

 public Enumeration keys() {
 return hashmap.keys();
 }

 public int size() {
 return hashmap.size();
 }
 }

//  */