package freebase.util;

/**
 * Copyright 2004 JJ Furman
 * Time: 4:22:28 PM
 */
public class LongList {
/**
 * A generally useful long list class.  too bad no templates
 */

    long[] data;
    int len;

    public LongList(int initialCap) {
        data = new long[initialCap];
    }

    public LongList() {
        this(10);
    }

    public int size() {
        return len;
    }

    final public long get(int pos) {
        if(pos >= len)
            throw new ArrayIndexOutOfBoundsException("List is only " + len + " long but you requested " + pos);

        return data[pos];
    }

    public void add(long value) {
        if(len == data.length) {
            long[] newData = new long[data.length * 2];
            System.arraycopy(data, 0, newData, 0, data.length);
            data = newData;
        }

        data[len] = value;
        len++;
    }

    public void set(int pos, long val) {
        if(pos >= len)
            throw new ArrayIndexOutOfBoundsException("List is only " + len + " long but you requested " + pos);

        data[pos] = val;
    }

    public void remove(int pos) {
        remove(pos, pos);
    }

    /**
     * removes [from,to] inclusive
     * @param from
     * @param to to must be >= from
     */
    public void remove(int from, int to) {
        System.arraycopy(data, to + 1, data, from, data.length - to - 1);
        len -= (to - from + 1);
    }

    /**
     * Insert this val into the ordered list of values.  Return the position that was inserted.
     */
    public int insertOrdered(long val) {
        int pos = findSpot(val);
        insert(pos, val);
        return pos;
    }

    /**
     * Find the spot such that this spot is >= val, but before is it less.
     * Only works on an ordered list
     * @param val
     * @return
     */
    public int findSpot(long val) {
        /*
        int pos = 0;
        while(pos  < len && data[pos] <= val) {
            pos ++;
        }

        return pos;
        */
        return findSpot(val, 0, len);
    }

    /**
     * From the ordered list, find the index position of the val such that the given val is above val
     * but the next one down is below.  IE, this is the insert spot.  lo and hi are the bounds.
     * @param val
     * @return
     */
    private int findSpot(long val, int lo, int hi) {
        if(lo == hi) {
            return lo;
        }

        int pivot = (lo + hi) / 2;
        if(data[pivot] == val) {
            return pivot;
        } else if(data[pivot] > val) {
            return findSpot(val, lo, pivot);
        } else {
            return findSpot(val, pivot + 1, hi);
        }
    }

    public void insert(int pos, long value) {
        if(len == data.length) {
            long[] newData = new long[data.length * 2];
            if(pos != 0) {
                System.arraycopy(data, 0, newData, 0, pos);
            }

            if(pos != len) {
                System.arraycopy(data, pos, newData, pos + 1, len - pos);
            }
            data = newData;
        } else {
            if(pos != len) {
                System.arraycopy(data, pos, data, pos + 1, len - pos);
            }
        }

        data[pos] = value;
        len++;
    }

    /**
     * append the data to the end of this list.
     */
    public void append(LongList d) {
        if(data.length < (len + d.len)) {
            long[] newData = new long[len + d.len];
            System.arraycopy(data, 0, newData, 0, data.length);
            data = newData;
        }

        System.arraycopy(d.data, 0, data, len, d.len);
        len += d.len;
    }

    public void append(long val) {
        insert(len, val);
    }

    public long[] toArray() {
        long[] out = new long[size()];
        System.arraycopy(data, 0, out, 0, size());
        return out;
    }

    /**
     * return the maximum val in the entire list.
     */
    public float max() {
        float max = Float.MIN_VALUE;
        for(int i = 0; i < len; i++) {
            if(data[i] > max)
                max = data[i];
        }

        return max;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer("FloatList: (");
        for(int i = 0; i < size(); i++) {
            if(i != 0) {
                sb.append(",");
            }
            sb.append(data[i]);
        }
        sb.append(")");
        return sb.toString();
    }

    public static void main(String[] args) {
        LongList a = new LongList();
        a.insert(3, 0);
        a.insert(1, 0);
        a.insert(2, 1);
        a.append(4);

        /*
        System.out.println("find 1.5   " + a.findSpot(1.5f, 0, a.size() ));
        System.out.println("find .5   " + a.findSpot(.5f, 0, a.size()));
        System.out.println("find 2.5   " + a.findSpot(2.5f, 0, a.size() ));
        System.out.println("find 2   " + a.findSpot(2f, 0, a.size()));
        System.out.println("find 100   " + a.findSpot(100f, 0, a.size()));
        */

        int removePos = a.findSpot(2);
        a.remove(0, removePos - 1);

        System.out.println(a);
    }
}
