/*
 * Copyright JJ Furman
 * User: user
 * Date: May 10, 2002
 * Time: 12:08:32 PM
 */
package freebase.util;

import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * Number helper functions
 */
public class NumberUtil {
    /**
     * 20 as a BigInteger (oh boy)
     */
    final public static BigInteger BIG_INTEGER_TWENTY = BigInteger.valueOf(20);

    /**
     * What our current precision is.
     */
    final public static int INTERNAL_PRECISION = 40;

    /**
     * Returns a/b
     */
    public static BigDecimal divide(BigDecimal a, BigDecimal b) {
        return a.divide(b, INTERNAL_PRECISION, BigDecimal.ROUND_HALF_EVEN);
    }


    /**
     * Java does not allow easy comparison of values of different number classes
     * TODO:  study performance and real cases of this to see if this could be made faster
     *     if Long or Double or whatever is common
     */
    public static int compareNumbers(Number a, Number b) {
        BigDecimal A = convertToBigDecimal(a);
        BigDecimal B = convertToBigDecimal(b);

        return A.compareTo(B);
    }

    /**
     * Converts the Number to BigDecimal if needed.
     */
    public static BigDecimal convertToBigDecimal(Number number) {
        if(number instanceof BigDecimal)
            return (BigDecimal)number;
        else if(number instanceof Double || number instanceof Float)
            return new BigDecimal(number.doubleValue());
        else if(number instanceof BigInteger) {
            return new BigDecimal((BigInteger)number);
        } else {
            return BigDecimal.valueOf(number.longValue());
        }
    }
}
