package freebase.util;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Properties encapsulation.
 * Copyright Mar 2, 2004 JJ Furman
 * Time: 12:16:11 PM
 */
public class Props {
    static Props singleton;

    Properties daProps;

    /**
     *
     * @param defaultsFileName must not be null
     * @param instanceFileName if null, will load defaults only
     * @throws IOException
     */
    public Props(String defaultsFileName, String instanceFileName) throws IOException {
        this(new BufferedInputStream(new FileInputStream(defaultsFileName)),
             (instanceFileName == null) ? null : new BufferedInputStream(new FileInputStream(instanceFileName)));
    }

    public Props(String filename) throws IOException {
        this(filename, null);
    }

    public Props(InputStream defaultsFile, InputStream instanceFile) throws IOException {
        Properties defaults = new Properties();
        defaults.load(defaultsFile);

        daProps = new Properties(defaults);
        if(instanceFile != null) {
            daProps.load(instanceFile);
        }
    }

    /******************************************************************
     * Currently these static accessor methods are it.  Later it would be fine
     * to pass in the properties into each component.  I don't want to hear about
     * bad design:  there is currently only one property file per database instance and
     * only one instance per VM.  SO THERE!
     */
    public static void setInstance(Props it) {
        singleton = it;
    }

    public static Props getInstance() {
        return singleton;
    }

    /******************************************************************
     * static accessor methods
     */
    public static boolean containsKey(String key) {
        String val = singleton.daProps.getProperty(key);
        return val != null;
    }

    public static String get(String key) {
        return singleton.daProps.getProperty(key);
    }

    public static int getInteger(String key) {
        return Integer.parseInt(get(key));
    }

    public static long getLong(String key) {
        return Long.parseLong(get(key));
    }

    public static boolean getBoolean(String key) {
        return Boolean.valueOf(get(key)).booleanValue();
    }

    public static Class getClass(String key) throws ClassNotFoundException {
        return Class.forName(get(key));
    }
}
