package freebase.util;

/**
 *  works with SqlType to present.  Needs to eventually be abstract with some specifics
 * filled in the
 *  TODO:  SqlValues are expected to be recycled in a pool at some point
 *
 * Technically this should be an interface, with multiple different implementations.
 * For now, the implementation is with an Object inside
 *
 * Number precision is going to be very imporant.
 *
 * NOTE:  this is being mothballed on 3/15/2002.  The concept lives on, but if it is ever
 * used it should be done transparently to the application, or at least there should be a mini
 * little hierarchy of objects corresponding to the java objects and representing the
 * various capabilties.  In fact, we may move to a system where whole Tuples are stored in essentially
 * one place, and other little objects like this one flutter around.  This will be a big refactoring
 * effort for sure, if for no other reason that Immutable objects can be passed around in a more
 * promiscuous way than mutable pooled objects.  But whatever needs to be done will need careful
 * consideration, far more than is warrented at this time.
 */
public class SqlValue {
    /**
     *   sql type
     */
    int type;

    /**
     * Value
     */
    Comparable value;

    //  some val fetching methods.
    //  need some recycling and setting methods

    public SqlValue(int type, Comparable value) {
        this.type = type;
        this.value = value;
    }

    public boolean equals(Object other) {
        if(other == null || ! (other instanceof SqlValue)) {
            return false;
        }

        Object otherValue = ((SqlValue)other).value;

          //  are pointers exactly equal?
        if(value == otherValue)
            return true;

          //  if not, but actualValue is null, then the other is not equal
        if(value == null)
            return false;

          //  if they are after all equal
        if(value.equals(otherValue)) {
            return true;
        }

        return false;
    }

    public int compareTo(SqlValue other) {
        return -1;
    }

    public void dispose() {
        //  should know how to go back to the pool or something
    }


    public boolean isNull() {
        return value == null;
    }

    public String toString() {
        if(isNull())
            return "null";

        //if(val instanceof String) {
        //    return SqlTypes.sqlize((String)val);
        //}

        return value.toString();
    }

    public Object clone() {
        return new SqlValue(type, value);
    }

    /**
     * TODO:  for sure this method will be gotten rid of
     */
    public Comparable getValue() {
        return (Comparable)value;
    }
}